@@ -1,6 +1,4 @@
-## $Id: kildclient.pl 1152 2011-03-20 15:31:10Z ekalin $
-
-### Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+### Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
 ###
 ### This program is free software; you can redistribute it and/or modify
 ### it under the terms of the GNU General Public License as published by
@@ -30,6 +28,8 @@
 use strict;
 
 use Locale::gettext;
+use JSON;
+use MSDP;
 
 our $__helpfile;
 our @PLUGINDIRS;
@@ -710,6 +710,106 @@ sub _ { return gettext($_[0]); }
   }
 
 
+  sub silentenatrigger {
+    my $self = shift;
+
+    my $silent = $self->{SILENT};
+    $self->{SILENT} = 1;
+    $self->enatrigger(@_);
+    $self->{SILENT} = $silent;
+  }
+
+
+  sub silentdistrigger {
+    my $self = shift;
+
+    my $silent = $self->{SILENT};
+    $self->{SILENT} = 1;
+    $self->distrigger(@_);
+    $self->{SILENT} = $silent;
+  }
+
+
+  sub silentenaalias {
+    my $self = shift;
+
+    my $silent = $self->{SILENT};
+    $self->{SILENT} = 1;
+    $self->enaalias(@_);
+    $self->{SILENT} = $silent;
+  }
+
+
+  sub silentdisalias {
+    my $self = shift;
+
+    my $silent = $self->{SILENT};
+    $self->{SILENT} = 1;
+    $self->disalias(@_);
+    $self->{SILENT} = $silent;
+  }
+
+
+  sub silentenamacro {
+    my $self = shift;
+
+    my $silent = $self->{SILENT};
+    $self->{SILENT} = 1;
+    $self->enamacro(@_);
+    $self->{SILENT} = $silent;
+  }
+
+
+  sub silentdismacro {
+    my $self = shift;
+
+    my $silent = $self->{SILENT};
+    $self->{SILENT} = 1;
+    $self->dismacro(@_);
+    $self->{SILENT} = $silent;
+  }
+
+
+  sub silentenatimer {
+    my $self = shift;
+
+    my $silent = $self->{SILENT};
+    $self->{SILENT} = 1;
+    $self->enatimer(@_);
+    $self->{SILENT} = $silent;
+  }
+
+
+  sub silentdistimer {
+    my $self = shift;
+
+    my $silent = $self->{SILENT};
+    $self->{SILENT} = 1;
+    $self->distimer(@_);
+    $self->{SILENT} = $silent;
+  }
+
+
+  sub silentenahook {
+    my $self = shift;
+
+    my $silent = $self->{SILENT};
+    $self->{SILENT} = 1;
+    $self->enahook(@_);
+    $self->{SILENT} = $silent;
+  }
+
+
+  sub silentdishook {
+    my $self = shift;
+
+    my $silent = $self->{SILENT};
+    $self->{SILENT} = 1;
+    $self->dishook(@_);
+    $self->{SILENT} = $silent;
+  }
+
+
   sub triggerenabled {
     my $self    = shift;
     my $trigger = shift;
@@ -1035,6 +1135,35 @@ sub _ { return gettext($_[0]); }
 
     $self->{SILENT} = $prevstate;
   }
+
+
+  sub sendserverdata {
+    my $self = shift;
+    my ($data) = @_;
+
+    my $protocol = $data->{'protocol'};
+    my $converted;
+    if ($protocol eq "GMCP") {
+      $converted = ::_preparegmcpdata($data);
+    } elsif ($protocol eq "MSDP") {
+      $converted = ::_preparemsdpdata($data);
+    } else {
+      $self->echonl(_('Unknown protocol'));
+      return;
+    }
+
+   $self->_sendserverdata($protocol, $converted);
+  }
+
+
+  sub sendmsdpcommand {
+    my $self = shift;
+
+    my $msdp = MSDP->new();
+    my $converted = $msdp->encode_command(@_);
+
+   $self->_sendserverdata('MSDP', $converted);
+  }
 }
 
 
@@ -1169,6 +1298,97 @@ sub help {
 }
 
 
+sub _parseserverdata {
+  if (scalar @_ < 1) {
+    $::world->echonl(sprintf(_('%s: This is an internal function, do not call directly.'), '_parseserverdata'));
+    return;
+  }
+
+  # Clear previous data
+  %::server_data = ();
+  $::server_data{'protocol'} = $_[0];
+
+  if ($::server_data{'protocol'} eq "GMCP") {
+    _parsegmcpserverdata();
+  } elsif ($::server_data{'protocol'} eq "MSDP") {
+    _parsemsdpserverdata();
+  } else {
+    $::server_data{'error'} = "Unknown protocol";
+    $::server_data{'data'} = $::_serverdata;
+  }
+}
+
+
+sub _parsegmcpserverdata {
+  my ($namespace, $data) = split(/\s+/, $::_serverdata, 2);
+  my ($p1, $p2, $p3) = split(/\./, $namespace, 3);
+
+  if (defined $p3) {
+    $::server_data{'package'}    = $p1;
+    $::server_data{'subpackage'} = $p2;
+    $::server_data{'message'}    = $p3;
+  } else {
+    $::server_data{'package'}    = $p1;
+    $::server_data{'message'}    = $p2;
+  }
+
+  if (defined $data) {
+    eval {
+      my $json = JSON->new->allow_nonref;
+
+      $::server_data{'data'} = $json->decode($data);
+      1;
+    } or do {
+      $::server_data{'error'} = $@;
+      $::server_data{'data'} = $::_serverdata;
+    }
+  }
+}
+
+
+sub _parsemsdpserverdata {
+  eval {
+    my $msdp = MSDP->new();
+    $::server_data{'data'} = $msdp->decode($::_serverdata);
+    1;
+  } or do {
+    $::server_data{'error'} = $@;
+    $::server_data{'data'} = $::_serverdata;
+  }
+}
+
+
+sub _preparegmcpdata {
+  my ($data) = @_;
+
+  my $converted = $data->{'package'};
+  if (defined $data->{'subpackage'}) {
+    $converted .= ".";
+    $converted .= $data->{'subpackage'};
+  }
+  if (defined $data->{'message'}) {
+    $converted .= ".";
+    $converted .= $data->{'message'};
+  }
+
+  if (exists $data->{'data'}) {
+    my $json = JSON->new->allow_nonref;
+    $converted .= " ";
+    $converted .= $json->encode($data->{'data'});
+  }
+
+  return $converted;
+}
+
+
+sub _preparemsdpdata {
+  my ($data) = @_;
+
+  my $msdp = MSDP->new();
+  return $msdp->encode($data->{'data'});
+}
+
+
 ### Deprecated functions
 sub getwindowsize {
   $::world->echonl(_('Warning: the getwindowsize() function is deprecated and will be removed.'));
