@@ -1,7 +1,5 @@
-/* $Id: aliases.c 1176 2011-05-28 14:43:45Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +21,7 @@
 #  include <kcconfig.h>
 #endif
 
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <locale.h>
 #include <gtk/gtk.h>
 
@@ -55,12 +53,13 @@ substitute_aliases(World *world, const char *cmd)
     return NULL;
   }
 
+  GRAB_PERL(world);
+
   aliasptr = world->aliases;
   i = 0;
   while (aliasptr) {
     alias = (Alias *) aliasptr->data;
-    if (alias->enabled && (result = perl_substitute(world->perl_interpreter,
-                                                    cmd,
+    if (alias->enabled && (result = perl_substitute(cmd,
                                                     alias->pattern_re,
                                                     alias->substitution,
                                                     alias->perl_eval,
@@ -83,12 +82,15 @@ substitute_aliases(World *world, const char *cmd)
                 result);
       }
 
+      RELEASE_PERL(world);
       return result;
     }
     aliasptr = aliasptr->next;
     ++i;
   }
 
+  RELEASE_PERL(world);
+
   return NULL;
 }
 
@@ -100,7 +102,7 @@ remove_alias(World *world, GSList *aliasitem)
 
   /* Necessary when free_alias dereferences alias->pattern_re */
   if (world->perl_interpreter) {
-    PERL_SET_CONTEXT(world->perl_interpreter);
+    GRAB_PERL(world);
   }
 
   we_alias_delete_alias(world, alias);
@@ -111,6 +113,10 @@ remove_alias(World *world, GSList *aliasitem)
   }
   free_alias(alias, NULL);
   g_slist_free(aliasitem);
+
+  if (world->perl_interpreter) {
+    RELEASE_PERL(world);
+  }
 }
 
 
@@ -210,18 +216,20 @@ alias_precompute_res(World *world)
   GSList *aliasptr = world->aliases;
   Alias  *alias;
 
+  GRAB_PERL(world);
+
   while (aliasptr) {
     alias = (Alias *) aliasptr->data;
 
     /* Aliases loaded from a script file already have the precomputed RE */
     if (!alias->pattern_re) {
-      alias->pattern_re = precompute_re(world->perl_interpreter,
-                                        alias->pattern,
-                                        alias->ignore_case);
+      alias->pattern_re = precompute_re(alias->pattern, alias->ignore_case);
     }
 
     aliasptr = aliasptr->next;
   }
+
+  RELEASE_PERL(world);
 }
 
 
