@@ -1,7 +1,5 @@
-/* $Id: cmdhistory.c 1143 2011-02-13 16:13:58Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +22,7 @@
 #endif
 
 #include <string.h>
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <locale.h>
 #include <gtk/gtk.h>
 #include <gmodule.h>
@@ -54,7 +52,8 @@ static GtkWidget *create_cmd_history_dialog(GtkWindow *parent, World *world);
 static gboolean   cmdhistory_send_timer_cb(gpointer data);
 static void       save_tentative_command(World *world);
 /* XML UI signals */
-G_MODULE_EXPORT void menu_command_history_cb(GtkWidget *widget, gpointer data);
+G_MODULE_EXPORT void menu_command_history_cb(GtkMenuItem *widget,
+                                             gpointer     data);
 G_MODULE_EXPORT void cmdhistory_recall_cb(GtkWidget *widget, gpointer data);
 G_MODULE_EXPORT void cmdhistory_start_send_cb(GtkWidget *widget, gpointer data);
 G_MODULE_EXPORT void cmdhistory_find_cb(GtkWidget *widget, gpointer data);
@@ -163,8 +162,9 @@ find_prev_or_next_command(World *world, gboolean next)
   }
 }
 
+
 void
-menu_command_history_cb(GtkWidget *widget, gpointer data)
+menu_command_history_cb(GtkMenuItem *widget, gpointer data)
 {
   if (!currentWorld) {
     return;
@@ -384,7 +384,7 @@ cmdhistory_start_send_cb(GtkWidget *widget, gpointer data)
       gtk_tree_model_get_iter(model, &iter, (GtkTreePath *) list_iter->data);
       command = cmdhistory_get_command(model, &iter);
 
-      parse_commands(world, command, strlen(command));
+      parse_commands(world, command);
       g_free(command);
       list_iter = list_iter->next;
     }
@@ -405,7 +405,7 @@ cmdhistory_send_timer_cb(gpointer data)
   n = cmd_data->lines_at_a_time;
   while (n-- && cmd_data->list_iter) {
     gchar *command = (gchar *) cmd_data->list_iter->data;
-    parse_commands(cmd_data->world, command, strlen(command));
+    parse_commands(cmd_data->world, command);
     cmd_data->list_iter = cmd_data->list_iter->next;
   }
 
