@@ -1,7 +1,5 @@
-/* $Id: kcircularqueue.c 1143 2011-02-13 16:13:58Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,12 +45,6 @@ k_circular_queue_free(KCircularQueue *q)
 }
 
 
-/**
-  Frees memory used by a KCircularQueue.
-
-  @param q The KCircularQueue. The pointer will be invalid after the
-           call to this function.
-*/
 void
 k_circular_queue_push_copies(KCircularQueue *q,
                              const void     *val,
