@@ -1,7 +1,5 @@
-/* $Id: kildclient.h 1176 2011-05-28 14:43:45Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,10 +35,6 @@
 #  include <gnutls/gnutls.h>
 #endif
 
-#ifdef __MINGW32__
-#  include <winsock2.h>
-#endif
-
 
 /**********************
  * Limits and defines *
@@ -50,6 +44,8 @@
 #define VTE_ENC      "UTF-8"
 #define DEFAULT_TERMINAL_FONT "Monospace 10"
 #define DEFAULT_LOGTIME_FORMAT "%c> "
+#define CLIENT_NAME "KildClient"
+#define CLIENT_NAME_LEN 10
 
 
 /**************
@@ -85,6 +81,8 @@ enum TelnetState_e {
   TELNETFSM_SB_IAC_COMPRESS,
   TELNETFSM_SB_IAC_COMPRESS2,
   TELNETFSM_SB_IAC_COMPRESS_WILL, /* Also serves for SB_IAC_COMPRESS2_IAC */
+  TELNETFSM_SERVER_DATA,
+  TELNETFSM_SERVER_DATA_IAC,
 };
 typedef enum TelnetState_e TelnetState;
 
@@ -94,6 +92,7 @@ enum proxy_type_e {
   PROXY_NONE,
   PROXY_SOCKS4,
   PROXY_SOCKS5,
+  PROXY_HTTP,
 };
 typedef enum proxy_type_e proxy_type;
 
@@ -166,6 +165,10 @@ struct HookList_s
   gint                OnCloseConnected_pos;
   GtkTreeModel       *OnCloseConnected_model;
   GtkTreeModelFilter *OnCloseConnected_model_filter;
+  GSList             *OnServerData;
+  gint                OnServerData_pos;
+  GtkTreeModel       *OnServerData_model;
+  GtkTreeModelFilter *OnServerData_model_filter;
 };
 typedef struct HookList_s HookList;
 
@@ -206,8 +209,6 @@ struct TAState_s
      from the ANSI state. */
   GtkTextTag     *foreground_color_tag;
   GtkTextTag     *background_color_tag;
-
-  gboolean        linedraw;
 };
 typedef struct TAState_s TAState;
 
@@ -253,17 +254,6 @@ struct TextAppearance_s
   GtkTextTag *strike_tag;
   GtkTextTag *italics_tag;
 
-  /* The current color and attributes is determined from these variables */
-  TAState state;
-  /* A place to store the state at the end of the last complete line,
-     so that it can be restored when an incomplete line is erased. */
-  TAState saved_state;
-
-  /* For ANSI parsing */
-  ANSIFSMState ansifsm_state;
-  guchar       ansiseq_buffer[MAX_BUFFER]; /* Holds incomplete ansi seqs. */
-  gint         ansiseq_pos;
-
   /* Character groups supported. -1: unknown. 0: not supported. 1: supported
      If a group is not supported, replacement ASCII characters (not
      necessarily good) are used when in vt100 line drawing mode. */
@@ -279,6 +269,17 @@ struct TextAppearance_s
 typedef struct TextAppearance_s TextAppearance;
 
 
+struct AnsiParseState_s
+{
+  /* For ANSI parsing */
+  ANSIFSMState ansifsm_state;
+  gboolean     linedraw;
+  guchar       ansiseq_buffer[MAX_BUFFER]; /* Holds incomplete ansi seqs. */
+  gint         ansiseq_pos;
+};
+typedef struct AnsiParseState_s AnsiParseState;
+
+
 struct WorldGUI_s
 {
   struct world_s    *world;
@@ -305,8 +306,8 @@ struct WorldGUI_s
   GtkWidget         *commandArea;
   GtkWidget         *btnClear;
   union {
-    SimoComboBox      *cmbEntry;        /* For MUD windows */
-    GtkEntry          *txtEntry;        /* For KCWin's */
+    SimoComboBox    *cmbEntry;        /* For MUD windows */
+    GtkEntry        *txtEntry;        /* For KCWin's */
   };
 
   GtkBox            *statusbar_box;
@@ -327,6 +328,13 @@ struct WorldGUI_s
 
   /* Determines the appearance of the text */
   TextAppearance     ta;
+  AnsiParseState     ansi_parse;
+
+  /* The current color and attributes is determined from these variables */
+  TAState ta_state;
+  /* A place to store the state at the end of the last complete line,
+     so that it can be restored when an incomplete line is erased. */
+  TAState saved_ta_state;
 };
 typedef struct WorldGUI_s WorldGUI;
 
@@ -361,11 +369,11 @@ struct world_s
   gint              wrap_indent;
   gboolean          use_wrap_column;
   gint              wrap_column;
-  GdkColor         *deffore;
-  GdkColor         *defbold;
-  GdkColor         *defback;
-  GdkColor         *defboldback;
-  GdkColor         *ansicolors;
+  GdkRGBA          *deffore;
+  GdkRGBA          *defbold;
+  GdkRGBA          *defback;
+  GdkRGBA          *defboldback;
+  GdkRGBA          *ansicolors;
   NameDisplayStyle  name_display_style;
   char             *name_display_pattern;
   gboolean          use_tooltips;
@@ -471,6 +479,13 @@ struct world_s
   /* For MCCP handling */
   z_stream        *zstream;                  /* Used for decompression */
   gint             mccp_ver;                 /* Version of MCCP supported */
+  /* Server data */
+  guchar           server_data_protocol;     /* Protocol */
+  guchar          *server_data_buffer;       /* Buffer */
+  guint            server_data_alloc_size;   /* Allocated size */
+  guint            server_data_size;         /* Current size */
+  gboolean         gmcp_enabled;             /* Is GMCP enabled? */
+  gboolean         msdp_enabled;             /* Is MSDP enabled? */
 
   /* Time counts */
   guint            printed_until;
@@ -491,7 +506,6 @@ struct world_s
   gboolean         noecho;
   gboolean         use_naws;
   GdkRectangle     last_naws_size;
-  gboolean         dont_flush;
   gboolean         has_focus;
   gboolean         is_current_tab;
   gboolean         has_unread_text;
@@ -528,6 +542,7 @@ struct world_s
   GSList             *selected_OnGetFocus_hooks;
   GSList             *selected_OnLoseFocus_hooks;
   GSList             *selected_OnCloseConnected_hooks;
+  GSList             *selected_OnServerData_hooks;
   gboolean            confirm_delete;
   gboolean            show_plugin_items;
 
@@ -550,9 +565,28 @@ struct world_s
   /* TLS/SSL support */
   gboolean       use_tls;
 #ifdef HAVE_LIBGNUTLS
-  gnutls_session tls_session;
-  int            tls_handshake_dir;
+  gnutls_session_t tls_session;
+  int              tls_handshake_dir;
 #endif
+
+  /* Inter-thread communication */
+  GRecMutex    perl_mutex;
+  GThread     *process_line_thread;
+  GAsyncQueue *process_line_operations;
+
+  GThread     *ansiparse_thread;
+  GAsyncQueue *ansiparse_operations;
+
+  GMutex       write_log_line_mutex;
+
+  GAsyncQueue *mainthread_operations;
+  guint        mainthread_operations_idle_id;
+
+  GMutex       wait_command_mutex;
+  GCond        wait_command_cond;
+  gboolean     command_executed;
+
+  gboolean     to_be_destructed;
 };
 typedef struct world_s World;
 
@@ -639,11 +673,6 @@ struct trigger_s
   gint      high_italic;
   gint      high_strike;
   gint      high_underline;
-  /* These are not really part of the trigger, but here is a perfect
-     place to store them. They are only valid immediately after a
-     match, and serve only for that. */
-  gint match_start;
-  gint match_end;
 };
 typedef struct trigger_s Trigger;
 
@@ -677,7 +706,7 @@ struct timer_s
   Plugin   *owner_plugin;
   gboolean  enabled;
   World    *for_world;
-  guint     interval;
+  gdouble   interval;
   gint      count;
   char     *action;
   gboolean  temporary;
@@ -685,6 +714,7 @@ struct timer_s
 };
 typedef struct timer_s Timer;
 
+
 /* For multi-line send */
 enum MLSendState_e {
   ML_TRY_INITIALBUFFER,
@@ -722,6 +752,120 @@ struct MLSendData_s
 typedef struct MLSendData_s MLSendData;
 
 
+struct HighlightApplication_s
+{
+  Trigger *trigger;
+  gint     match_start;
+  gint     match_end;
+};
+typedef struct HighlightApplication_s HighlightApplication;
+
+
+struct HookExecution_s
+{
+  World *world;
+  gchar *data;
+};
+typedef struct HookExecution_s HookExecution;
+
+
+/* Line processing operations */
+enum ProcessLineOperationType_s {
+  PROCESS_COMPLETE_LINE,
+  PROCESS_INCOMPLETE_LINE,
+  DUMMY_PROCESS_LINE,
+};
+typedef enum ProcessLineOperationType_s ProcessLineOperationType;
+
+struct ProcessLineOperation_s
+{
+  ProcessLineOperationType  action;
+  gchar                    *line;
+  gsize                     len;
+};
+typedef struct ProcessLineOperation_s ProcessLineOperation;
+
+
+/* Ansi parsing operation */
+enum AnsiParseOperationType_s {
+  CLEAR_PENDING_LINE_ANSI,
+  APPEND_LINE,
+  APPEND_ECHOED_LINE,
+  APPLY_HIGHLIGHTS_AND_MOVE_MARK,
+  FINALIZE_LINE,
+  DUMMY_ANSIPARSE,
+};
+typedef enum AnsiParseOperationType_s AnsiParseOperationType;
+
+struct AnsiParseOperation_s
+{
+  AnsiParseOperationType action;
+
+  union
+  {
+    struct
+    {
+      gchar *line;
+      gsize  line_len;
+    } line;
+
+    GSList *highlights;
+  };
+};
+typedef struct AnsiParseOperation_s AnsiParseOperation;
+
+
+/* Main thread operations */
+enum MainThreadOperationType_s {
+  INSERT_AT_END,
+  INSERT_AT_MARK,
+  TEXT_ADDED,
+  CLEAR_PENDING_LINE,
+  MOVE_LINESTART_MARK,
+  APPLY_HIGHLIGHT,
+  PARSE_COMMANDS,
+  EXECUTE_ON_RECEIVED_TEXT,
+};
+typedef enum MainThreadOperationType_s MainThreadOperationType;
+
+struct MainThreadOperation_s
+{
+  MainThreadOperationType  action;
+
+  union
+  {
+    /* For INSERT_AT_END and INSERT_AT_MARK */
+    struct
+    {
+      gchar      *text;
+      gboolean    use_ta;
+      TAState     ta_state;
+      GtkTextTag *extra_tag;
+    } text_insert;
+
+    /* For APPLY_HIGHLIGHT */
+    HighlightApplication *highlight;
+
+    /* For PARSE_COMMANDS */
+    struct
+    {
+      gchar    *commands;
+      gchar    *colorline;
+      AV       *matches;
+      gboolean  notify;
+    } commands;
+
+    /* For EXECUTE_ON_RECEIVED_TEXT */
+    HookExecution *hook_execution;
+  };
+};
+typedef struct MainThreadOperation_s MainThreadOperation;
+
+/* Function that enqueues or executes a text view operation. */
+typedef void  (*MainThreadOperationFunction)(WorldGUI            *gui,
+                                             MainThreadOperation *operation);
+
+
 /**************************************************
  * Model Columns in GUI Editors for Triggers, etc *
  **************************************************/
@@ -789,14 +933,14 @@ extern gboolean    debug_matches;        /* Show when a trigger/alias matches */
 extern Prefs       globalPrefs;          /* Holds the global preferences */
 extern FILE       *dumpFile;             /* File to dump decompr. data*/
 extern FILE       *rdumpFile;            /* File to dump raw received data */
-extern GdkColor    deffore;              /* Default foreground color */
-extern GdkColor    defbold;              /* Default bold foreground color */
-extern GdkColor    defback;              /* Default background color */
-extern GdkColor    defboldback;          /* Default bold background color */
-extern GdkColor    defansi[];            /* Default ansi colors */
+extern GdkRGBA     deffore;              /* Default foreground color */
+extern GdkRGBA     defbold;              /* Default bold foreground color */
+extern GdkRGBA     defback;              /* Default background color */
+extern GdkRGBA     defboldback;          /* Default bold background color */
+extern GdkRGBA     defansi[];            /* Default ansi colors */
 #ifdef HAVE_LIBGNUTLS
 /* gnutls certificate credentials */
-extern gnutls_certificate_credentials tls_certif_cred;
+extern gnutls_certificate_credentials_t tls_certif_cred;
 #endif
 
 
@@ -804,6 +948,8 @@ extern gnutls_certificate_credentials tls_certif_cred;
  * Function prototypes - Main UI *
  *********************************/
 GtkWidget* create_main_window(WorldGUI *gui);
+void       set_main_window_size(int x, int y, int width, int height,
+                                WorldGUI *gui);
 World*     get_world_to_connect();
 void       remove_world(World *world, gboolean remove_gui);
 gboolean   remove_world_timer(World *world);
@@ -818,8 +964,8 @@ gboolean   open_new_world(WorldGUI    *gui,
 gboolean   update_times_cb(gpointer data);
 gboolean   confirm_quit(void);
 void       worldgui_destroy_tooltip_window(WorldGUI *gui);
-G_MODULE_EXPORT void     menu_findnext_activate_cb(GtkWidget *widget,
-                                                   gpointer   data);
+G_MODULE_EXPORT void     menu_findnext_activate_cb(GtkMenuItem *widget,
+                                                   gpointer     data);
 
 
 /*********************
@@ -846,7 +992,48 @@ void      completion_cell_data_function(GtkCellLayout   *cell_layout,
                                         GtkTreeIter     *iter,
                                         gpointer         data);
 void      find_next_cb(GtkButton *button, gpointer data);
-G_MODULE_EXPORT void menu_url_open(GtkAction *action, gchar *url);
+G_MODULE_EXPORT void menu_url_open(GtkMenuItem *menu, gchar *url);
+
+
+/************************************************
+ * Function prototypes - Line processing thread *
+ ************************************************/
+gpointer process_line_thread(gpointer data);
+void     queue_process_line_operation(World                *world,
+                                      ProcessLineOperation *operation);
+void     free_process_line_operation(gpointer data);
+void     match_triggers(World     *world,
+                        gchar    **line,
+                        gchar    **color_line,
+                        gsize     *output_len,
+                        gboolean  *gag_output,
+                        gboolean  *gag_log,
+                        gint      *n_matches,
+                        GSList   **trigger_response);
+
+
+/**************************************
+ * Function prototypes - ANSI parsing *
+ **************************************/
+gpointer ansiparse_thread(gpointer data);
+void     queue_ansiparse_operation(World *gui, AnsiParseOperation *operation);
+void     free_ansiparse_operation(gpointer data);
+void     insert_ansi_string(WorldGUI                    *gui,
+                            const gchar                 *str,
+                            gsize                        len,
+                            MainThreadOperationType      where,
+                            MainThreadOperationFunction  operation_func);
+
+
+/*********************************************************************
+ * Function prototypes - Operations that must run in the main thread *
+ *********************************************************************/
+void      queue_mainthread_operation(World               *world,
+                                     MainThreadOperation *operation);
+void      free_mainthread_operation(gpointer data);
+gboolean  mainthread_operations_idle_cb(gpointer data);
+void      process_mainthread_operation(WorldGUI            *world,
+                                       MainThreadOperation *operation);
 
 
 /***************************************************
@@ -862,19 +1049,29 @@ void      ansitextview_append_stringf(WorldGUI    *gui,
 void      ansitextview_append_string_with_fgcolor(WorldGUI    *gui,
                                                   const gchar *str,
                                                   gint         fg_color);
-void      ansitextview_append_ansi_string(WorldGUI    *gui,
-                                          const gchar *str,
-                                          gsize        len);
-void      ansitextview_append_echoed_string(WorldGUI    *gui,
-                                            const gchar *str,
-                                            gsize        len);
+void      ansitextview_append_ansi_string_immediately(WorldGUI *gui,
+                                                      gchar *str,
+                                                      gsize len);
 void      ansitextview_get_char_size(WorldGUI *gui,
                                      gint     *char_height,
                                      gint     *char_width);
 void      ansitextview_get_size(WorldGUI *gui, guint *lines, guint *cols);
 void      ansitextview_update_color_tags(WorldGUI *gui, World *world);
 gboolean  ansitextview_prune_extra_lines(gpointer data);
-guchar   *strip_ansi(const guchar *original, int len);
+gchar    *strip_ansi(const gchar *original, gsize len);
+/* Main thread operations */
+void      ansitextview_op_insert_at_end(WorldGUI            *gui,
+                                        MainThreadOperation *operation);
+void      ansitextview_op_insert_at_mark(WorldGUI            *gui,
+                                         MainThreadOperation *operation);
+void      ansitextview_op_text_added(WorldGUI            *gui,
+                                     MainThreadOperation *operation);
+void      ansitextview_op_clear_pending_line(WorldGUI            *gui,
+                                             MainThreadOperation *operation);
+void      ansitextview_op_move_linestart_mark(WorldGUI            *gui,
+                                              MainThreadOperation *operation);
+void      ansitextview_op_apply_highlight(WorldGUI            *gui,
+                                          MainThreadOperation *operation);
 
 
 /********************************
@@ -882,6 +1079,7 @@ guchar   *strip_ansi(const guchar *original, int len);
  ********************************/
 void      set_default_colors(World *world);
 World    *create_new_world(gboolean load_defaults);
+void      init_threads(World *world);
 void      free_world(World *world);
 void      remove_plugin_objects(World *world, Plugin *plugin);
 gboolean  load_world_from_file(const char *file,
@@ -1044,14 +1242,21 @@ void      perform_auto_logon(World *world);
 void        read_global_preferences(int *x, int *y, int *width, int *height);
 void        save_global_preferences(void);
 const char *create_kildclient_directory(void);
-G_MODULE_EXPORT void show_preferences_dialog_cb(GtkWidget *widget,
-                                                gpointer data);
+G_MODULE_EXPORT void show_preferences_dialog_cb(GtkMenuItem *widget,
+                                                gpointer     data);
 
 
 /********************************
  * Function Prototypes - Parser *
  ********************************/
-void parse_commands(World *world, const char *cmdline, gint len);
+void parse_commands(World *world, const char *cmdline);
+void parse_commands_full(World       *world,
+                         const char  *cmdline,
+                         AV          *matches,
+                         const gchar *colorline,
+                         gboolean     notify);
+void mainthread_op_parse_commands(World *world, MainThreadOperation *operation);
+void parse_command_line_immediately(World *world, const char *cmdline);
 void send_to_world(World *world, const char *cmd, gint len);
 void send_to_world_no_check(World      *world,
                             const char *utf8_command,
@@ -1066,14 +1271,6 @@ void send_to_world_no_check(World      *world,
  * Function Prototypes - Triggers *
  **********************************/
 Trigger  *new_trigger(void);
-void      match_triggers(World     *world,
-                         guchar   **line,
-                         guchar   **color_line,
-                         gsize     *output_len,
-                         gboolean  *gag_output,
-                         gboolean  *gag_log,
-                         gint      *n_matches,
-                         GSList   **trigger_response);
 void      remove_trigger(World *world, GSList *triggeritem);
 gboolean  move_trigger(World *world, gint old_pos, gint new_pos);
 void      free_trigger(Trigger *trigger, gpointer data);
@@ -1136,9 +1333,12 @@ gboolean      connect_hook(World       *world,
                            gchar       *action,
                            gchar       *name,
                            gboolean     enabled);
-gboolean      execute_hook(World       *world,
+void          execute_hook(World       *world,
                            const gchar *hook,
                            const gchar *hookdata);
+void          execute_hook_immediately(World       *world,
+                                       const gchar *hook,
+                                       const gchar *hookdata);
 void          list_hooks(World *world, const gchar *hookname, Plugin *plugin);
 void          list_hook(World *world, const gchar *hookname, int pos);
 gboolean      delete_hook(World *world,
@@ -1157,6 +1357,8 @@ GSList       *get_hook_list_for_reading(World       *world,
                                         const gchar *hookname,
                                         gint        *pos);
 GtkTreeModel *get_hook_model(World *world, const gchar *hookname);
+gboolean      are_there_enabled_server_data_hooks(World *world);
+void          mainthread_op_execute_on_received_text_hook(HookExecution *hook_execution);
 
 
 /*********************************
@@ -1184,5 +1386,4 @@ void     make_startup_plugin_list_for_editing(World *world);
 void play_sound(const gchar *file);
 
 
-
 #endif
