@@ -1,7 +1,5 @@
-/* $Id: macros.c 1176 2011-05-28 14:43:45Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +22,7 @@
 #endif
 
 #include <string.h>
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <locale.h>
 #include <gtk/gtk.h>
 
@@ -62,7 +60,7 @@ check_macros(World *world, guint keyval, GdkModifierType modifiers)
     macro = (Macro *) macroptr->data;
     if (macro->enabled &&
         macro->keyval == keyval && macro->modifiers == (modifiers & GDK_MODIFIER_MASK)) {
-      parse_commands(world, macro->action, strlen(macro->action));
+      parse_commands(world, macro->action);
       return TRUE;
     }
     macroptr = macroptr->next;
