@@ -1,7 +1,5 @@
-/* $Id: main.c 1143 2011-02-13 16:13:58Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,13 +22,17 @@
 #endif
 
 #include <signal.h>
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <locale.h>
 #include <gtk/gtk.h>
 #ifdef HAVE_LIBGNUTLS
 #  include <gnutls/gnutls.h>
 #endif
 
+#ifdef __MINGW32__
+#  include <winsock2.h>
+#endif
+
 #if defined __CYGWIN32__
 #  include <sys/cygwin.h>
 #  include <windows.h>
@@ -40,10 +42,6 @@
 #  include <shlobj.h>
 #endif
 
-#ifdef __MINGW32__
-#  include <winsock2.h>
-#endif
-
 #include "kildclient.h"
 #include "perlscript.h"
 
@@ -60,7 +58,7 @@ FILE *rdumpFile = NULL;
  * File global variables *
  *************************/
 #ifdef HAVE_LIBGNUTLS
-gnutls_certificate_credentials tls_certif_cred;
+gnutls_certificate_credentials_t tls_certif_cred;
 #endif
 
 
@@ -155,33 +153,15 @@ main(int argc, char *argv[], char *env[])
 #endif
 
   gtk_init(&argc, &argv);
-  gtk_rc_parse_string("gtk-entry-select-on-focus=0");
-
-  if (!g_thread_supported()) {
-    g_thread_init(NULL);
-  }
+  gtk_settings_set_long_property(gtk_settings_get_default(),
+                                 "gtk-entry-select-on-focus", 0,
+                                 "KildClient");
 
   read_global_preferences(&x, &y, &width, &height);
 
   gui = world_gui_new(TRUE);
   wndMain = create_main_window(gui);
-  if (x != -1) {
-    gtk_window_move(GTK_WINDOW(wndMain), x, y);
-    gtk_window_resize(GTK_WINDOW(wndMain), width, height);
-  } else {
-    /* Since world_gui_size_textview() sets the minimum size, and we
-       only want to set the initial size, which is only possible for
-       windows, we use a little trick. */
-    GtkRequisition requisition;
-
-    world_gui_size_textview(gui);
-    gtk_widget_size_request(wndMain, &requisition);
-    gtk_widget_set_size_request(GTK_WIDGET(gui->txtView),
-                                100, 100);
-    gtk_window_resize(GTK_WINDOW(wndMain),
-                      requisition.width,
-                      requisition.height);
-  }
+  set_main_window_size(x, y, width, height, gui);
 
   init_perl_system();
 
@@ -262,7 +242,6 @@ confirm_quit(void)
   int        n_open;
   gboolean   retval = FALSE;
   GtkWidget *msgdlg;
-  GtkWidget *dlgbutton;
 
   n_open = 0;
   worldptr = open_worlds;
@@ -294,12 +273,10 @@ confirm_quit(void)
                                     n_open);
   }
   gtk_window_set_title(GTK_WINDOW(msgdlg), _("Really Quit KildClient?"));
-  gtk_dialog_add_button(GTK_DIALOG(msgdlg),
-                        _("Don't quit"), GTK_RESPONSE_NO);
-  dlgbutton = gtk_button_new_from_stock(GTK_STOCK_QUIT);
-  gtk_widget_show_all(dlgbutton);
-  gtk_dialog_add_action_widget(GTK_DIALOG(msgdlg),
-                               dlgbutton, GTK_RESPONSE_YES);
+  gtk_dialog_add_buttons(GTK_DIALOG(msgdlg),
+                         _("_Don't quit"), GTK_RESPONSE_NO,
+                         _("_Quit"), GTK_RESPONSE_YES,
+                         NULL);
   if (gtk_dialog_run(GTK_DIALOG(msgdlg)) == GTK_RESPONSE_YES) {
     retval = TRUE;
   }
@@ -331,9 +308,8 @@ get_kildclient_directory_path(void)
 const char *
 get_kildclient_installed_file(const char *file)
 {
-  /* file is the name of a file that is in the same directory
-     as kildclient.exe. This function finds the installation directory
-     and returns the full path to that file. */
+  /* Gets the path to a file in the data directory (typically
+   * /usr/share/kildclient) */
   static char path[PATH_MAX];
 
   strcpy(path, PKGDATADIR "/");
@@ -366,9 +342,10 @@ get_kildclient_directory_path(void)
 const char *
 get_kildclient_installed_file(const char *file)
 {
-  /* file is the name of a file that is in the same directory
-     as kildclient.exe. This function finds the installation directory
-     and returns the full path to that file. */
+  /* Under windows, paths start at the same directory that
+     kildclient.exe is in. file is the name of a file that should be
+     in the same directory as kildclient.exe. This function finds the
+     installation directory and returns the full path to that file. */
   static char path[PATH_MAX];
   char       *pos;
 
