@@ -1,7 +1,5 @@
-/* $Id: mainwindow.c 1176 2011-05-28 14:43:45Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +25,7 @@
 #include <string.h>
 #include <time.h>
 #include <errno.h>
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <glib.h>
 #include <gtk/gtk.h>
 #include <gmodule.h>
@@ -65,10 +63,10 @@ gboolean    debug_matches        = 0;
 /***********************
  * Function prototypes *
  ***********************/
+static void     restore_main_window_size(int x, int y, int width, int height,
+                                         WorldGUI *gui);
+static void     set_initial_window_size(WorldGUI *gui);
 static gboolean update_world_with_focus(gpointer data);
-static void     about_dlg_url_hook(GtkAboutDialog *about,
-                                   const gchar    *link,
-                                   gpointer        data);
 static gint     attach_world_gui(WorldGUI *gui);
 static void     detach_world_gui(WorldGUI *gui);
 static gchar   *format_time_string(World *world, int ctime, int itime);
@@ -83,72 +81,67 @@ G_MODULE_EXPORT gboolean window_focus_in_cb(GtkWidget     *widget,
 G_MODULE_EXPORT gboolean window_focus_out_cb(GtkWidget     *widget,
                                              GdkEventFocus *evt,
                                              gpointer       data);
-G_MODULE_EXPORT void     notebook_page_changed_cb(GtkNotebook     *notebook,
-                                                  GtkNotebookPage *page,
-                                                  guint            page_num,
-                                                  gpointer         data);
+G_MODULE_EXPORT void     notebook_page_changed_cb(GtkNotebook *notebook,
+                                                  gpointer    *page,
+                                                  guint        page_num,
+                                                  gpointer     data);
 G_MODULE_EXPORT void     notebook_page_reordered_cb(GtkNotebook *notebook,
                                                     GtkWidget   *page,
                                                     guint        page_num,
                                                     gpointer     data);
+G_MODULE_EXPORT gboolean about_dlg_url_hook(GtkAboutDialog *about,
+                                            gchar          *link,
+                                            gpointer        data);
 /* Menus */
-G_MODULE_EXPORT void     menu_world_open_cb(gpointer data, GtkWidget *widget);
-G_MODULE_EXPORT void     open_new_world_cb(GtkWidget *widget,
-                                           gpointer data);
-G_MODULE_EXPORT void     edit_this_world_cb(GtkWidget *widget,
-                                            gpointer data);
-G_MODULE_EXPORT void     menu_statistics_cb(GtkWidget *widget,
-                                            gpointer data);
+G_MODULE_EXPORT void     menu_world_open_cb(GtkMenuItem *widget, gpointer data);
+G_MODULE_EXPORT void     open_new_world_cb(GtkMenuItem *widget, gpointer data);
+G_MODULE_EXPORT void     edit_this_world_cb(GtkMenuItem *widget, gpointer data);
+G_MODULE_EXPORT void     menu_statistics_cb(GtkMenuItem *widget, gpointer data);
 
-G_MODULE_EXPORT void     menu_edit_open_cb(GtkWidget *widget,
-                                           gpointer   data);
-G_MODULE_EXPORT void     menu_cut_activate_cb(GtkWidget *widget,
-                                              gpointer   data);
-G_MODULE_EXPORT void     menu_copy_activate_cb(GtkWidget *widget,
-                                               gpointer   data);
-G_MODULE_EXPORT void     menu_paste_activate_cb(GtkWidget *widget,
-                                                gpointer   data);
-G_MODULE_EXPORT void     menu_delete_activate_cb(GtkWidget *widget,
-                                                 gpointer   data);
-G_MODULE_EXPORT void     menu_find_activate_cb(GtkWidget *widget,
-                                               gpointer   data);
+G_MODULE_EXPORT void     menu_edit_open_cb(GtkMenuItem *widget, gpointer data);
+G_MODULE_EXPORT void     menu_cut_activate_cb(GtkMenuItem *widget,
+                                              gpointer     data);
+G_MODULE_EXPORT void     menu_copy_activate_cb(GtkMenuItem *widget,
+                                               gpointer     data);
+G_MODULE_EXPORT void     menu_paste_activate_cb(GtkMenuItem *widget,
+                                                gpointer     data);
+G_MODULE_EXPORT void     menu_delete_activate_cb(GtkMenuItem *widget,
+                                                 gpointer     data);
+G_MODULE_EXPORT void     menu_find_activate_cb(GtkMenuItem   *widget,
+                                               gpointer       data);
 
-G_MODULE_EXPORT void     menu_input_clear_cb(GtkWidget *widget,
-                                             gpointer data);
-G_MODULE_EXPORT void     menu_input_prev_cb(GtkWidget *widget,
-                                            gpointer data);
-G_MODULE_EXPORT void     menu_input_next_cb(GtkWidget *widget,
-                                            gpointer data);
-G_MODULE_EXPORT void     menu_input_find_prev_cb(GtkWidget *widget,
-                                                 gpointer data);
-G_MODULE_EXPORT void     menu_input_find_next_cb(GtkWidget *widget,
-                                                 gpointer data);
+G_MODULE_EXPORT void     menu_input_clear_cb(GtkMenuItem *widget,
+                                             gpointer     data);
+G_MODULE_EXPORT void     menu_input_prev_cb(GtkMenuItem *widget, gpointer data);
+G_MODULE_EXPORT void     menu_input_next_cb(GtkMenuItem *widget, gpointer data);
+G_MODULE_EXPORT void     menu_input_find_prev_cb(GtkMenuItem *widget,
+                                                 gpointer     data);
+G_MODULE_EXPORT void     menu_input_find_next_cb(GtkMenuItem *widget,
+                                                 gpointer     data);
 
 
-G_MODULE_EXPORT void     menu_preferences_open_cb(GtkWidget *widget,
+G_MODULE_EXPORT void     menu_preferences_open_cb(GtkMenuItem *widget,
                                                   gpointer data);
-G_MODULE_EXPORT void     menu_disable_triggers_cb(GtkWidget *widget,
+G_MODULE_EXPORT void     menu_disable_triggers_cb(GtkMenuItem *widget,
                                                   gpointer data);
-G_MODULE_EXPORT void     menu_disable_aliases_cb(GtkWidget *widget,
-                                                 gpointer data);
-G_MODULE_EXPORT void     menu_disable_macros_cb(GtkWidget *widget,
-                                                gpointer data);
-G_MODULE_EXPORT void     menu_disable_timers_cb(GtkWidget *widget,
-                                                gpointer data);
-G_MODULE_EXPORT void     mnuDebugMatches_activate_cb(GtkWidget *widget,
-                                                     gpointer data);
-G_MODULE_EXPORT void     menu_edit_default_world_cb(GtkWidget *widget,
-                                                    gpointer data);
+G_MODULE_EXPORT void     menu_disable_aliases_cb(GtkMenuItem *widget,
+                                                 gpointer     data);
+G_MODULE_EXPORT void     menu_disable_macros_cb(GtkMenuItem *widget,
+                                                gpointer     data);
+G_MODULE_EXPORT void     menu_disable_timers_cb(GtkMenuItem *widget,
+                                                gpointer     data);
+G_MODULE_EXPORT void     menu_debug_matches_cb(GtkMenuItem *widget,
+                                               gpointer     data);
+G_MODULE_EXPORT void     menu_edit_default_world_cb(GtkMenuItem *widget,
+                                                    gpointer     data);
 
-G_MODULE_EXPORT void     menu_display_open_cb(GtkWidget *widget,
-                                              gpointer data);
-G_MODULE_EXPORT void     menu_split_screen_cb(GtkAction *widget,
-                                              gpointer data);
+G_MODULE_EXPORT void     menu_display_open_cb(GtkMenuItem *widget,
+                                              gpointer     data);
+G_MODULE_EXPORT void     menu_split_screen_cb(GtkMenuItem *widget,
+                                              gpointer     data);
 
-G_MODULE_EXPORT void     menu_manual_cb(GtkWidget *widget,
-                                        gpointer data);
-G_MODULE_EXPORT void     about_menu_cb(GtkWidget *widget,
-                                       gpointer data);
+G_MODULE_EXPORT void     menu_manual_cb(GtkMenuItem *widget, gpointer data);
+G_MODULE_EXPORT void     menu_about_cb(GtkMenuItem *widget, gpointer data);
 
 
 
@@ -224,6 +217,43 @@ create_main_window(WorldGUI *gui)
 }
 
 
+void
+set_main_window_size(int x, int y, int width, int height, WorldGUI *gui)
+{
+  if (x != -1) {
+    restore_main_window_size(x, y, width, height, gui);
+  } else {
+    set_initial_window_size(gui);
+  }
+}
+
+
+void
+restore_main_window_size(int x, int y, int width, int height, WorldGUI *gui)
+{
+  gtk_window_move(GTK_WINDOW(wndMain), x, y);
+  gtk_window_resize(GTK_WINDOW(wndMain), width, height);
+}
+
+
+void
+set_initial_window_size(WorldGUI *gui)
+{
+  /* Since world_gui_size_textview() sets the minimum size, and we
+     only want to set the initial size, which is only possible for
+     windows, we use a little trick. */
+  GtkRequisition requisition;
+
+  world_gui_size_textview(gui);
+  gtk_widget_get_preferred_size(GTK_WIDGET(gui->txtView), &requisition, NULL);
+  gtk_widget_set_size_request(GTK_WIDGET(gui->txtView),
+                              100, 100);
+  gtk_window_resize(GTK_WINDOW(wndMain),
+                    requisition.width,
+                    requisition.height * 1.5);
+}
+
+
 gboolean
 exit_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
 {
@@ -339,10 +369,10 @@ adjust_window_title(void)
 
 
 void
-notebook_page_changed_cb(GtkNotebook     *notebook,
-                         GtkNotebookPage *page,
-                         guint            page_num,
-                         gpointer         data)
+notebook_page_changed_cb(GtkNotebook *notebook,
+                         gpointer    *page,
+                         guint        page_num,
+                         gpointer     data)
 {
   World *old_world = NULL;
   gint   old_page;
@@ -413,8 +443,6 @@ update_world_with_focus(gpointer data)
       --worlds_with_new_text;
     }
 
-    PERL_SET_CONTEXT(currentWorld->perl_interpreter);
-
     adjust_window_title();
 
     if (currentWorld->connected) {
@@ -428,7 +456,7 @@ update_world_with_focus(gpointer data)
 
 
 void
-open_new_world_cb(GtkWidget *widget, gpointer data)
+open_new_world_cb(GtkMenuItem *widget, gpointer data)
 {
   open_new_world(NULL, NULL, FALSE, NULL);
 }
@@ -508,7 +536,7 @@ open_new_world(WorldGUI    *gui,
 
 
 void
-edit_this_world_cb(GtkWidget *widget, gpointer data)
+edit_this_world_cb(GtkMenuItem *widget, gpointer data)
 {
   gboolean dummy;
 
@@ -520,7 +548,7 @@ edit_this_world_cb(GtkWidget *widget, gpointer data)
 
 
 void
-menu_statistics_cb(GtkWidget *widget, gpointer data)
+menu_statistics_cb(GtkMenuItem *widget, gpointer data)
 {
   GtkBuilder *ui_builder;
   GError     *error = NULL;
@@ -538,7 +566,6 @@ menu_statistics_cb(GtkWidget *widget, gpointer data)
   GtkLabel   *lblBytes;
   GtkLabel   *lblCRatio;
   GtkLabel   *lblLines;
-  GtkLabel   *lblTLS;
   gchar      *str;
   time_t      now;
   GString    *timestr;
@@ -565,7 +592,6 @@ menu_statistics_cb(GtkWidget *widget, gpointer data)
   lblBytes       = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblBytes"));
   lblCRatio      = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblCRatio"));
   lblLines       = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblLines"));
-  lblTLS         = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblTLS"));
 
   /* Basic information */
   gtk_label_set_text(lblName, currentWorld->name);
@@ -641,28 +667,30 @@ menu_statistics_cb(GtkWidget *widget, gpointer data)
   /* TLS information */
 #ifdef HAVE_LIBGNUTLS
   if (currentWorld->use_tls) {
-    GtkLabel                *lblTLSCipher;
-    GtkLabel                *lblTLSMAC;
-    GtkLabel                *lblTLSKX;
-    GtkLabel                *lblTLSComp;
-    GtkLabel                *lblTLSCertType;
-    GtkLabel                *lblTLSCertHostname;
-    GtkLabel                *lblTLSCertActivation;
-    GtkLabel                *lblTLSCertExpiration;
-    GtkLabel                *lblTLSCertFingerprint;
-    gnutls_certificate_type  cert_type;
-    const gnutls_datum      *cert_list;
-    guint                    cert_list_size;
-    gboolean                 hostname_matches = FALSE;
-    time_t                   act_time;
-    time_t                   exp_time;
-    gchar                    digest[20];
-    size_t                   digest_size;
-    const gchar             *val;
-    gchar                   *mval;
-    gchar                    buf[MAX_BUFFER];
-    guint                    i;
+    GtkLabel                  *lblTLS;
+    GtkLabel                  *lblTLSCipher;
+    GtkLabel                  *lblTLSMAC;
+    GtkLabel                  *lblTLSKX;
+    GtkLabel                  *lblTLSComp;
+    GtkLabel                  *lblTLSCertType;
+    GtkLabel                  *lblTLSCertHostname;
+    GtkLabel                  *lblTLSCertActivation;
+    GtkLabel                  *lblTLSCertExpiration;
+    GtkLabel                  *lblTLSCertFingerprint;
+    gnutls_certificate_type_t  cert_type;
+    const gnutls_datum_t      *cert_list;
+    guint                      cert_list_size;
+    gboolean                   hostname_matches = FALSE;
+    time_t                     act_time;
+    time_t                     exp_time;
+    gchar                      digest[20];
+    size_t                     digest_size;
+    const gchar               *val;
+    gchar                     *mval;
+    gchar                      buf[MAX_BUFFER];
+    guint                      i;
 
+    lblTLS = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblTLS"));
     val = gnutls_protocol_get_name(gnutls_protocol_get_version(currentWorld->tls_session));
     gtk_label_set_text(lblTLS, val);
 
@@ -701,10 +729,10 @@ menu_statistics_cb(GtkWidget *widget, gpointer data)
     cert_list = gnutls_certificate_get_peers(currentWorld->tls_session,
                                              &cert_list_size);
     if (cert_type == GNUTLS_CRT_X509) {
-      GtkLabel       *lblTLSCertSubDN;
-      GtkLabel       *lblTLSCertIssDN;
-      gnutls_x509_crt crt;
-      size_t          size;
+      GtkLabel          *lblTLSCertSubDN;
+      GtkLabel          *lblTLSCertIssDN;
+      gnutls_x509_crt_t  crt;
+      size_t             size;
 
       if (cert_list_size > 0) {   /* Note that we consider only the first */
         gnutls_x509_crt_init(&crt);
@@ -765,6 +793,8 @@ menu_statistics_cb(GtkWidget *widget, gpointer data)
     mval[strlen(mval) - 1] = 0; /* Remove last : */
     gtk_label_set_text(lblTLSCertFingerprint, buf);
   } else {
+    GtkLabel *lblTLS;
+    lblTLS = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblTLS"));
     gtk_label_set_text(lblTLS, _("None"));
   }
 #endif
@@ -777,7 +807,7 @@ menu_statistics_cb(GtkWidget *widget, gpointer data)
 
 
 void
-menu_edit_default_world_cb(GtkWidget *widget, gpointer data)
+menu_edit_default_world_cb(GtkMenuItem *widget, gpointer data)
 {
   gboolean dummy;
 
@@ -786,7 +816,7 @@ menu_edit_default_world_cb(GtkWidget *widget, gpointer data)
 
 
 void
-menu_manual_cb(GtkWidget *widget, gpointer data)
+menu_manual_cb(GtkMenuItem *widget, gpointer data)
 {
   gchar *manualurl;
 
@@ -801,14 +831,12 @@ menu_manual_cb(GtkWidget *widget, gpointer data)
 
 
 void
-about_menu_cb(GtkWidget *widget, gpointer data)
+menu_about_cb(GtkMenuItem *widget, gpointer data)
 {
-  GError           *error = NULL;
-  static GtkWidget *dlgAbout = NULL;
+  GError                *error = NULL;
+  static GtkAboutDialog *dlgAbout = NULL;
 
   if (!dlgAbout) {
-    gtk_about_dialog_set_url_hook(about_dlg_url_hook, NULL, NULL);
-
     if (!gtk_builder_add_from_file(main_builder,
                                    get_kildclient_installed_file("dlgAbout.ui"),
                                    &error)) {
@@ -816,23 +844,27 @@ about_menu_cb(GtkWidget *widget, gpointer data)
       g_error_free(error);
       return;
     }
-    dlgAbout = GTK_WIDGET(gtk_builder_get_object(main_builder, "dlgAbout"));
-    gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(dlgAbout), VERSION);
+    dlgAbout = GTK_ABOUT_DIALOG(gtk_builder_get_object(main_builder,
+                                                       "dlgAbout"));
+    gtk_about_dialog_set_version(dlgAbout, VERSION);
+    /* TODO: Will not be necessary if future glade versions do not insist
+       on setting this property. */
+    gtk_about_dialog_set_logo(dlgAbout, NULL);
 
     gtk_builder_connect_signals(main_builder, NULL);
   }
 
-  gtk_widget_show_all(dlgAbout);
+  gtk_widget_show_all(GTK_WIDGET(dlgAbout));
   gtk_window_present(GTK_WINDOW(dlgAbout));
 }
 
 
-static
-void
-about_dlg_url_hook(GtkAboutDialog *about, const gchar *link, gpointer data)
+gboolean
+about_dlg_url_hook(GtkAboutDialog *about, gchar *link, gpointer data)
 {
   gchar *url = g_strdup(link);
   menu_url_open(NULL, url);
+  return TRUE;
 }
 
 
@@ -941,25 +973,25 @@ set_notebook_tab_position(GtkPositionType pos)
 
 
 void
-menu_world_open_cb(gpointer data, GtkWidget *widget)
+menu_world_open_cb(GtkMenuItem *widget, gpointer data)
 {
-  GtkAction *mnuDisconnect = GTK_ACTION(data);
+  GtkWidget *mnuDisconnect = GTK_WIDGET(data);
 
   if (currentWorld && (currentWorld->connected || currentWorld->connecting)) {
-    gtk_action_set_sensitive(mnuDisconnect, TRUE);
+    gtk_widget_set_sensitive(mnuDisconnect, TRUE);
   } else {
-    gtk_action_set_sensitive(mnuDisconnect, FALSE);
+    gtk_widget_set_sensitive(mnuDisconnect, FALSE);
   }
 }
 
 
 void
-menu_edit_open_cb(GtkWidget *widget, gpointer data)
+menu_edit_open_cb(GtkMenuItem *widget, gpointer data)
 {
-  GtkAction    *mnuCut;
-  GtkAction    *mnuCopy;
-  GtkAction    *mnuPaste;
-  GtkAction    *mnuDelete;
+  GtkWidget    *mnuCut;
+  GtkWidget    *mnuCopy;
+  GtkWidget    *mnuPaste;
+  GtkWidget    *mnuDelete;
   GdkDisplay   *display;
   GtkClipboard *clipboard;
   gboolean      input_selected;
@@ -971,44 +1003,42 @@ menu_edit_open_cb(GtkWidget *widget, gpointer data)
     return;
   }
 
-  mnuCut     = GTK_ACTION(gtk_builder_get_object(main_builder, "mnuCut"));
-  mnuCopy    = GTK_ACTION(gtk_builder_get_object(main_builder, "mnuCopy"));
-  mnuPaste   = GTK_ACTION(gtk_builder_get_object(main_builder, "mnuPaste"));
-  mnuDelete  = GTK_ACTION(gtk_builder_get_object(main_builder, "mnuDelete"));
+  mnuCut     = GTK_WIDGET(gtk_builder_get_object(main_builder, "mnuCut"));
+  mnuCopy    = GTK_WIDGET(gtk_builder_get_object(main_builder, "mnuCopy"));
+  mnuPaste   = GTK_WIDGET(gtk_builder_get_object(main_builder, "mnuPaste"));
+  mnuDelete  = GTK_WIDGET(gtk_builder_get_object(main_builder, "mnuDelete"));
 
   /* We can paste when there is text in the clipboard */
   display = gtk_widget_get_display(wndMain);
   clipboard = gtk_clipboard_get_for_display(display, GDK_SELECTION_CLIPBOARD);
-  gtk_action_set_sensitive(mnuPaste,
+  gtk_widget_set_sensitive(mnuPaste,
                            gtk_clipboard_wait_is_text_available(clipboard));
 
   /* We can cut and delete if there is text selected in the input box */
   input_selected
     = simo_combo_box_get_selection_bounds(currentWorld->gui->cmbEntry,
                                           NULL, NULL);
-  gtk_action_set_sensitive(mnuCut,    input_selected);
-  gtk_action_set_sensitive(mnuDelete, input_selected);
+  gtk_widget_set_sensitive(mnuCut,    input_selected);
+  gtk_widget_set_sensitive(mnuDelete, input_selected);
 
   /* We can copy if there is a selection in the input box or the output
      window */
   output_selected
     = gtk_text_buffer_get_selection_bounds(currentWorld->gui->txtBuffer,
                                            NULL, NULL);
-  gtk_action_set_sensitive(mnuCopy, input_selected || output_selected);
+  gtk_widget_set_sensitive(mnuCopy, input_selected || output_selected);
 }
 
 
 void
-menu_cut_activate_cb(GtkWidget *widget,
-                     gpointer data)
+menu_cut_activate_cb(GtkMenuItem *widget, gpointer data)
 {
   simo_combo_box_cut_clipboard(currentWorld->gui->cmbEntry);
 }
 
 
 void
-menu_copy_activate_cb(GtkWidget *widget,
-                      gpointer data)
+menu_copy_activate_cb(GtkMenuItem *widget, gpointer data)
 {
   GtkTextIter start;
   GtkTextIter end;
@@ -1033,23 +1063,21 @@ menu_copy_activate_cb(GtkWidget *widget,
 
 
 void
-menu_paste_activate_cb(GtkWidget *widget,
-                       gpointer data)
+menu_paste_activate_cb(GtkMenuItem *widget, gpointer data)
 {
   simo_combo_box_paste_clipboard(currentWorld->gui->cmbEntry);
 }
 
 
 void
-menu_delete_activate_cb(GtkWidget *widget,
-                        gpointer data)
+menu_delete_activate_cb(GtkMenuItem *widget, gpointer data)
 {
   simo_combo_box_delete_selection(currentWorld->gui->cmbEntry);
 }
 
 
 void
-menu_find_activate_cb(GtkWidget *widget, gpointer data)
+menu_find_activate_cb(GtkMenuItem *widget, gpointer data)
 {
   GtkTextIter search_start;
 
@@ -1082,7 +1110,7 @@ menu_find_activate_cb(GtkWidget *widget, gpointer data)
 
 
 void
-menu_findnext_activate_cb(GtkWidget *widget, gpointer data)
+menu_findnext_activate_cb(GtkMenuItem *widget, gpointer data)
 {
   if (!currentWorld || !currentWorld->gui || !currentWorld->gui->search_box) {
     return;
@@ -1098,8 +1126,7 @@ menu_findnext_activate_cb(GtkWidget *widget, gpointer data)
 
 
 void
-menu_input_clear_cb(GtkWidget *widget,
-                    gpointer data)
+menu_input_clear_cb(GtkMenuItem *widget, gpointer data)
 {
   if (currentWorld) {
     clear_button_cb(NULL, currentWorld->gui);
@@ -1108,7 +1135,7 @@ menu_input_clear_cb(GtkWidget *widget,
 
 
 void
-menu_input_prev_cb(GtkWidget *widget, gpointer data)
+menu_input_prev_cb(GtkMenuItem *widget, gpointer data)
 {
   if (currentWorld) {
     prev_or_next_command(currentWorld, FALSE);
@@ -1117,7 +1144,7 @@ menu_input_prev_cb(GtkWidget *widget, gpointer data)
 
 
 void
-menu_input_next_cb(GtkWidget *widget, gpointer data)
+menu_input_next_cb(GtkMenuItem *widget, gpointer data)
 {
   if (currentWorld) {
     prev_or_next_command(currentWorld, TRUE);
@@ -1126,7 +1153,7 @@ menu_input_next_cb(GtkWidget *widget, gpointer data)
 
 
 void
-menu_input_find_prev_cb(GtkWidget *widget, gpointer data)
+menu_input_find_prev_cb(GtkMenuItem *widget, gpointer data)
 {
   if (currentWorld) {
     find_prev_or_next_command(currentWorld, FALSE);
@@ -1135,7 +1162,7 @@ menu_input_find_prev_cb(GtkWidget *widget, gpointer data)
 
 
 void
-menu_input_find_next_cb(GtkWidget *widget, gpointer data)
+menu_input_find_next_cb(GtkMenuItem *widget, gpointer data)
 {
   if (currentWorld) {
     find_prev_or_next_command(currentWorld, TRUE);
@@ -1145,119 +1172,111 @@ menu_input_find_next_cb(GtkWidget *widget, gpointer data)
 
 
 void
-menu_preferences_open_cb(GtkWidget *widget,
-                         gpointer data)
+menu_preferences_open_cb(GtkMenuItem *widget, gpointer data)
 {
-  GtkToggleAction *mnuDisableTriggers;
-  GtkToggleAction *mnuDisableAliases;
-  GtkToggleAction *mnuDisableMacros;
-  GtkToggleAction *mnuDisableTimers;
+  GtkCheckMenuItem *mnuDisableTriggers;
+  GtkCheckMenuItem *mnuDisableAliases;
+  GtkCheckMenuItem *mnuDisableMacros;
+  GtkCheckMenuItem *mnuDisableTimers;
 
   if (!currentWorld) {
     return;
   }
 
   mnuDisableTriggers
-    = GTK_TOGGLE_ACTION(gtk_builder_get_object(main_builder, "mnuDisTriggers"));
+    = GTK_CHECK_MENU_ITEM(gtk_builder_get_object(main_builder,
+                                                 "mnuDisTriggers"));
   mnuDisableAliases
-    = GTK_TOGGLE_ACTION(gtk_builder_get_object(main_builder, "mnuDisAliases"));
+    = GTK_CHECK_MENU_ITEM(gtk_builder_get_object(main_builder,
+                                                 "mnuDisAliases"));
   mnuDisableMacros
-    = GTK_TOGGLE_ACTION(gtk_builder_get_object(main_builder, "mnuDisMacros"));
+    = GTK_CHECK_MENU_ITEM(gtk_builder_get_object(main_builder,
+                                                 "mnuDisMacros"));
   mnuDisableTimers
-    = GTK_TOGGLE_ACTION(gtk_builder_get_object(main_builder, "mnuDisTimers"));
+    = GTK_CHECK_MENU_ITEM(gtk_builder_get_object(main_builder,
+                                                 "mnuDisTimers"));
 
-  gtk_toggle_action_set_active(mnuDisableTriggers,
-                               currentWorld->disable_triggers);
-  gtk_toggle_action_set_active(mnuDisableAliases,
-                               currentWorld->disable_aliases);
-  gtk_toggle_action_set_active(mnuDisableMacros,
-                               currentWorld->disable_macros);
-  gtk_toggle_action_set_active(mnuDisableTimers,
-                               currentWorld->disable_timers);
+  gtk_check_menu_item_set_active(mnuDisableTriggers,
+                                 currentWorld->disable_triggers);
+  gtk_check_menu_item_set_active(mnuDisableAliases,
+                                 currentWorld->disable_aliases);
+  gtk_check_menu_item_set_active(mnuDisableMacros,
+                                 currentWorld->disable_macros);
+  gtk_check_menu_item_set_active(mnuDisableTimers,
+                                 currentWorld->disable_timers);
 }
 
 
 void
-menu_disable_triggers_cb(GtkWidget *widget,
-                         gpointer data)
+menu_disable_triggers_cb(GtkMenuItem *widget, gpointer data)
 {
   currentWorld->disable_triggers =
-    gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(widget));
+    gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget));
 }
 
 
 void
-menu_disable_aliases_cb(GtkWidget *widget,
-                        gpointer data)
+menu_disable_aliases_cb(GtkMenuItem *widget, gpointer data)
 {
   currentWorld->disable_aliases =
-    gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(widget));
+    gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget));
 }
 
 
 void
-menu_disable_macros_cb(GtkWidget *widget,
-                       gpointer data)
+menu_disable_macros_cb(GtkMenuItem *widget, gpointer data)
 {
   currentWorld->disable_macros =
-    gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(widget));
+    gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget));
 }
 
 
 void
-menu_disable_timers_cb(GtkWidget *widget,
-                       gpointer data)
+menu_disable_timers_cb(GtkMenuItem *widget, gpointer data)
 {
   currentWorld->disable_timers =
-    gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(widget));
+    gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget));
 }
 
 
 void
-mnuDebugMatches_activate_cb(GtkWidget *widget,
-                            gpointer data)
+menu_debug_matches_cb(GtkMenuItem *widget, gpointer data)
 {
-  debug_matches = gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(widget));
+  debug_matches = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget));
 }
 
 
 void
-menu_display_open_cb(GtkWidget *widget,
-                     gpointer data)
+menu_display_open_cb(GtkMenuItem *widget, gpointer data)
 {
-  GtkToggleAction *mnuSplitWindow;
+  GtkCheckMenuItem *mnuSplitWindow;
 
   if (!currentWorld) {
     return;
   }
 
   mnuSplitWindow
-    = GTK_TOGGLE_ACTION(gtk_builder_get_object(main_builder, "mnuSplitWindow"));
+    = GTK_CHECK_MENU_ITEM(gtk_builder_get_object(main_builder,
+                                                 "mnuSplitWindow"));
 
   g_signal_handlers_block_by_func(G_OBJECT(mnuSplitWindow),
                                   menu_split_screen_cb,
                                   NULL);
-  gtk_toggle_action_set_active(mnuSplitWindow,
-                               gtk_paned_get_position(currentWorld->gui->split_pane) > 0);
+  gtk_check_menu_item_set_active(mnuSplitWindow,
+                                 gtk_paned_get_position(currentWorld->gui->split_pane) > 0);
   g_signal_handlers_unblock_by_func(G_OBJECT(mnuSplitWindow),
                                     menu_split_screen_cb,
                                     NULL);
 }
 
 void
-menu_split_screen_cb(GtkAction *action, gpointer data)
+menu_split_screen_cb(GtkMenuItem *widget, gpointer data)
 {
   if (gtk_paned_get_position(currentWorld->gui->split_pane) == 0) {
     if (currentWorld->gui->last_split_position == 0) {
       GtkAllocation allocation;
-
-#if GTK_MINOR_VERSION < 18
-      allocation = GTK_WIDGET(currentWorld->gui->split_pane)->allocation;
-#else
       gtk_widget_get_allocation(GTK_WIDGET(currentWorld->gui->split_pane),
                                 &allocation);
-#endif
-
       currentWorld->gui->last_split_position = allocation.height * .65;
     }
 
