@@ -1,7 +1,5 @@
-/* $Id: net.c 1161 2011-04-25 20:29:35Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +26,7 @@
 #include <time.h>
 #include <errno.h>
 #include <sys/time.h>
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <locale.h>
 #include <gtk/gtk.h>
 #include <zlib.h>
@@ -40,9 +38,6 @@
 #  include <netdb.h>
 #  include <arpa/inet.h>
 #else /* defined __MINGW32__ */
-/* This is so we can use getaddrinfo() et al. The code will only run
-   in Windows XP. */
-#  define _WIN32_WINNT 0x0501
 #  include <winsock2.h>
 #  include <ws2tcpip.h>
 #endif
@@ -57,7 +52,6 @@
  * File global variables *
  *************************/
 #define MAX_US 250000
-static char *terminal_type = "KildClient";
 
 
 /**************
@@ -65,29 +59,6 @@ static char *terminal_type = "KildClient";
  **************/
 
 
-/***************************
- * Telnet protocol defines *
- ***************************/
-#define TELNET_IAC               255        /* 0xFF, 0377 */
-#define TELNET_WILL              251        /* 0xFB, 0373 */
-#define TELNET_WONT              252        /* 0xFC, 0374 */
-#define TELNET_DO                253        /* 0xFD, 0375 */
-#define TELNET_DONT              254        /* 0xFE, 0376 */
-#define TELNET_SB                250        /* 0xFA, 0372 */
-#define TELNET_SE                240        /* 0xF0, 0360 */
-#define TELNET_GA                249        /* 0xF9, 0371 */
-
-#define TELNET_OPT_ECHO          1          /* D'oh */
-#define TELNET_OPT_NAWS          31         /* 0x1f, 0037 */
-
-#define TELNET_OPT_TERMINAL_TYPE 24         /* 0x18, 0030 */
-#define TELNET_TERMTYPE_IS       0
-#define TELNET_TERMTYPE_SEND     1
-
-#define TELNET_OPT_COMPRESS      85         /* 0x55, 0125, "U" */
-#define TELNET_OPT_COMPRESS2     86         /* 0x56, 0126, "V" */
-
-
 /***********************
  * Function prototypes *
  ***********************/
@@ -106,22 +77,11 @@ static gboolean data_ready_cb(GIOChannel   *iochan,
                               GIOCondition  cond,
                               gpointer      data);
 static gboolean flush_buffer(World *world, int action);
-static void     process_recv_line(World *world,
+static void     convert_recv_line(World *world,
                                   int start, int end,
                                   gboolean incomplete);
-static void     process_telnet_iac(World *world, int pos, int end);
-static void     start_mccp(World *world, int pos, int end);
-static void     end_mccp(World *world);
 static ssize_t  write_escaped(World *world, const void *buf, size_t count);
 
-/* Send and receive data */
-#ifdef HAVE_LIBGNUTLS
-static ssize_t kc_recv(World *world, void *buf, size_t s);
-static ssize_t kc_send(World *world, const void *buf, size_t s);
-#else
-#define kc_recv(w, b, s) (recv((w)->sock, (b), (s), 0))
-#define kc_send(w, b, s) (send((w)->sock, (b), (s), 0))
-#endif
 
 #ifndef __MINGW32__
 #  define socket_valid(a) ((a) != -1)
@@ -135,35 +95,26 @@ connect_to(World *world, gchar **errmsg)
 {
   gchar *msg;
   struct resolve_data_s *resolve_data;
+  GThread *resolver_thread;
 
 #ifndef HAVE_LIBGNUTLS
   /* Oops, we cannot do that */
   if (world->use_tls) {
-    GtkWidget *msgdlg;
-    GtkWidget *dlgbutton;
-    GtkWidget *dlgbuttonimage;
-
-    msgdlg = gtk_message_dialog_new(GTK_WINDOW(world->dlgEditWorld),
-                                    GTK_DIALOG_MODAL,
-                                    GTK_MESSAGE_QUESTION,
-                                    GTK_BUTTONS_NONE,
-                                    _("You're attempting to connect to a world that uses SSL, but this version of KildClient has been built without SSL support. Try to connect without SSL?"));
+    GtkDialog *msgdlg = msgdlg = gtk_message_dialog_new(GTK_WINDOW(world->dlgEditWorld),
+                                                        GTK_DIALOG_MODAL,
+                                                        GTK_MESSAGE_QUESTION,
+                                                        GTK_BUTTONS_NONE,
+                                                        _("You're attempting to connect to a world that uses SSL, but this version of KildClient has been built without SSL support. Try to connect without SSL?"));
     gtk_window_set_title(GTK_WINDOW(msgdlg), _("KildClient Question"));
-    gtk_dialog_add_button(GTK_DIALOG(msgdlg),
-                          _("Connect without SSL"), GTK_RESPONSE_YES);
-    dlgbutton = gtk_button_new_with_label(_("Do not connect"));
-    dlgbuttonimage = gtk_image_new_from_stock(GTK_STOCK_CANCEL,
-                                              GTK_ICON_SIZE_MENU);
-    gtk_button_set_image(GTK_BUTTON(dlgbutton), dlgbuttonimage);
-    gtk_widget_show_all(dlgbutton);
-    gtk_dialog_add_action_widget(GTK_DIALOG(msgdlg),
-                                 dlgbutton, GTK_RESPONSE_NO);
-    GTK_WIDGET_SET_FLAGS(dlgbutton, GTK_CAN_DEFAULT);
-    gtk_dialog_set_default_response(GTK_DIALOG(msgdlg), GTK_RESPONSE_NO);
-    //gtk_widget_grab_default(dlgbutton);
-  if (gtk_dialog_run(GTK_DIALOG(msgdlg)) == GTK_RESPONSE_YES) {
+    gtk_dialog_add_buttons(msgdlg,
+                           _("Connect without SSL"), GTK_RESPONSE_YES,
+                           _("Do not connect"), GTK_RESPONSE_NO,
+                           NULL);
+    gtk_dialog_set_default_response(msgdlg, GTK_RESPONSE_NO);
+
+    if (gtk_dialog_run(msgdlg) == GTK_RESPONSE_YES) {
       world->use_tls = FALSE;
-      gtk_widget_destroy(msgdlg);
+      gtk_widget_destroy(GTK_WIDGET(msgdlg));
     } else {
       *errmsg = g_strdup(_("This version of KildClient does not have SSL support."));
       gtk_widget_destroy(msgdlg);
@@ -200,11 +151,17 @@ connect_to(World *world, gchar **errmsg)
   world_for_perl = world;
   if (!world->connected_at_least_once) {
     init_perl_script(world);
+
+    init_threads(world);
+
+    GRAB_PERL(world);
     load_permanent_variables(world);
     load_startup_plugins(world);
     if (world->scriptfile) {
-      perl_script_import_file(world->perl_interpreter, world->scriptfile);
+      perl_script_import_file(world->scriptfile);
     }
+    RELEASE_PERL(world);
+
     load_command_history(world);
   } else {
     if (world->has_unread_text) {
@@ -261,6 +218,11 @@ connect_to(World *world, gchar **errmsg)
     resolve_data->port     = g_strdup(world->proxy_used.port);
     resolve_data->callback = continue_connecting_cb;
     resolve_data->ai_family = AF_UNSPEC;
+  } else if (world->proxy_used.type == PROXY_HTTP) {
+    resolve_data->host     = g_strdup(world->proxy_used.server);
+    resolve_data->port     = g_strdup(world->proxy_used.port);
+    resolve_data->callback = continue_connecting_cb;
+    resolve_data->ai_family = AF_UNSPEC;
   } else {
     resolve_data->host     = g_strdup(world->host);
     resolve_data->port     = g_strdup(world->port);
@@ -268,10 +230,9 @@ connect_to(World *world, gchar **errmsg)
     resolve_data->ai_family = AF_UNSPEC;
   }
 
-  if (!g_thread_create(resolve_name_thread, resolve_data, FALSE, NULL)) {
-    *errmsg = g_strdup(_("Could not start name-resolution thread."));
-    return FALSE;
-  }
+  resolver_thread = g_thread_new("resolver",
+                                 resolve_name_thread, resolve_data);
+  g_thread_unref(resolver_thread);
 
   return TRUE;
 }
@@ -313,19 +274,25 @@ continue_connecting_cb(gpointer data)
 
   g_free(resolve_data->port);
 
-  if (resolve_data->ret != 0) {
+  if (g_list_index(open_worlds, world) == -1) {
     /* It could happen that the world has been closed */
-    if (g_list_index(open_worlds, world) != -1) {
-      gchar *msg;
-      gsize  len;
+    g_free(resolve_data->host);
+    g_free(resolve_data);
+
+    return FALSE;
+  }
+
+  if (resolve_data->ret != 0) {
+    gchar *msg;
+    gsize  len;
+
+    msg = g_locale_to_utf8(gai_strerror(resolve_data->ret), -1,
+                           NULL, &len, NULL);
+    world->errmsg = g_strdup_printf(_("Could not resolve host %s: %s"),
+                                    resolve_data->host, msg);
+    g_free(msg);
+    disconnect_world(world, FALSE);
 
-      msg = g_locale_to_utf8(gai_strerror(resolve_data->ret), -1,
-                             NULL, &len, NULL);
-      world->errmsg = g_strdup_printf(_("Could not resolve host %s: %s"),
-                                      resolve_data->host, msg);
-      g_free(msg);
-      disconnect_world(world, FALSE);
-    }
     g_free(resolve_data->host);
     g_free(resolve_data);
 
@@ -363,9 +330,6 @@ continue_connecting_cb(gpointer data)
   gtk_label_set_text(world->gui->lblStatus, msg);
   g_free(msg);
 
-  while (gtk_events_pending())
-    gtk_main_iteration();
-
   world->sock = socket(world->addrinfo->ai_family,
                        world->addrinfo->ai_socktype,
                        world->addrinfo->ai_protocol);
@@ -547,6 +511,8 @@ connected_to_dispatcher(World *world)
     return proxy_socks4_connected_to(world);
   } else if (world->proxy_used.type == PROXY_SOCKS5) {
     return proxy_socks5_connected_to(world);
+  } else if (world->proxy_used.type == PROXY_HTTP) {
+    return proxy_http_connected_to(world);
   } else {
     return perform_tls_handshake(world);
   }
@@ -569,10 +535,10 @@ perform_tls_handshake(World *world)
 
 #ifndef __x86_64      /* This is to remove a warning about pointer sizes */
     gnutls_transport_set_ptr(world->tls_session,
-                             (gnutls_transport_ptr) world->sock);
+                             (gnutls_transport_ptr_t) world->sock);
 #else
     gnutls_transport_set_ptr(world->tls_session,
-                             (gnutls_transport_ptr) (long) world->sock);
+                             (gnutls_transport_ptr_t) (long) world->sock);
 #endif
     /* Register callback to perform the TLS handshake
      * The first action is we sending data (ClientHello) */
@@ -1012,7 +978,7 @@ data_ready_cb(GIOChannel   *iochan,
     while (nlptr) {
       nlpos = nlptr - world->pbuffer;
 
-      process_recv_line(world, start, nlpos + 1, FALSE);
+      convert_recv_line(world, start, nlpos + 1, FALSE);
       /* We set printed_until to zero to indicate that a full line
          has been printed and there is no more pending text. */
       world->printed_until = 0;
@@ -1032,11 +998,6 @@ data_ready_cb(GIOChannel   *iochan,
      compressed, the end condition will be always true, and the loop
      will run forever, until stopped by the break condition above. */
 
-  /* Let other things happen before more data comes, including drawing
-     the just received lines. */
-  while (gtk_events_pending())
-    gtk_main_iteration();
-
   /* Set up a timeout to print incomplete lines if no more data arrives. */
   if (world->pbuf_size && !world->timeout_id) {
     world->timeout_id = g_timeout_add_full(G_PRIORITY_LOW,
@@ -1078,7 +1039,7 @@ flush_buffer(World *world, int action)
   /* Called when a command is sent, the line must be marked as new,
      even if there is nothing new. */
   if (action && world->pbuf_size) {
-    process_recv_line(world, 0, world->pbuf_size,
+    convert_recv_line(world, 0, world->pbuf_size,
                       FALSE);
     world->printed_until = 0;
     world->pbuf_size = 0;
@@ -1097,7 +1058,7 @@ flush_buffer(World *world, int action)
 
   /* action = 1 is when flushing because a command is sent. We should
      consider lines as finished in this case. */
-  process_recv_line(world, world->printed_until, world->pbuf_size,
+  convert_recv_line(world, world->printed_until, world->pbuf_size,
                     TRUE);
   world->printed_until = world->pbuf_size;
 
@@ -1107,354 +1068,37 @@ flush_buffer(World *world, int action)
 
 static
 void
-process_recv_line(World *world, int start, int end, gboolean incomplete)
+convert_recv_line(World *world, int start, int end, gboolean incomplete)
 {
-  guchar      *converted;
-  gsize        output_len;
-  guchar      *stripped = NULL;
-  GError      *error = NULL;
-  gboolean     gag_output = FALSE;
-  gboolean     gag_log    = FALSE;
-  gint         dummy;
-  GtkTextIter  line_start;
-  GtkTextIter  line_end;
+  gchar  *converted;
+  gsize   output_len;
+  GError *error = NULL;
 
   /* Clear pending line if it gets completed */
   if (!incomplete && world->printed_until) {
-    gtk_text_buffer_get_iter_at_mark(world->gui->txtBuffer,
-                                     &line_start,
-                                     world->gui->txtmark_linestart);
-    gtk_text_buffer_get_iter_at_mark(world->gui->txtBuffer,
-                                     &line_end,
-                                     world->gui->txtmark_end);
-    gtk_text_buffer_delete(world->gui->txtBuffer,
-                           &line_start, &line_end);
-
-    /* Since an ANSI sequence cannot be incomplete at the end of
-       one line and continue at another (because \n is not valid
-       inside ANSI strings), ANSI parsing at a line always starts
-       in DATA mode. */
-    world->gui->ta.ansifsm_state  = ANSIFSM_DATA;
-    world->gui->ta.ansiseq_pos = 0;
-    /* Furthermore, colors are restored to what they were in the
-       end of the last complete line. */
-    world->gui->ta.state = world->gui->ta.saved_state;
+    AnsiParseOperation *operation = g_new(AnsiParseOperation, 1);
+    operation->action = CLEAR_PENDING_LINE_ANSI;
+    queue_ansiparse_operation(world, operation);
   }
 
-  converted = (guchar *) g_convert((gchar *) (world->pbuffer + start),
-                                   end - start,
-                                   VTE_ENC, world->charset,
-                                   NULL, &output_len,
-                                   &error);
+  converted = g_convert((gchar *) (world->pbuffer + start),
+                        end - start,
+                        VTE_ENC, world->charset,
+                        NULL, &output_len,
+                        &error);
   if (converted) {
-    if (!incomplete) {
-      stripped = strip_ansi(converted, output_len);
-
-      match_triggers(world,
-                     &stripped, &converted, &output_len,
-                     &gag_output, &gag_log,
-                     &dummy, NULL);
-
-      /* The line is written to the log file in match_triggers. */
-    } else {
-      ansitextview_append_ansi_string(world->gui,
-                                      (gchar *) converted, output_len);
-    }
-
-    if (!gag_output) {
-      /* The actual printing of the line is done in match_triggers. */
-      if (!world->has_focus) {
-        if (!world->has_unread_text) {
-          world->has_unread_text = TRUE;
-          gtk_label_set_markup(world->gui->lblNotebook, world->new_text_name);
-          if (worlds_with_new_text != -1) {
-            ++worlds_with_new_text;
-
-            if (!window_has_focus) {
-              adjust_window_title();
-            }
-          }
-        }
-      }
-      if (!incomplete) {
-        execute_hook(world, "OnReceivedText", (gchar *) converted);
-      }
-    }
-    g_free(converted);
+    ProcessLineOperation *operation = g_new(ProcessLineOperation, 1);
+    operation->line = converted;
+    operation->len = output_len;
+    operation->action = incomplete ? PROCESS_INCOMPLETE_LINE
+                                   : PROCESS_COMPLETE_LINE;
+    queue_process_line_operation(world, operation);
   } else {
     fprintf(stderr, "Error in charset conversion: %s\n", error->message);
   }
-
-  g_timer_start(world->last_data_written);
 }
 
 
-static
-void
-process_telnet_iac(World *world, int pos, int end)
-{
-  /* This function processes Telnet IAC sequences and converts the
-     newlines to the canonical \n. */
-  unsigned char iac_response[4];
-  time_t        now;
-
-  while (pos < end) {
-    switch (world->telnet_state) {
-    case TELNETFSM_DATA:
-      switch (world->dbuffer[pos]) {
-      case TELNET_IAC:
-        world->telnet_state = TELNETFSM_IAC;
-        break;
-
-      case '\n':
-        world->pbuffer[world->pbuf_size++] = world->dbuffer[pos];
-        world->telnet_state = TELNETFSM_DATA_NL;
-        break;
-
-      case '\r':
-        world->telnet_state_ncrs = 1;
-        world->telnet_state = TELNETFSM_DATA_CR;
-        break;
-
-      case '\b': /* Backspace */
-        if (world->pbuf_size > 0
-            && world->pbuffer[world->pbuf_size - 1] != '\n') {
-          --world->pbuf_size;
-        }
-        break;
-
-      default:
-        world->pbuffer[world->pbuf_size++] = world->dbuffer[pos];
-      }
-      break;
-
-    case TELNETFSM_DATA_NL:
-      if (world->dbuffer[pos] != '\r' && world->dbuffer[pos] != '\b') {
-        world->pbuffer[world->pbuf_size++] = world->dbuffer[pos];
-        if (world->dbuffer[pos] != '\n') {
-          world->telnet_state = TELNETFSM_DATA;
-        }
-      } else {
-        world->telnet_state = TELNETFSM_DATA;
-      }
-      break;
-
-    case TELNETFSM_DATA_CR:
-      if (world->dbuffer[pos] == '\n') {
-        world->pbuffer[world->pbuf_size++] = world->dbuffer[pos];
-        world->telnet_state = TELNETFSM_DATA;
-      } else if (world->dbuffer[pos] == '\r') {
-        ++world->telnet_state_ncrs;
-      } else {
-        int i;
-
-        /* Add the newlines. But check if there is space before. */
-        if (world->pbuf_size + world->telnet_state_ncrs >=
-            world->pbuf_alloc_size) {
-          world->pbuf_alloc_size += world->telnet_state_ncrs;
-          world->pbuffer = g_realloc(world->pbuffer,
-                                     world->pbuf_alloc_size);
-        }
-
-        for (i = 0; i < world->telnet_state_ncrs; ++i) {
-          world->pbuffer[world->pbuf_size++] = '\n';
-        }
-        world->pbuffer[world->pbuf_size++] = world->dbuffer[pos];
-        world->telnet_state = TELNETFSM_DATA;
-      }
-      break;
-
-    case TELNETFSM_IAC:
-      switch (world->dbuffer[pos]) {
-      case TELNET_IAC:
-        world->pbuffer[world->pbuf_size++] = TELNET_IAC;
-        world->telnet_state = TELNETFSM_DATA;
-        break;
-
-      case TELNET_DO:
-        world->telnet_state = TELNETFSM_DO;
-        break;
-
-      case TELNET_DONT:
-        world->telnet_state = TELNETFSM_DONT;
-        break;
-
-      case TELNET_WILL:
-        world->telnet_state = TELNETFSM_WILL;
-        break;
-
-      case TELNET_WONT:
-        world->telnet_state = TELNETFSM_WONT;
-        break;
-
-      case TELNET_SB:
-        world->telnet_state = TELNETFSM_SB;
-        break;
-
-      default:
-        world->telnet_state = TELNETFSM_DATA;
-        break;
-      }
-      break;
-
-    case TELNETFSM_DO:
-      switch (world->dbuffer[pos]) {
-      case TELNET_OPT_TERMINAL_TYPE:
-        iac_response[0] = TELNET_IAC;
-        iac_response[1] = TELNET_WILL;
-        iac_response[2] = TELNET_OPT_TERMINAL_TYPE;
-        kc_send(world, iac_response, 3);
-        break;
-
-      case TELNET_OPT_NAWS:
-        iac_response[0] = TELNET_IAC;
-        iac_response[1] = TELNET_WILL;
-        iac_response[2] = TELNET_OPT_NAWS;
-        kc_send(world, iac_response, 3);
-        world->use_naws = TRUE;
-        send_naws_size(world);
-      }
-      world->telnet_state = TELNETFSM_DATA;
-      break;
-
-    case TELNETFSM_DONT:
-      switch (world->dbuffer[pos]) {
-      case TELNET_OPT_NAWS:
-        world->use_naws = FALSE;
-        break;
-      }
-      world->telnet_state = TELNETFSM_DATA;
-      break;
-
-    case TELNETFSM_WILL:
-      switch (world->dbuffer[pos]) {
-      case TELNET_OPT_ECHO:
-        world->noecho = TRUE;
-        if (!world->never_hide_input) {
-          simo_combo_box_set_visibility(world->gui->cmbEntry, FALSE);
-        }
-        iac_response[0] = TELNET_IAC;
-        iac_response[1] = TELNET_DO;
-        iac_response[2] = TELNET_OPT_ECHO;
-        kc_send(world, iac_response, 3);
-        break;
-
-      case TELNET_OPT_COMPRESS2:
-        now = time(NULL);
-        if (world->mccp_behavior == MCCP_ALWAYS
-            || (world->mccp_behavior == MCCP_AFTER_CONNECT
-                && difftime(now, world->connected_time) <= 60)) {
-          world->mccp_ver = 2;
-          iac_response[0] = TELNET_IAC;
-          iac_response[1] = TELNET_DO;
-          iac_response[2] = TELNET_OPT_COMPRESS2;
-          kc_send(world, iac_response, 3);
-        }
-        break;
-
-      case TELNET_OPT_COMPRESS:
-        /* Do nothing if version 2 is supported */
-        now = time(NULL);
-        if (world->mccp_ver <= 1
-            && (world->mccp_behavior == MCCP_ALWAYS
-                || (world->mccp_behavior == MCCP_AFTER_CONNECT
-                    && difftime(now, world->connected_time) <= 60))) {
-          world->mccp_ver = 1;
-          iac_response[0] = TELNET_IAC;
-          iac_response[1] = TELNET_DO;
-          iac_response[2] = TELNET_OPT_COMPRESS;
-          kc_send(world, iac_response, 3);
-        }
-        break;
-      }
-      world->telnet_state = TELNETFSM_DATA;
-      break;
-
-    case TELNETFSM_WONT:
-      if (world->dbuffer[pos] == TELNET_OPT_ECHO) {
-        world->noecho = FALSE;
-        simo_combo_box_set_visibility(world->gui->cmbEntry, TRUE);
-        iac_response[0] = TELNET_IAC;
-        iac_response[1] = TELNET_DONT;
-        iac_response[2] = TELNET_OPT_ECHO;
-        kc_send(world, iac_response, 3);
-      }
-      world->telnet_state = TELNETFSM_DATA;
-      break;
-
-    case TELNETFSM_SB:
-      if (world->dbuffer[pos] == TELNET_IAC) {
-        world->telnet_state = TELNETFSM_SB_IAC;
-      } else if (world->dbuffer[pos] == TELNET_OPT_TERMINAL_TYPE) {
-        world->telnet_state = TELNETFSM_SB_IAC_TERMTYPE;
-      } else if (world->dbuffer[pos] == TELNET_OPT_COMPRESS) {
-        world->telnet_state = TELNETFSM_SB_IAC_COMPRESS;
-      } else if (world->dbuffer[pos] == TELNET_OPT_COMPRESS2) {
-        world->telnet_state = TELNETFSM_SB_IAC_COMPRESS2;
-      }
-      break;
-
-    case TELNETFSM_SB_IAC:
-      if (world->dbuffer[pos] == TELNET_SE) {
-        world->telnet_state = TELNETFSM_DATA;
-      } else {
-        world->telnet_state = TELNETFSM_SB;
-      }
-      break;
-
-    case TELNETFSM_SB_IAC_TERMTYPE:
-      if (world->dbuffer[pos] == TELNET_TERMTYPE_SEND) {
-        char *tmp;
-
-        tmp = g_strdup_printf("%c%c%c%c%s%c%c",
-                              TELNET_IAC, TELNET_SB,
-                              TELNET_OPT_TERMINAL_TYPE, TELNET_TERMTYPE_IS,
-                              terminal_type,
-                              TELNET_IAC,
-                              TELNET_SE);
-        kc_send(world, tmp, strlen(terminal_type) + 7);
-        g_free(tmp);
-      }
-      world->telnet_state = TELNETFSM_SB;
-      break;
-
-    case TELNETFSM_SB_IAC_COMPRESS:
-      if (world->dbuffer[pos] == TELNET_WILL) {
-        world->telnet_state = TELNETFSM_SB_IAC_COMPRESS_WILL;
-      } else {
-        /* This shouldn't happen, and if it does, it will probably mess
-           up everything. But let's hope for the best and wait for an SE */
-        world->telnet_state = TELNETFSM_SB_IAC;
-      }
-      break;
-
-    case TELNETFSM_SB_IAC_COMPRESS2:
-      if (world->dbuffer[pos] == TELNET_IAC) {
-        world->telnet_state = TELNETFSM_SB_IAC_COMPRESS_WILL;
-      } else {
-        /* This shouldn't happen, and if it does, it will probably mess
-           up everything. But let's hope for the best and wait for an SE */
-        world->telnet_state = TELNETFSM_SB_IAC;
-      }
-      break;
-
-    case TELNETFSM_SB_IAC_COMPRESS_WILL:
-      if (world->dbuffer[pos] == TELNET_SE) {
-        start_mccp(world, pos, end);
-        world->telnet_state = TELNETFSM_DATA;
-        return;
-      } else {
-      /* This shouldn't happen, and if it does, it will probably mess
-         up everything. But let's hope for the best and wait for an SE */
-        world->telnet_state = TELNETFSM_SB_IAC;
-      }
-    }
-    ++pos;
-  }
-}
-
-
-static
 void
 start_mccp(World *world, int pos, int end)
 {
@@ -1496,7 +1140,6 @@ start_mccp(World *world, int pos, int end)
 }
 
 
-static
 void
 end_mccp(World *world)
 {
@@ -1535,9 +1178,7 @@ send_to_world(World *world, const char *cmd, gint len)
   gboolean  has_incomplete_line;
 
   has_incomplete_line = world->printed_until;
-  if (!world->dont_flush) {
-    flush_buffer(world, 1);
-  }
+  flush_buffer(world, 1);
 
   locale_cmd = g_convert(cmd, len,
                          world->charset, VTE_ENC,
@@ -1635,7 +1276,7 @@ send_to_world_no_check(World      *world,
     }
   }
 
-  execute_hook(world, "OnSentCommand", utf8_command);
+  execute_hook_immediately(world, "OnSentCommand", utf8_command);
 
   if (needs_freeing) {
     g_free(locale_command);
@@ -1727,65 +1368,7 @@ write_escaped(World *world, const void *buf, size_t count)
 }
 
 
-void
-send_naws_size(World *world)
-{
-  /* 3 for start, 2 for end. Each value is 2 bytes, but may be 4
-     if it happens to be 0xFFFF and must be IAC escaped. Thus 8 for both
-     values. */
-  guchar   buffer[3 + 8 + 2];
-  int      bsize;
-  guint16  size;
-  guint    cols;
-  guint    rows;
-
-  ansitextview_get_size(world->gui, &rows, &cols);
-
-  buffer[0] = TELNET_IAC;
-  buffer[1] = TELNET_SB;
-  buffer[2] = TELNET_OPT_NAWS;
-  bsize = 3;
-
-  size = htons((guint16) cols);
-  memcpy(buffer + bsize, &size, sizeof(size));
-  /* Escape IAC */
-  if (buffer[bsize] == TELNET_IAC) {
-    buffer[bsize+2] = buffer[bsize+1];
-    buffer[bsize+1] = TELNET_IAC;
-    ++bsize;
-  }
-  ++bsize;
-  if (buffer[bsize] == TELNET_IAC) {
-    buffer[bsize+2] = buffer[bsize+1];
-    buffer[bsize+1] = TELNET_IAC;
-    ++bsize;
-  }
-  ++bsize;
-
-  size = htons((guint16) rows);
-  memcpy(buffer + bsize, &size, sizeof(size));
-  /* Escape IAC */
-  if (buffer[bsize] == TELNET_IAC) {
-    buffer[bsize+2] = buffer[bsize+1];
-    buffer[bsize+1] = TELNET_IAC;
-    ++bsize;
-  }
-  ++bsize;
-  if (buffer[bsize] == TELNET_IAC) {
-    buffer[bsize+2] = buffer[bsize+1];
-    buffer[bsize+1] = TELNET_IAC;
-    ++bsize;
-  }
-  ++bsize;
-
-  buffer[bsize++] = TELNET_IAC;
-  buffer[bsize++] = TELNET_SE;
-  kc_send(world, buffer, bsize);
-}
-
-
 #ifdef HAVE_LIBGNUTLS
-static
 ssize_t
 kc_recv(World *world, void *buf, size_t s)
 {
@@ -1797,7 +1380,6 @@ kc_recv(World *world, void *buf, size_t s)
 }
 
 
-static
 ssize_t
 kc_send(World *world, const void *buf, size_t s)
 {
