@@ -1,7 +1,5 @@
-/* $Id: parser.c 1143 2011-02-13 16:13:58Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +24,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <locale.h>
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <gtk/gtk.h>
 
 /* Perl includes */
@@ -51,26 +49,90 @@ static void process_command(World *world, const char *cmd, int len);
 
 
 void
-parse_commands(World *world, const char *cmdline, int totallen)
+parse_commands(World *world, const char *cmdline)
 {
+  parse_commands_full(world, cmdline, NULL, NULL, FALSE);
+}
+
+
+void
+parse_commands_with_matches_and_notify(World      *world,
+                                       const char *cmdline,
+                                       AV         *matches)
+{
+  parse_commands_full(world, cmdline, matches, NULL, TRUE);
+}
+
+
+void
+parse_commands_full(World       *world,
+                    const gchar *cmdline,
+                    AV          *matches,
+                    const gchar *colorline,
+                    gboolean     notify)
+{
+  MainThreadOperation *operation = g_new0(MainThreadOperation, 1);
+  operation->action              = PARSE_COMMANDS;
+  operation->commands.commands   = g_strdup(cmdline);
+  operation->commands.matches    = matches;
+  operation->commands.colorline  = g_strdup(colorline);
+  operation->commands.notify     = notify;
+
+  queue_mainthread_operation(world, operation);
+}
+
+
+void
+mainthread_op_parse_commands(World *world, MainThreadOperation *operation)
+{
+  gboolean needs_perl = operation->commands.colorline
+                        || operation->commands.matches;
+
+  if (needs_perl) {
+    GRAB_PERL(world);
+  }
+
+  if (operation->commands.colorline) {
+    SV *Pcolorline;
+
+    Pcolorline = get_sv("colorline", TRUE);
+    sv_setpv(Pcolorline, operation->commands.colorline);
+  }
+
+  if (operation->commands.matches) {
+    perl_set_arg_array(operation->commands.matches);
+  }
+
+  parse_command_line_immediately(world, operation->commands.commands);
+
+  if (needs_perl) {
+    RELEASE_PERL(world);
+  }
+
+  /* Notify waiting match trigger thread */
+  if (operation->commands.notify) {
+    g_mutex_lock(&world->wait_command_mutex);
+    world->command_executed = TRUE;
+    g_cond_signal(&world->wait_command_cond);
+    g_mutex_unlock(&world->wait_command_mutex);
+  }
+}
+
+
+void
+parse_command_line_immediately(World *world, const char *cmdline)
+{
+  int      totallen;
   int      seppos;
   int      start;
   int      end;
   gboolean quit;
   int      cmdseplen;
 
+  totallen = strlen(cmdline);
   cmdseplen = strlen(world->command_separator);
 
   start = 0;
-  /* Skip spaces */
-  while (start < totallen) {
-    if (isspace(cmdline[start])) {
-      start++;
-    } else {
-      break;
-    }
-  }
-
   quit = FALSE;
   do {
     seppos = find_next_command_separator(world, cmdline + start);
@@ -155,7 +217,7 @@ process_aliases(World *world, const char *cmdline, int start, int end)
 
   result = substitute_aliases(world, cmd);
   if (result) {
-    parse_commands(world, result, strlen(result));
+    parse_command_line_immediately(world, result);
     g_free(result);
   } else {
     process_command(world, cmd, len);
@@ -176,7 +238,7 @@ process_command(World *world, const char *cmd, int len)
       SV *commands;
       SV *error;
 
-      PERL_SET_CONTEXT(world->perl_interpreter);
+      GRAB_PERL(world);
       commands = newSVpv(cmd + 1, len - 1);
       eval_sv(commands, TRUE | G_EVAL);
       error = get_sv("@", TRUE);
@@ -184,6 +246,7 @@ process_command(World *world, const char *cmd, int len)
         ansitextview_append_stringf(world->gui,
                                     "Perl Error: %s", SvPV_nolen(error));
       }
+      RELEASE_PERL(world);
     }
   }
 }
