@@ -1,7 +1,5 @@
-/* $Id: perlscript.c 1176 2011-05-28 14:43:45Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +21,7 @@
 #  include <kcconfig.h>
 #endif
 
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <time.h>
 #include <locale.h>
 #include <gtk/gtk.h>
@@ -36,6 +34,7 @@
 #include "kildclient.h"
 #include "ansi.h"
 #include "perlscript.h"
+#include "net.h"
 
 
 /*****************
@@ -58,6 +57,8 @@
  ***********************/
 static GSList *AVtoGSList(AV *array);
 static SV     *create_world_var(World *world);
+static AV     *perl_clone_array(AV *orig);
+static void    perl_copy_array(AV *source, AV *dest);
 static SV     *create_window_var(void);
 static World  *fetch_world(int items, SV *self_ref, HV **self_to_store);
 static void    xs_init();
@@ -79,9 +80,8 @@ World *world_for_perl = NULL;
 /*************************
  * File global variables *
  *************************/
-static int   my_argc_static = 4;
+static int   my_argc_static = 3;
 static char *my_argv_static[] = { "kildperl",
-                                  "-Mencoding=utf8",
                                   "-MData::Dumper",
                                   "-w" };
 
@@ -96,11 +96,11 @@ init_perl_system(void)
 void
 init_perl_script(World *world)
 {
+  int    my_argc;
   char **my_argv;
   char  *perl_cmds;
   char  *system_plugin_dir;
   char  *user_plugin_dir;
-  int    i;
 
   SV *world_ref;
   SV *world_var;
@@ -108,11 +108,14 @@ init_perl_script(World *world)
   SV *window_var;
 
   my_argv = g_malloc(sizeof(char *) * (my_argc_static + 2));
-  for (i = 0; i < my_argc_static; ++i) {
-    my_argv[i] = my_argv_static[i];
+  for (my_argc = 0; my_argc < my_argc_static; ++my_argc) {
+    my_argv[my_argc] = my_argv_static[my_argc];
   }
-  my_argv[i] = g_strdup_printf("-I%s", get_kildclient_installed_file(""));
-  my_argv[i+1] = g_strdup(get_kildclient_installed_file("kildclient.pl"));
+  my_argv[my_argc++] = g_strdup_printf("-I%s", get_kildclient_installed_file(""));
+#ifdef __WIN32__
+  my_argv[my_argc++] = g_strdup_printf("-I%s", get_kildclient_installed_file("perl"));
+#endif
+  my_argv[my_argc] = g_strdup(get_kildclient_installed_file("kildclient.pl"));
 
 #ifndef __WIN32__
   user_plugin_dir = g_strdup_printf("%s/plugins",
@@ -139,7 +142,7 @@ init_perl_script(World *world)
   perl_construct(world->perl_interpreter);
 
   perl_parse(world->perl_interpreter, xs_init,
-             my_argc_static + 2, my_argv, NULL);
+             my_argc, my_argv, NULL);
   /* PL_exit_flags |= PERL_EXIT_DESTRUCT_END; */
   eval_pv(perl_cmds, TRUE);
 
@@ -153,8 +156,11 @@ init_perl_script(World *world)
   window_var = get_sv("window", TRUE);
   sv_setsv(window_var, window_ref);
 
-  g_free(my_argv[i]);
-  g_free(my_argv[i+1]);
+  g_free(my_argv[my_argc]);
+  g_free(my_argv[my_argc - 1]);
+#ifdef __WIN32__
+  g_free(my_argv[my_argc - 2]);
+#endif
   g_free(my_argv);
   g_free(perl_cmds);
 }
@@ -169,6 +175,7 @@ create_world_var(World *world)
   HV *world_hash;
   IV  worldptriv;
   SV *worldptrsv;
+  SV *silentsv;
   SV *world_ref;
   HV *world_pkg_stash;
 
@@ -178,6 +185,9 @@ create_world_var(World *world)
   worldptrsv = newSViv(worldptriv);
   (void) hv_store(world_hash, "WORLDPTR", 8, worldptrsv, 0);
 
+  silentsv = newSViv(0);
+  (void) hv_store(world_hash, "SILENT", 6, silentsv, 0);
+
   world_ref = newRV_noinc((SV *) world_hash);
   world_pkg_stash = gv_stashpv("World", TRUE);
   world_ref = sv_bless(world_ref, world_pkg_stash);
@@ -219,22 +229,21 @@ destruct_perl_script(PerlInterpreter *perl_interpreter)
 
 
 void
-perl_script_import_file(PerlInterpreter *interpreter, const char *file)
+perl_script_import_file(const char *file)
 {
   char *cmd = g_strdup_printf("do '%s';", file);
-  PERL_SET_CONTEXT(interpreter);
   eval_pv(cmd, TRUE);
   g_free(cmd);
 }
 
 
 int
-perl_match(PerlInterpreter     *interpreter,
-           const unsigned char *line,
-           SV                  *pattern_re,
-           int                 *match_start,
-           int                 *match_end,
-           int                  substring)
+perl_match(const char      *line,
+           SV              *pattern_re,
+           int             *match_start,
+           int             *match_end,
+           int              substring,
+           AV             **matches)
 {
   SV  *string;
   SV  *re;
@@ -244,11 +253,9 @@ perl_match(PerlInterpreter     *interpreter,
   int  end;
   dSP;
 
-  PERL_SET_CONTEXT(interpreter);
-
   /* Set the string in a variable */
   string = get_sv("triggerline", TRUE);
-  sv_setpv(string, (char *) line);
+  sv_setpv(string, line);
   SvUTF8_on(string);
   /* Set the regexp in a variable */
   re = get_sv("_re", TRUE);
@@ -256,11 +263,10 @@ perl_match(PerlInterpreter     *interpreter,
   /* Create the command to run */
   command = newSVpvf("$triggerline =~ tr/\\n\\r//d;"
                      "my $target = %d;"
-                     "my @__M = ($triggerline =~ m\x05$_re\x05);"
+                     "@__M = ($triggerline =~ m\x05$_re\x05);"
                      "my $match = scalar @__M;"
                      "if ($match) {"
-                     "  @_ = @__M;"
-                     "  unshift(@_, $triggerline);"
+                     "  unshift(@__M, $triggerline);"
                      "}"
                      "(defined($-[$target]) ? $-[$target] : 0,"
                      " defined($+[$target]) ? $+[$target] : 0,"
@@ -276,6 +282,16 @@ perl_match(PerlInterpreter     *interpreter,
   retval = POPi;
   end    = POPi;
   start  = POPi;
+
+  if (matches) {
+    if (retval) {
+      AV *M = get_av("__M", 0);
+      *matches = perl_clone_array(M);
+    } else {
+      *matches = NULL;
+    }
+  }
+
   PUTBACK;
   FREETMPS;
   LEAVE;
@@ -294,8 +310,7 @@ perl_match(PerlInterpreter     *interpreter,
 
 
 char *
-perl_substitute(PerlInterpreter *interpreter,
-                const char      *line,
+perl_substitute(const char      *line,
                 SV              *pattern_re,
                 const char      *substitution,
                 int              perl_eval,
@@ -308,8 +323,6 @@ perl_substitute(PerlInterpreter *interpreter,
   SV  *result;
   dSP;
 
-  PERL_SET_CONTEXT(interpreter);
-
   /* Set the string in a variable */
   string = get_sv("aliascmd", TRUE);
   sv_setpv(string, line);
@@ -346,19 +359,14 @@ perl_substitute(PerlInterpreter *interpreter,
 
 
 SV *
-precompute_re(PerlInterpreter *interpreter,
-              const char      *re_string,
+precompute_re(const char      *re_string,
               int              ignore_case)
 {
-  PerlInterpreter *old_interpreter;
   SV              *re_int;
   SV              *re_new;
   SV              *command;
   dSP;
 
-  old_interpreter = PERL_GET_CONTEXT;
-  PERL_SET_CONTEXT(interpreter);
-
   command = newSVpvf("$_re = qr\x05%s\x05%s;",
                      re_string,
                      ignore_case ? "i" : "");
@@ -375,12 +383,84 @@ precompute_re(PerlInterpreter *interpreter,
 
   SvREFCNT_dec(command);
 
-  PERL_SET_CONTEXT(old_interpreter);
-
   return re_new;
 }
 
 
+void
+perl_set_arg_array(AV *matches)
+{
+  AV *args = get_av("_", 0);
+  av_clear(args);
+  perl_copy_array(matches, args);
+}
+
+
+static
+AV *
+perl_clone_array(AV *orig)
+{
+  AV *clone;
+  int high_elem;
+
+  high_elem = av_len(orig);
+
+  clone = newAV();
+  av_extend(clone, high_elem);
+
+  perl_copy_array(orig, clone);
+  return clone;
+}
+
+
+static
+void
+perl_copy_array(AV *source, AV *dest)
+{
+  int i;
+  int highest_elem;
+
+  highest_elem = av_len(source);
+  for (i = 0; i <= highest_elem; ++i) {
+    SV **val = av_fetch(source, i, 0);
+    if (val) {
+      SvREFCNT_inc(*val);
+      if (!av_store(dest, i, *val)) {
+        SvREFCNT_dec(*val);
+      }
+    }
+  }
+}
+
+
+void
+parse_server_data(char *protocol,
+                  char *data)
+{
+  SV  *serverdataSV;
+  SV  *command;
+  dSP;
+
+  /* Set the string in a variable */
+  serverdataSV = get_sv("_serverdata", TRUE);
+  sv_setpv(serverdataSV, data);
+  SvUTF8_on(serverdataSV);
+  command = newSVpvf("_parseserverdata(%s)", protocol);
+
+  /* Run the command */
+  ENTER;
+  SAVETMPS;
+  PUSHMARK(SP);
+  eval_sv(command, G_VOID);
+  SPAGAIN;
+  PUTBACK;
+  FREETMPS;
+  LEAVE;
+
+  SvREFCNT_dec(command);
+}
+
+
 static
 GSList *
 AVtoGSList(AV *array)
@@ -412,9 +492,11 @@ menu_perl_run_cb(GtkWidget *widget, gpointer data)
   page_num = get_focused_world();
   currentWorld = (World *) g_list_nth_data(open_worlds, page_num);
   world_for_perl = currentWorld;
-  PERL_SET_CONTEXT(world_for_perl->perl_interpreter);
+  GRAB_PERL(world_for_perl);
 
   eval_pv(cmd, TRUE);
+
+  RELEASE_PERL(world_for_perl);
 }
 
 
@@ -453,6 +535,18 @@ World *fetch_world(int items, SV *self_ref, HV **self_to_store)
 }
 
 
+static
+void
+XS_getclientname(pTHX_ CV *perlcv)
+{
+  dMARK;
+  dAX;
+
+  XST_mPV(0, CLIENT_NAME);
+  XSRETURN(1);
+}
+
+
 static
 void
 XS_getversion(pTHX_ CV *perlcv)
@@ -868,6 +962,27 @@ XS_commandecho(pTHX_ CV *perlcv)
 }
 
 
+static
+void
+XS_cmdseparator(pTHX_ CV *perlcv)
+{
+  FETCHWORLDVARS;
+  dXSARGS;
+
+  FETCHWORLD;
+
+  /* If there is an argument, set the option */
+  if (items == 2) {
+    g_strlcpy(world->command_separator, SvPV_nolen(ST(1)),
+              sizeof(world->command_separator));
+  }
+
+  /* And now return the current value */
+  XST_mPV(0, world->command_separator);
+  XSRETURN(1);
+}
+
+
 static
 void
 XS_getconntime(pTHX_ CV *perlcv)
@@ -917,9 +1032,13 @@ XS_echo(pTHX_ CV *perlcv)
   for (i = 1; i < items; ++i) {
     word = SvPV(ST(i), len);
     if (world && world->gui->txtView) {
-      ansitextview_append_echoed_string(world->gui, word, len);
+      AnsiParseOperation *operation = g_new(AnsiParseOperation, 1);
+      operation->action = APPEND_ECHOED_LINE;
+      operation->line.line = g_strdup(word);
+      operation->line.line_len = len;
+      queue_ansiparse_operation(world, operation);
     } else {
-      printf(word);
+      puts(word);
     }
   }
 
@@ -1064,8 +1183,6 @@ static
 void
 XS_interpret(pTHX_ CV *perlcv)
 {
-  char   *line;
-  STRLEN  len;
   FETCHWORLDVARS;
   dXSARGS;
 
@@ -1076,8 +1193,7 @@ XS_interpret(pTHX_ CV *perlcv)
     XSRETURN_EMPTY;
   }
 
-  line = SvPV(ST(1), len);
-  parse_commands(world, line, len);
+  parse_commands(world, SvPV_nolen(ST(1)));
 
   XSRETURN_EMPTY;
 }
@@ -1141,6 +1257,57 @@ XS_getline(pTHX_ CV *perlcv)
 }
 
 
+static
+void
+XS__sendserverdata(pTHX_ CV *perlcv)
+{
+  char   *protocol;
+  char   *data;
+  STRLEN  len;
+  unsigned char telnet_code[3];
+  FETCHWORLDVARS;
+  dXSARGS;
+
+  FETCHWORLD;
+
+  if (items < 3) {
+    warn(_("%s: This is an internal function, do not call directly."),
+         "_sendserverdata");
+    XSRETURN_EMPTY;
+  }
+
+  protocol = SvPV_nolen(ST(1));
+  data = SvPV(ST(2), len);
+
+  if (strcmp(protocol, "GMCP") == 0) {
+    if (!world->gmcp_enabled) {
+      warn(_("Protocol %s not enabled."), protocol);
+      XSRETURN_EMPTY;
+    }
+    telnet_code[2] = TELNET_OPT_GMCP;
+  } else if (strcmp(protocol, "MSDP") == 0) {
+    if (!world->msdp_enabled) {
+      warn(_("Protocol %s not enabled."), protocol);
+      XSRETURN_EMPTY;
+    }
+    telnet_code[2] = TELNET_OPT_MSDP;
+  } else {
+    warn(_("Unknown protocol"));
+    XSRETURN_EMPTY;
+  }
+
+  telnet_code[0] = TELNET_IAC;
+  telnet_code[1] = TELNET_SB;
+  kc_send(world, telnet_code, 3);
+  kc_send(world, data, len);
+  /* telnet_code[0] = TELNET_IAC; - Already set */
+  telnet_code[1] = TELNET_SE;
+  kc_send(world, telnet_code, 2);
+
+  XSRETURN_EMPTY;
+}
+
+
 static
 void
 XS_play(pTHX_ CV *perlcv)
@@ -1162,9 +1329,9 @@ static
 void
 XS_stripansi(pTHX_ CV *perlcv)
 {
-  unsigned char *original;
-  STRLEN         len;
-  unsigned char *stripped;
+  char   *original;
+  STRLEN  len;
+  char   *stripped;
   dXSARGS;
 
   if (items == 0) {
@@ -1172,10 +1339,10 @@ XS_stripansi(pTHX_ CV *perlcv)
     XSRETURN_EMPTY;
   }
 
-  original = (unsigned char *) SvPV(ST(0), len);
+  original = SvPV(ST(0), len);
   stripped = strip_ansi(original, len);
 
-  XST_mPV(0, (char *) stripped);
+  XST_mPV(0, stripped);
   g_free(stripped);
   XSRETURN(1);
 }
@@ -1383,8 +1550,7 @@ XS_trigger(pTHX_ CV *perlcv)
     if (trigger->pattern_re) {
       SvREFCNT_dec(trigger->pattern_re);
     }
-    trigger->pattern_re = precompute_re(world->perl_interpreter,
-                                        trigger->pattern,
+    trigger->pattern_re = precompute_re(trigger->pattern,
                                         trigger->ignore_case);
   }
 
@@ -1404,7 +1570,6 @@ XS_trigger(pTHX_ CV *perlcv)
       }
     }
   } else {
-
     if (!is_silent || !SvIV(*is_silent)) {
       ansitextview_append_string_nl(world->gui, _("Trigger modified."));
     }
@@ -1912,8 +2077,7 @@ XS_alias(pTHX_ CV *perlcv)
     if (alias->pattern_re) {
       SvREFCNT_dec(alias->pattern_re);
     }
-    alias->pattern_re = precompute_re(world->perl_interpreter,
-                                      alias->pattern,
+    alias->pattern_re = precompute_re(alias->pattern,
                                       alias->ignore_case);
   }
 
@@ -2407,7 +2571,7 @@ XS_timer(pTHX_ CV *perlcv)
 {
   int      i = -1;
   int      argpos = 1;
-  int      new_interval = 0;
+  gdouble  new_interval = 0;
   int      new_enabled = -999;
   HV      *attributes;
   SV     **value;
@@ -2468,7 +2632,7 @@ XS_timer(pTHX_ CV *perlcv)
 
   value = hv_fetch(attributes, "interval", 8, FALSE);
   if (value) {
-    if (!SvIOK(*value)) {
+    if (!SvNIOK(*value)) {
       warn(_("%s: interval must be numeric"), "timer");
       if (i == -1) {
         g_free(timer);
@@ -2476,7 +2640,7 @@ XS_timer(pTHX_ CV *perlcv)
       XSRETURN_EMPTY;
     }
 
-    new_interval = SvIV(*value);
+    new_interval = SvNV(*value);
   }
 
   value = hv_fetch(attributes, "count", 5, FALSE);
@@ -2519,6 +2683,13 @@ XS_timer(pTHX_ CV *perlcv)
     }
     XSRETURN_EMPTY;
   }
+  if (new_interval < 0) {
+    warn(_("%s: cannot create timer with negative interval"), "timer");
+    if (i == -1) {
+      g_free(timer);
+    }
+    XSRETURN_EMPTY;
+  }
 
   /* Recreate timer function if interval is changed */
   if (i != -1 && new_enabled == -999 &&
@@ -2616,7 +2787,7 @@ XS__listtimer(pTHX_ CV *perlcv)
   ansitextview_append_stringf(world->gui,
                               _("Timer number %d\n"
                                 "  Name:         %s\n"
-                                "  Interval:     %d\n"
+                                "  Interval:     %.3f\n"
                                 "  Repeat count: %d\n"
                                 "  Action:       %s\n"
                                 "  Enabled:      %s\n"
@@ -3112,10 +3283,6 @@ XS__writetolog(pTHX_ CV *perlcv)
 
   FETCHWORLD;
 
-  if (!world->log_file) {
-    XSRETURN_EMPTY;
-  }
-
   for (i = 1; i < items; ++i) {
     line = SvPV_nolen(ST(i));
     write_log_line(world, line, NULL);
@@ -3278,6 +3445,8 @@ XS_listplugin(pTHX_ CV *perlcv)
   list_hooks(world, "OnLoseFocus",      plugin);
   ansitextview_append_string(world->gui, "\n");
   list_hooks(world, "OnCloseConnected", plugin);
+  ansitextview_append_string(world->gui, "\n");
+  list_hooks(world, "OnServerData", plugin);
 
   XSRETURN_YES;
 }
@@ -3404,40 +3573,46 @@ xs_init()
 
 
   /* Our subroutines */
-  newXS("getversion",       XS_getversion, "");
-  newXS("quit",             XS_quit,       "");
+  newXS("getclientname", XS_getclientname, "");
+  newXS("getversion",    XS_getversion,    "");
+  newXS("quit",          XS_quit,          "");
+
+  newXS("gotow",    XS_gotow,    "");
+  newXS("getworld", XS_getworld, "");
+
+  newXS("play", XS_play, "");
+
+  newXS("stripansi", XS_stripansi, "");
+
 
   newXS("Window::minimize",       XS_window_minimize,       "Window");
   newXS("Window::getsize",        XS_window_getsize,        "Window");
   newXS("Window::settitle",       XS_window_settitle,       "Window");
   newXS("Window::seturgencyhint", XS_window_seturgencyhint, "Window");
 
+
   newXS("World::dc",           XS_dc,           "World");
   newXS("World::reconnect",    XS_reconnect,    "World");
   newXS("World::connectother", XS_connectother, "World");
   newXS("World::save",         XS_save,         "World");
   newXS("World::close",        XS_close,        "World");
   newXS("World::next",         XS_next,         "World");
-  newXS("gotow",               XS_gotow,        "");
-  newXS("getworld",            XS_getworld,     "");
 
   newXS("World::getname",      XS_getname,      "World");
   newXS("World::getcharacter", XS_getcharacter, "World");
   newXS("World::getmainfont",  XS_getmainfont,  "World");
   newXS("World::getentryfont", XS_getentryfont, "World");
   newXS("World::commandecho",  XS_commandecho,  "World");
+  newXS("World::cmdseparator", XS_cmdseparator, "World");
   newXS("World::getconntime",  XS_getconntime,  "World");
   newXS("World::getidletime",  XS_getidletime,  "World");
 
-  newXS("World::echo",      XS_echo,      "World");
-  newXS("World::send",      XS_send,      "World");
-  newXS("World::_mlsend",   XS__mlsend,   "World");
-  newXS("World::interpret", XS_interpret, "World");
-  newXS("World::getline",   XS_getline,   "World");
-
-  newXS("play", XS_play, "");
-
-  newXS("stripansi", XS_stripansi, "");
+  newXS("World::echo",            XS_echo,      "World");
+  newXS("World::send",            XS_send,      "World");
+  newXS("World::_mlsend",         XS__mlsend,   "World");
+  newXS("World::interpret",       XS_interpret, "World");
+  newXS("World::getline",         XS_getline,   "World");
+  newXS("World::_sendserverdata", XS__sendserverdata, "World");
 
   newXS("World::setstatus", XS_setstatus, "World");
 
