@@ -1,7 +1,5 @@
-/* $Id: permanentvariables.c 1143 2011-02-13 16:13:58Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +22,7 @@
 #endif
 
 #include <string.h>
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <locale.h>
 #include <gtk/gtk.h>
 
@@ -54,7 +52,7 @@ load_permanent_variables(World *world)
 
   if (world->file) {
     variables_file = g_strdup_printf("%s.var", world->file);
-    perl_script_import_file(world->perl_interpreter, variables_file);
+    perl_script_import_file(variables_file);
     g_free(variables_file);
   }
 }
@@ -63,7 +61,6 @@ load_permanent_variables(World *world)
 void
 save_permanent_variables(World *world)
 {
-  PerlInterpreter *old_interpreter;
   gchar           *variables_file;
   GSList          *iter;
   gchar           *varname;
@@ -86,8 +83,7 @@ save_permanent_variables(World *world)
 
   file_contents = g_string_sized_new(512);
 
-  old_interpreter = PERL_GET_CONTEXT;
-  PERL_SET_CONTEXT(world->perl_interpreter);
+  GRAB_PERL(world);
 
   iter = world->permanent_variables;
   while (iter) {
@@ -117,6 +113,8 @@ save_permanent_variables(World *world)
     iter = iter->next;
   }
 
+  RELEASE_PERL(world);
+
   if (!g_file_set_contents(variables_file,
                            file_contents->str, file_contents->len,
                            &error)) {
@@ -134,8 +132,6 @@ save_permanent_variables(World *world)
   }
   g_free(variables_file);
   g_string_free(file_contents, TRUE);
-
-  PERL_SET_CONTEXT(old_interpreter);
 }
 
 
