@@ -1,7 +1,5 @@
-/* $Id: plugins.c 1150 2011-03-04 00:31:25Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +21,7 @@
 #  include <kcconfig.h>
 #endif
 
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <locale.h>
 #include <gtk/gtk.h>
 
@@ -92,7 +90,7 @@ load_plugin(World *world, gchar *filename, GError **error)
   SV          *command;
   SV          *ret;
 
-  PERL_SET_CONTEXT(world->perl_interpreter);
+  GRAB_PERL(world);
   command = newSVpvf("($_error, $_pname) = $::world->_loadplugin('%s');",
                      filename);
   eval_sv(command, G_ARRAY);
@@ -103,8 +101,11 @@ load_plugin(World *world, gchar *filename, GError **error)
                 G_FILE_ERROR, G_FILE_ERROR_NOENT,
                 _("Failed loading file %s: %s"),
                 filename, SvPV_nolen(ret));
+    RELEASE_PERL(world);
     return FALSE;
   }
+
+  RELEASE_PERL(world);
   return TRUE;
 }
 
@@ -115,14 +116,13 @@ load_startup_plugins(World *world)
 {
   GSList *pluginitem;
 
-  PERL_SET_CONTEXT(world->perl_interpreter);
   pluginitem = world->startup_plugin_files;
   while (pluginitem) {
     GError *error = NULL;
 
     add_startup_plugin(world, pluginitem->data, &error);
     if (error) {
-      fprintf(stderr, error->message);
+      fputs(error->message, stderr);
       fputc('\n', stderr);
       g_error_free(error);
     } else {
@@ -149,7 +149,7 @@ add_startup_plugin(World *world, gchar *filename, GError **error)
   gchar       *author;
   GtkTreeIter  iter;
 
-  PERL_SET_CONTEXT(world->perl_interpreter);
+  GRAB_PERL(world);
   command = newSVpvf("($_error, $_name, $_vers, $_desc, $_author) = $::world->_getplugininfo('%s');",
                      filename);
   eval_sv(command, G_ARRAY);
@@ -160,6 +160,7 @@ add_startup_plugin(World *world, gchar *filename, GError **error)
                 G_FILE_ERROR, G_FILE_ERROR_NOENT,
                 _("Failed loading file %s: %s"),
                 filename, SvPV_nolen(ret));
+    RELEASE_PERL(world);
     return FALSE;
   }
 
@@ -172,6 +173,8 @@ add_startup_plugin(World *world, gchar *filename, GError **error)
   ret    = get_sv("_author", FALSE);
   author = SvPV_nolen(ret);
 
+  RELEASE_PERL(world);
+
   gtk_list_store_append(GTK_LIST_STORE(world->startup_plugins), &iter);
   gtk_list_store_set(GTK_LIST_STORE(world->startup_plugins), &iter,
                      SPLUGIN_FILE, filename,
