@@ -1,7 +1,5 @@
-/* $Id: prefs.c 1175 2011-05-28 14:24:01Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +26,7 @@
 #include <errno.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
@@ -77,7 +75,7 @@ G_MODULE_EXPORT void     cmbProxyType_changed_cb(GtkComboBox *combo, gpointer da
 
 
 void
-show_preferences_dialog_cb(GtkWidget *widget, gpointer data)
+show_preferences_dialog_cb(GtkMenuItem *widget, gpointer data)
 {
   static GtkWidget *dialog = NULL;
   GtkNotebook      *notebook;
@@ -176,7 +174,6 @@ show_preferences_dialog_cb(GtkWidget *widget, gpointer data)
     /* Remove "Use Global Settings" - these are the global settings */
     cmbProxyType
       = GTK_COMBO_BOX(gtk_builder_get_object(main_builder, "cmbGProxyType"));
-    gtk_combo_box_remove_text(cmbProxyType, 0);
     gtk_combo_box_set_active(cmbProxyType, globalPrefs.proxy.type);
 
 #ifndef __WIN32__
