@@ -1,7 +1,5 @@
-/* $Id: proxy.c 1154 2011-03-26 22:57:37Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,10 +27,11 @@
 #include <errno.h>
 #include <stdint.h>
 #include <sys/time.h>
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <locale.h>
 #include <gtk/gtk.h>
 #include <zlib.h>
+#include <glib.h>
 
 #ifndef __MINGW32__
 #  include <fcntl.h>
@@ -41,9 +40,6 @@
 #  include <netdb.h>
 #  include <arpa/inet.h>
 #else /* defined __MINGW32__ */
-/* This is so we can use getaddrinfo() et al. The code will only run
-   in Windows XP. */
-#  define _WIN32_WINNT 0x0501
 #  include <winsock2.h>
 #  include <ws2tcpip.h>
 #endif
@@ -134,6 +130,12 @@ static gboolean proxy_socks5_data_ready_auth(World *world);
 static gboolean proxy_socks5_data_ready_connection(World *world);
 static void     proxy_socks5_send_auth(World *world);
 static void     proxy_socks5_send_connection_request(World *world);
+static gboolean proxy_http_data_ready_cb(GIOChannel   *iochan,
+                                         GIOCondition  cond,
+                                         gpointer      data);
+static gchar   *proxy_http_get_connection_string(World *world);
+
+#define HAS_PROXY_HTTP_RESPONSE(w) ((w)->pbuf_size >= 4 && memcmp((w)->inbuffer + (w)->pbuf_size - 4, "\r\n\r\n", 4) == 0)
 
 
 
@@ -175,6 +177,7 @@ proxy_socks4_resolved_real_host_cb(gpointer data)
   struct resolve_data_s *resolve_data = (struct resolve_data_s *) data;
   World *world = resolve_data->world;
   gchar *msg;
+  GThread *resolver_thread;
 
   g_free(resolve_data->host);
   g_free(resolve_data->port);
@@ -211,20 +214,15 @@ proxy_socks4_resolved_real_host_cb(gpointer data)
   gtk_label_set_text(world->gui->lblStatus, msg);
   g_free(msg);
 
-  while (gtk_events_pending())
-    gtk_main_iteration();
-
   resolve_data = g_new(struct resolve_data_s, 1);
   resolve_data->world     = world;
   resolve_data->host      = g_strdup(world->proxy_used.server);
   resolve_data->port      = g_strdup(world->proxy_used.port);
   resolve_data->ai_family = AF_UNSPEC;
   resolve_data->callback  = continue_connecting_cb;
-  if (!g_thread_create(resolve_name_thread, resolve_data, FALSE, NULL)) {
-    world->errmsg = g_strdup(_("Could not start name-resolution thread."));
-    disconnect_world(world, FALSE);
-    return FALSE;
-  }
+  resolver_thread = g_thread_new("resolver",
+                                 resolve_name_thread, resolve_data);
+  g_thread_unref(resolver_thread);
 
   return FALSE;
 }
@@ -649,3 +647,116 @@ proxy_socks5_send_connection_request(World *world)
   world->proxy_used.state = SOCKS5_STATE_CONNECTION;
   world->pbuf_size = 0;
 }
+
+
+/********
+ * HTTP *
+ ********/
+gboolean
+proxy_http_connected_to(World *world)
+{
+  gchar *msg;
+  gchar *request;
+
+  msg = g_strdup_printf(_("Connected to proxy server %s port %s, sending request..."),
+                        world->proxy_used.server, world->proxy_used.port);
+  ansitextview_append_string_with_fgcolor(world->gui, msg,
+                                          globalPrefs.idxInfoMsgColor);
+  ansitextview_append_string(world->gui, "\n");
+  gtk_label_set_text(world->gui->lblStatus, msg);
+  g_free(msg);
+
+  request = proxy_http_get_connection_string(world);
+
+  /* We do not use kc_send because TLS is never used at this point */
+  send(world->sock, request, strlen(request), 0);
+  g_free(request);
+
+  /* Set a callback to read the response */
+  world->pbuf_size = 0; /* This counter will hold the number of bytes
+                           read. So far, nothing has been read yet. */
+  g_source_remove(world->io_watch_id);
+  world->io_watch_id = g_io_add_watch_full(world->iochan,
+                                           G_PRIORITY_HIGH,
+                                           G_IO_IN | G_IO_HUP | G_IO_ERR,
+                                           proxy_http_data_ready_cb,
+                                           world,
+                                           NULL);
+
+  return TRUE;
+}
+
+
+static
+gboolean
+proxy_http_data_ready_cb(GIOChannel   *iochan,
+                         GIOCondition  cond,
+                         gpointer      data)
+{
+  World    *world;
+  ssize_t   nread;
+
+  world = (World *) data;
+
+  /* Since there is no way to know how long the proxy response will be,
+     we must read 1 byte at a time and check if it is complete in order
+     not to read the contents of the actual connection. */
+  nread = recv(world->sock,
+               world->inbuffer + world->pbuf_size,
+               1, 0);
+  if (nread == -1) {
+    world->errmsg = g_strdup(_("Error while reading from proxy host."));
+    world->pbuf_size = 0; /* disconnect_world() flushes the buffer. */
+    disconnect_world(world, FALSE);
+    return FALSE;
+  }
+  ++world->pbuf_size;
+  if (!HAS_PROXY_HTTP_RESPONSE(world)) {
+    /* Continue reading... */
+    return TRUE;
+  }
+
+  gchar *code_start = memchr(world->inbuffer, ' ', world->pbuf_size);
+  if (code_start == NULL || memcmp(code_start, " 200", 4) != 0) {
+    gchar *line_end = memchr(world->inbuffer, '\r', world->pbuf_size);
+    *line_end = '\0';
+
+    world->errmsg = g_strdup_printf(_("Proxy request failed: %s."),
+                                    code_start + 1);
+    world->pbuf_size = 0;
+    disconnect_world(world, FALSE);
+    return FALSE;
+  }
+
+  world->pbuf_size = 0;
+  perform_tls_handshake(world);
+  /* Not to be called again. */
+  return FALSE;
+}
+
+
+static
+gchar *
+proxy_http_get_connection_string(World *world)
+{
+  if (strcmp(world->proxy_used.user, "") == 0) {
+    return g_strdup_printf("CONNECT %s:%s HTTP/1.0\r\n\r\n",
+                           world->host, world->port);
+  } else {
+    gchar *auth_string = g_strdup_printf("%s:%s",
+                                         world->proxy_used.user,
+                                         world->proxy_used.password);
+    gchar *basic_auth_data = g_base64_encode((guchar *) auth_string,
+                                             strlen(auth_string));
+    g_free(auth_string);
+
+    gchar *result = g_strdup_printf("CONNECT %s:%s HTTP/1.0\r\n"
+                                    "Proxy-Authorization: Basic %s\r\n"
+                                    "\r\n",
+                                    world->host, world->port,
+                                    basic_auth_data);
+    g_free(basic_auth_data);
+    return result;
+  }
+}
+
