@@ -27,7 +27,7 @@
 #endif
 
 #include <string.h>
-#include <libintl.h>
+#include "libintl-wrapper.h"
 
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
@@ -45,7 +45,6 @@
 #include <windows.h>
 #include "gdk/gdkwin32.h"
 #endif
-#include "util_cursor.h"
 //#define WITHOUT_BUTTON
 
 #define CHILD_SPACING     1
@@ -88,12 +87,12 @@ struct _SimoComboBoxPrivate{
 #if defined(GDK_WINDOWING_X11) && defined(USE_SHADOWS)
   DropShadow *ds;
 #endif
-  SimoCursor hand;
+  GdkCursor *hand;
 
   GtkEntryCompletion *completion;
 
 #ifdef HAVE_GTKSPELL
-  GtkSpell *gtkspell;
+  GtkSpellChecker *gtkspell;
 #endif
 
   gboolean scrolls_visible;
@@ -128,6 +127,12 @@ static guint combo_box_signals[LAST_SIGNAL] = {0,};
 
 static void     simo_combo_box_class_init(SimoComboBoxClass *klass);
 static void     simo_combo_box_init(SimoComboBox *combo_box);
+static void     simo_combo_box_get_preferred_height(GtkWidget *widget,
+                                                    gint      *minimum_height,
+                                                    gint      *natural_height);
+static void     simo_combo_box_get_preferred_width(GtkWidget *widget,
+                                                   gint      *minimum_width,
+                                                   gint      *natural_width);
 static void     simo_combo_box_size_request(GtkWidget      *widget,
                                             GtkRequisition *requisition);
 static void     simo_combo_box_finalize(GObject *object);
@@ -218,7 +223,7 @@ simo_combo_box_get_type(void)
       (GInstanceInitFunc) simo_combo_box_init
     };
 
-    combo_box_type = g_type_register_static(GTK_TYPE_HBOX,
+    combo_box_type = g_type_register_static(GTK_TYPE_BOX,
                                             "SimoComboBox",
                                             &combo_box_info,
                                             (GTypeFlags) 0);
@@ -250,11 +255,9 @@ void
 simo_combo_box_class_init(SimoComboBoxClass *klass)
 {
   GObjectClass      *g_object_class;
-  GtkObjectClass    *gtk_object_class;
   GtkWidgetClass    *widget_class;
 
   g_object_class   = G_OBJECT_CLASS (klass);
-  gtk_object_class = (GtkObjectClass*) klass;
   widget_class     = (GtkWidgetClass*) klass;
 
   //widget_class->show = simo_combo_box_show;
@@ -267,7 +270,8 @@ simo_combo_box_class_init(SimoComboBoxClass *klass)
   g_object_class->set_property = simo_combo_box_set_property;
   g_object_class->get_property = simo_combo_box_get_property;
   widget_class->size_allocate  = simo_combo_box_size_allocate;
-  widget_class->size_request   = simo_combo_box_size_request;
+  widget_class->get_preferred_height = simo_combo_box_get_preferred_height;
+  widget_class->get_preferred_width = simo_combo_box_get_preferred_width;
   widget_class->grab_focus     = simo_combo_box_grab_focus_signal;
 
   klass->clicked = NULL;
@@ -275,42 +279,42 @@ simo_combo_box_class_init(SimoComboBoxClass *klass)
 
   /* signals */
   combo_box_signals[CLICKED] = g_signal_new("clicked",
-                                            G_OBJECT_CLASS_TYPE(gtk_object_class),
+                                            G_OBJECT_CLASS_TYPE(widget_class),
                                             G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
                                             G_STRUCT_OFFSET(SimoComboBoxClass, clicked),
                                             NULL, NULL,
                                             g_cclosure_marshal_VOID__VOID,
                                             G_TYPE_NONE, 0);
   combo_box_signals[POPUP_DISPLAYED] = g_signal_new("popup-displayed",
-                                                    G_OBJECT_CLASS_TYPE(gtk_object_class),
+                                                    G_OBJECT_CLASS_TYPE(widget_class),
                                                     G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
                                                     G_STRUCT_OFFSET(SimoComboBoxClass, popup_displayed),
                                                     NULL, NULL,
                                                     g_cclosure_marshal_VOID__VOID,
                                                     G_TYPE_NONE, 0);
   combo_box_signals[POPUP_CLOSED] = g_signal_new("popup-closed",
-                                                 G_OBJECT_CLASS_TYPE(gtk_object_class),
+                                                 G_OBJECT_CLASS_TYPE(widget_class),
                                                  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
                                                  G_STRUCT_OFFSET(SimoComboBoxClass, popup_closed),
                                                  NULL, NULL,
                                                  g_cclosure_marshal_VOID__VOID,
                                                  G_TYPE_NONE, 0);
   combo_box_signals[CHANGED] = g_signal_new("changed",
-                                            G_OBJECT_CLASS_TYPE(gtk_object_class),
+                                            G_OBJECT_CLASS_TYPE(widget_class),
                                             G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
                                             G_STRUCT_OFFSET(SimoComboBoxClass, changed),
                                             NULL, NULL,
                                             g_cclosure_marshal_VOID__VOID,
                                             G_TYPE_NONE, 0);
   combo_box_signals[ACTIVATE] = g_signal_new("activate",
-                                            G_OBJECT_CLASS_TYPE(gtk_object_class),
+                                            G_OBJECT_CLASS_TYPE(widget_class),
                                             G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
                                             G_STRUCT_OFFSET(SimoComboBoxClass, activate),
                                             NULL, NULL,
                                             g_cclosure_marshal_VOID__VOID,
                                             G_TYPE_NONE, 0);
   combo_box_signals[SIZE_CHANGED] = g_signal_new("size_changed",
-                                            G_OBJECT_CLASS_TYPE(gtk_object_class),
+                                            G_OBJECT_CLASS_TYPE(widget_class),
                                             G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
                                             G_STRUCT_OFFSET(SimoComboBoxClass, size_changed),
                                             NULL, NULL,
@@ -417,7 +421,7 @@ simo_combo_box_init(SimoComboBox *combo_box)
 
   mthis->n_lines = 1;
   mthis->max_lines = 10;
-  mthis->hand = simo_cursor_new( SIMO_HAND );
+  mthis->hand = gdk_cursor_new( GDK_HAND2 );
   // build the popup window
   mthis->popup_window = gtk_window_new( GTK_WINDOW_POPUP );
   align = gtk_alignment_new( 0, 0, 1, 1 );
@@ -466,7 +470,7 @@ simo_combo_box_init(SimoComboBox *combo_box)
   gtk_window_set_decorated( GTK_WINDOW( mthis->popup_window ), FALSE );
   gtk_window_set_resizable( GTK_WINDOW( mthis->popup_window ), TRUE );
   gtk_widget_realize( mthis->popup_window );
-  simo_cursor_set_delay(gtk_widget_get_window(mthis->popup_window),
+  gdk_window_set_cursor(gtk_widget_get_window(mthis->popup_window),
                         mthis->hand );
   mask = gdk_window_get_events(gtk_widget_get_window(mthis->popup_window));
   gdk_window_set_events(gtk_widget_get_window(mthis->popup_window),
@@ -522,8 +526,6 @@ simo_combo_box_init(SimoComboBox *combo_box)
 #ifdef GDK_WINDOWING_WIN32 // FIXME: should check gtk+ version number for 2.5+
   gtk_label_set_ellipsize( GTK_LABEL( combo_box->label ), PANGO_ELLIPSIZE_END );
 #endif
-  //	hbox										= gtk_hbox_new( FALSE, 0 );
-  combo_box->editable = TRUE;
 
   // give an initial allocation size to the entry box
   // Removed by Eduardo Kalinowski. Seems not to be necessary, and
@@ -588,7 +590,8 @@ simo_combo_box_init(SimoComboBox *combo_box)
   gtk_box_pack_start(GTK_BOX(combo_box),
                      mthis->scrolled_window, TRUE, TRUE, 0);
 
-  mthis->vbox_arrows = gtk_vbox_new(TRUE, 0);
+  mthis->vbox_arrows = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+  gtk_box_set_homogeneous(GTK_BOX(mthis->vbox_arrows), TRUE);
   arrow = gtk_arrow_new(GTK_ARROW_UP, GTK_SHADOW_OUT);
   gtk_misc_set_alignment(GTK_MISC(arrow), .5, 0);
   arrow_evtbox = gtk_event_box_new();
@@ -651,7 +654,7 @@ simo_combo_box_key_release_event(GtkWidget    *widget,
                                  GdkEventKey  *event,
                                  SimoComboBox *combo_box)
 {
-  if( event->keyval == GDK_Escape )
+  if( event->keyval == GDK_KEY_Escape )
     simo_combo_box_popup_close( combo_box );
 
   return FALSE;
@@ -678,7 +681,7 @@ simo_combo_box_finalize(GObject *object)
 #if defined(GDK_WINDOWING_X11) && defined(USE_SHADOWS)
   drop_shadow_destroy( mthis->ds );
 #endif
-  simo_cursor_free( mthis->hand );
+  g_object_unref( mthis->hand );
 
   g_free(mthis->ml_text);
 
@@ -766,6 +769,35 @@ simo_combo_box_set_property( GObject *object,
 }
 
 
+
+static
+void
+simo_combo_box_get_preferred_height(GtkWidget *widget,
+                                    gint      *minimum_height,
+                                    gint      *natural_height)
+{
+  GtkRequisition req;
+  simo_combo_box_size_request(widget, &req);
+  *minimum_height = req.height;
+  *natural_height = req.height;
+}
+
+
+
+static
+void
+simo_combo_box_get_preferred_width(GtkWidget *widget,
+                                   gint      *minimum_width,
+                                   gint      *natural_width)
+{
+  GtkRequisition req;
+  simo_combo_box_size_request(widget, &req);
+  *minimum_width = req.width;
+  *natural_width = req.width;
+}
+
+
+
 static
 void
 simo_combo_box_size_request( GtkWidget *widget, GtkRequisition *req )
@@ -781,13 +813,13 @@ simo_combo_box_size_request( GtkWidget *widget, GtkRequisition *req )
   //						combo_box->priv->entry_alloc.width,  combo_box->button->allocation.width, width, height );
 
   if (combo_box->priv->n_lines > 1) {
-    gtk_widget_size_request(combo_box->priv->scrolled_window, &ereq);
+    gtk_widget_get_preferred_size(combo_box->priv->scrolled_window, NULL, &ereq);
   } else {
-    gtk_widget_size_request(combo_box->entry, &ereq);
+    gtk_widget_get_preferred_size(combo_box->entry, NULL, &ereq);
   }
-  gtk_widget_size_request(combo_box->button, &breq);
-  gtk_widget_size_request(combo_box->priv->label_button, &lreq);
-  gtk_widget_size_request(combo_box->priv->vbox_arrows, &areq);
+  gtk_widget_get_preferred_size(combo_box->button, NULL, &breq);
+  gtk_widget_get_preferred_size(combo_box->priv->label_button, NULL, &lreq);
+  gtk_widget_get_preferred_size(combo_box->priv->vbox_arrows, NULL, &areq);
 
   if (lreq.width < ereq.width) {
     ereq.width = lreq.width;
@@ -844,13 +876,8 @@ simo_combo_box_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
   GTK_WIDGET_CLASS( parent_class )->size_allocate( widget, allocation );
   //		g_print( "combo box allocate: %d:%d:%d:%d:%d\n", allocation->x, allocation->y, allocation->width, allocation->height, combo->priv->fixed_entry_size );
 
-#if GTK_MINOR_VERSION < 18
-  combobtnAlloc   = combo->button->allocation;
-  vboxarrowsAlloc = combo->priv->vbox_arrows->allocation;
-#else
   gtk_widget_get_allocation(combo->button, &combobtnAlloc);
   gtk_widget_get_allocation(combo->priv->vbox_arrows, &vboxarrowsAlloc);
-#endif
 
   ealloc.x = allocation->x;
   ealloc.y = allocation->y;
@@ -1032,11 +1059,7 @@ simo_combo_box_update_pressed( GtkWidget *widget,
   gdk_window_get_root_origin( gtk_widget_get_window(widget),
                               &xoffset, &yoffset );
 
-#if GTK_MINOR_VERSION < 18
-  alloc = widget->allocation;
-#else
   gtk_widget_get_allocation(widget, &alloc);
-#endif
 
   xoffset += alloc.x;
   yoffset += alloc.y;
@@ -1062,8 +1085,8 @@ simo_combo_box_popup_close( SimoComboBox *combo_box )
   g_signal_emit( combo_box, combo_box_signals[POPUP_CLOSED], 0 );
 
   gtk_grab_remove( combo_box->priv->popup_window );
-  gdk_keyboard_ungrab( GDK_CURRENT_TIME );
-  gdk_pointer_ungrab( GDK_CURRENT_TIME );
+  //gdk_keyboard_ungrab( GDK_CURRENT_TIME );
+  //gdk_pointer_ungrab( GDK_CURRENT_TIME );
 
   g_signal_handlers_disconnect_by_func( G_OBJECT( combo_box->priv->popup_window ),
                                         G_CALLBACK( simo_combo_box_update_pressed ),
@@ -1073,9 +1096,9 @@ simo_combo_box_popup_close( SimoComboBox *combo_box )
 #endif
 #ifdef GDK_WINDOWING_WIN32
   {
-    RECT rcFrom,rcTo;
-    HWND from_hwnd =GDK_WINDOW_HWND(combo_box->priv->popup_window->window);
-    HWND to_hwnd = GDK_WINDOW_HWND(GTK_WIDGET(combo_box)->window);
+    RECT rcFrom, rcTo;
+    HWND from_hwnd = GDK_WINDOW_HWND(gtk_widget_get_window(combo_box->priv->popup_window));
+    HWND to_hwnd = GDK_WINDOW_HWND(gtk_widget_get_window(GTK_WIDGET(combo_box)));
 
     GetWindowRect(from_hwnd,&rcFrom);
     GetWindowRect(to_hwnd,&rcTo);
@@ -1094,16 +1117,6 @@ simo_combo_box_popup_close( SimoComboBox *combo_box )
   gtk_widget_hide(GTK_WIDGET(combo_box->priv->popup_window));
   // the label should have focus after the combobox is clicked
   // make sure our widget has focus after being clicked on
-#if GTK_MINOR_VERSION < 18
-  if (GTK_WIDGET_CAN_FOCUS(combo_box)) {
-    //    gtk_widget_grab_focus( GTK_WIDGET( combo_box ) );
-    if (GTK_WIDGET_CAN_FOCUS(combo_box->entry)) {
-      gtk_widget_grab_focus(GTK_WIDGET(combo_box->entry));
-    } else if (GTK_WIDGET_CAN_FOCUS(combo_box->priv->label_button)) {
-      gtk_widget_grab_focus(GTK_WIDGET( combo_box->priv->label_button));
-    }
-  }
-#else
   if (gtk_widget_get_can_focus(GTK_WIDGET(combo_box))) {
     //    gtk_widget_grab_focus( GTK_WIDGET( combo_box ) );
     if (gtk_widget_get_can_focus(combo_box->entry)) {
@@ -1112,7 +1125,6 @@ simo_combo_box_popup_close( SimoComboBox *combo_box )
       gtk_widget_grab_focus(GTK_WIDGET( combo_box->priv->label_button));
     }
   }
-#endif
 }
 
 
@@ -1181,20 +1193,25 @@ simo_combo_box_compute_tree_size( SimoComboBox *combo,
   for( i = gtk_tree_view_get_columns( combo->treeview ); i; i = g_list_next( i ) ){
     spacing += gtk_tree_view_column_get_spacing( GTK_TREE_VIEW_COLUMN( i->data ) );
     // loop over each cell renderer
-    for( j = gtk_cell_layout_get_cells( GTK_CELL_LAYOUT( i->data ) );
-         j; j = g_list_next( j ) ){
+    for (j = gtk_cell_layout_get_cells(GTK_CELL_LAYOUT(i->data));
+         j;
+         j = g_list_next(j)) {
+      GtkRequisition size_req;
 
-      gtk_cell_renderer_get_size( GTK_CELL_RENDERER( j->data ), GTK_WIDGET( combo->treeview ),
-                                  NULL, NULL, NULL, NULL, &height );
-      if( gtk_cell_renderer_text_get_type() == G_OBJECT_TYPE( j->data ) ){
-        if( desc ){
-          pango_font_description_free( desc );
+      gtk_cell_renderer_get_preferred_size(GTK_CELL_RENDERER(j->data),
+                                           GTK_WIDGET(combo->treeview),
+                                           NULL,
+                                           &size_req);
+      if (gtk_cell_renderer_text_get_type() == G_OBJECT_TYPE(j->data)) {
+        if (desc) {
+          pango_font_description_free(desc);
           desc = 0;
         }
-        g_object_get( G_OBJECT( j->data ), "font-desc", &desc, NULL );
+        g_object_get(G_OBJECT(j->data), "font-desc", &desc, NULL);
       }
-      height += gtk_tree_view_column_get_spacing( GTK_TREE_VIEW_COLUMN( i->data ) );
-      if( height > max_height ){
+      height = size_req.height
+        + gtk_tree_view_column_get_spacing(GTK_TREE_VIEW_COLUMN(i->data));
+      if (height > max_height) {
         max_height = height;
       }
     }
@@ -1242,15 +1259,9 @@ simo_combo_box_popup( SimoComboBox *combo_box )
 
   if (mthis->parent == NULL) {
     mthis->parent = gtk_widget_get_toplevel(widget);
-#if GTK_MINOR_VERSION < 18
-   if (!GTK_WIDGET_TOPLEVEL(mthis->parent)) {
-      return;
-    }
-#else
    if (!gtk_widget_is_toplevel(mthis->parent)) {
       return;
     }
-#endif
   }
 
   g_signal_emit( combo_box, combo_box_signals[POPUP_DISPLAYED], 0 );
@@ -1260,11 +1271,7 @@ simo_combo_box_popup( SimoComboBox *combo_box )
   //	g_signal_connect( G_OBJECT( mthis->popup_window ), "event_after",
   //										G_CALLBACK( simo_combo_box_button_after ), combo_box );
 
-#if GTK_MINOR_VERSION < 18
-  alloc = widget->allocation;
-#else
   gtk_widget_get_allocation(widget, &alloc);
-#endif
 
   // begin determining the window position
 
@@ -1274,19 +1281,11 @@ simo_combo_box_popup( SimoComboBox *combo_box )
   screen_height = gdk_screen_height() - by;
   screen_width = gdk_screen_width();
 
-#if GTK_MINOR_VERSION < 18
-  alloc = mthis->popup_window->allocation;
-#else
   gtk_widget_get_allocation(mthis->popup_window, &alloc);
   height = alloc.height;
   width = alloc.width;
-#endif
 
-#if GTK_MINOR_VERSION < 18
-  alloc = widget->allocation;
-#else
   gtk_widget_get_allocation(widget, &alloc);
-#endif
 
   gtk_window_move( GTK_WINDOW( mthis->popup_window ), -500, -500 );
   gtk_widget_set_size_request( mthis->popup_window, alloc.width, 20 );
@@ -1381,28 +1380,19 @@ simo_combo_box_popup( SimoComboBox *combo_box )
 
   //grab_on_window( mthis->popup_window->window, gtk_get_current_event_time() );
   //	grab_on_window( GTK_WIDGET(combo_box->treeview)->window, gtk_get_current_event_time() );
-#if GTK_MINOR_VERSION < 18
-  if (!GTK_WIDGET_HAS_FOCUS(combo_box->treeview)) {
-    GTK_WIDGET_SET_FLAGS(combo_box->treeview, GTK_CAN_DEFAULT);
-    gdk_keyboard_grab(gtk_widget_get_window(mthis->popup_window),
-                      FALSE, GDK_CURRENT_TIME);
-    gtk_widget_grab_focus(GTK_WIDGET(combo_box->treeview));
-  }
-#else
   if (!gtk_widget_has_focus(GTK_WIDGET(combo_box->treeview))) {
     gtk_widget_set_can_default(GTK_WIDGET(combo_box->treeview), TRUE);
-    gdk_keyboard_grab(gtk_widget_get_window(mthis->popup_window),
-                      FALSE, GDK_CURRENT_TIME);
+    //gdk_keyboard_grab(gtk_widget_get_window(mthis->popup_window),
+    //                  FALSE, GDK_CURRENT_TIME);
     gtk_widget_grab_focus(GTK_WIDGET(combo_box->treeview));
   }
-#endif
 
   gtk_grab_add( mthis->popup_window );
-  gdk_pointer_grab( gtk_widget_get_window(mthis->popup_window), TRUE,
-                    GDK_BUTTON_PRESS_MASK |
-                    GDK_BUTTON_RELEASE_MASK |
-                    GDK_POINTER_MOTION_MASK,
-                    NULL, NULL, GDK_CURRENT_TIME );
+  //gdk_pointer_grab( gtk_widget_get_window(mthis->popup_window), TRUE,
+  //                  GDK_BUTTON_PRESS_MASK |
+  //                  GDK_BUTTON_RELEASE_MASK |
+  //                  GDK_POINTER_MOTION_MASK,
+  //                  NULL, NULL, GDK_CURRENT_TIME );
   //	gtk_grab_add( combo_box->treeview );
 
   //	gtk_widget_grab_focus( mthis->parent );
@@ -1483,11 +1473,7 @@ simo_combo_box_treeview_release_event( GtkWidget *widget,
   GtkTreeViewColumn *col;
   GtkAllocation      alloc;
 
-#if GTK_MINOR_VERSION < 18
-  alloc = widget->allocation;
-#else
   gtk_widget_get_allocation(widget, &alloc);
-#endif
 
   if( (event->x > alloc.x) &&
       (event->x < (alloc.x + alloc.width)) &&
@@ -1896,8 +1882,8 @@ void
 simo_combo_box_set_entry_font(SimoComboBox         *combo_box,
                               PangoFontDescription *fontDesc)
 {
-  gtk_widget_modify_font(combo_box->entry,   fontDesc);
-  gtk_widget_modify_font(combo_box->txtview, fontDesc);
+  gtk_widget_override_font(combo_box->entry,   fontDesc);
+  gtk_widget_override_font(combo_box->txtview, fontDesc);
   combo_box->priv->cached_line_height = 0;
 }
 
@@ -1941,7 +1927,7 @@ simo_combo_box_txtview_keypress_cb(GtkWidget   *widget,
                                    GdkEventKey *evt,
                                    gpointer    data)
 {
-  if ((evt->keyval == GDK_Return || evt->keyval == GDK_KP_Enter)
+  if ((evt->keyval == GDK_KEY_Return || evt->keyval == GDK_KEY_KP_Enter)
       && !(evt->state & GDK_MOD1_MASK)
       && !(evt->state & GDK_SHIFT_MASK)
       && !(evt->state & GDK_CONTROL_MASK)) {
@@ -2098,18 +2084,23 @@ gboolean simo_combo_box_set_spell(SimoComboBox  *combo,
 #ifdef HAVE_GTKSPELL
   if (combo->priv->gtkspell) {   /* We already have one */
     if (!use_spell) {
-      gtkspell_detach(combo->priv->gtkspell);
+      gtk_spell_checker_detach(combo->priv->gtkspell);
       combo->priv->gtkspell = NULL;
       return TRUE;
     } else {
-      return gtkspell_set_language(combo->priv->gtkspell, language, error);
+      return gtk_spell_checker_set_language(combo->priv->gtkspell,
+                                            language, error);
     }
   } else { /* We don't have one */
     if (use_spell) {
-      combo->priv->gtkspell = gtkspell_new_attach(GTK_TEXT_VIEW(combo->txtview),
-                                                  language,
-                                                  error);
-      if (!combo->priv->gtkspell) {
+      gboolean success;
+
+      combo->priv->gtkspell = gtk_spell_checker_new();
+      gtk_spell_checker_attach(combo->priv->gtkspell,
+                               GTK_TEXT_VIEW(combo->txtview));
+      success = gtk_spell_checker_set_language(combo->priv->gtkspell,
+                                               language, error);
+      if (!success) {
         return FALSE;
       }
       return TRUE;
