@@ -1,7 +1,5 @@
-/* $Id: sound.c 1143 2011-02-13 16:13:58Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +23,7 @@
 
 #include <stdlib.h>
 #include <string.h>
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
