@@ -1,7 +1,5 @@
-/* $Id: timers.c 1176 2011-05-28 14:43:45Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +21,7 @@
 #  include <kcconfig.h>
 #endif
 
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <locale.h>
 #include <gtk/gtk.h>
 
@@ -54,7 +52,7 @@ run_timer(gpointer data)
   }
 
   world_for_perl = world;
-  parse_commands(world, timer->action, strlen(timer->action));
+  parse_commands(world, timer->action);
   we_timer_update_timer(world, timer);
   world_for_perl = currentWorld;
 
@@ -171,7 +169,7 @@ list_timers(World *world, Plugin *plugin)
 
   /* Calculate maximum size necessary for the interval and repeat
      count fields */
-  interval_width = 3;
+  interval_width = 5;
   repeat_width   = 5;
   timerptr = world->timers;
   while (timerptr) {
@@ -180,7 +178,7 @@ list_timers(World *world, Plugin *plugin)
 
     timer = (Timer *) timerptr->data;
 
-    this_int_width = (guint) log10(timer->interval) + 1;
+    this_int_width = (guint) log10(timer->interval) + 5;
     if (this_int_width > interval_width) {
       interval_width = this_int_width;
     }
@@ -223,7 +221,7 @@ list_timers(World *world, Plugin *plugin)
     timer = (Timer *) timerptr->data;
 
     if (timer->owner_plugin == plugin) {
-      ansitextview_append_stringf(world->gui,"%3d %3.3s %4.4s %*u %*d %-*.*s\n",
+      ansitextview_append_stringf(world->gui,"%3d %3.3s %4.4s %*.3f %*d %-*.*s\n",
                                   i,
                                   timer->enabled ? _("y") : _("n"),
                                   timer->temporary ? _("y") : _("n"),
@@ -246,7 +244,7 @@ save_timer(GString *str, Timer *timer)
     g_string_append_printf(str, "name=\"%s\" ", timer->name);
   }
   g_string_append_printf(str,
-                         "interval=\"%d\" count=\"%d\" enabled=\"%d\" temporary=\"%d\">",
+                         "interval=\"%f\" count=\"%d\" enabled=\"%d\" temporary=\"%d\">",
                          timer->interval,
                          timer->count,
                          timer->enabled,
