@@ -1,7 +1,5 @@
-/* $Id: we_advanced.c 1143 2011-02-13 16:13:58Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +24,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <sys/stat.h>
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <gtk/gtk.h>
 
 #include "kildclient.h"
@@ -389,6 +387,7 @@ show_plugin_toggled_cb(GtkToggleButton *button, gpointer data)
   gtk_tree_model_filter_refilter(world->hooks.OnGetFocus_model_filter);
   gtk_tree_model_filter_refilter(world->hooks.OnLoseFocus_model_filter);
   gtk_tree_model_filter_refilter(world->hooks.OnCloseConnected_model_filter);
+  gtk_tree_model_filter_refilter(world->hooks.OnServerData_model_filter);
 }
 
 
