@@ -1,7 +1,5 @@
-/* $Id: we_colors.c 1143 2011-02-13 16:13:58Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +22,7 @@
 #endif
 
 #include <string.h>
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <gtk/gtk.h>
 
 #include "kildclient.h"
@@ -43,52 +41,45 @@ G_MODULE_EXPORT void set_default_colors_cb(GtkButton *btn, gpointer data);
 GObject *
 get_we_colors_tab(World *world, GtkBuilder *ui_builder)
 {
-  GObject  *panel;
-  GtkTable *table;
-  GObject  *colorBtnAnsi;
-  GObject  *colorBtnDeffore;
-  GObject  *colorBtnDefbold;
-  GObject  *colorBtnDefback;
-  GObject  *colorBtnDefboldback;
-  gchar     btnname[18]; /* 18 is enough to hold the button names */
-  int       i;
+  GObject *panel;
+  GObject *colorBtnAnsi;
+  GObject *colorBtnDeffore;
+  GObject *colorBtnDefbold;
+  GObject *colorBtnDefback;
+  GObject *colorBtnDefboldback;
+  gchar    btnname[18]; /* 18 is enough to hold the button names */
+  int      i;
 
   panel = gtk_builder_get_object(ui_builder, "we_panel_colors");
 
   /* Ansi colors category */
-  table = GTK_TABLE(gtk_builder_get_object(ui_builder, "table_ansi_colors"));
-  gtk_table_set_col_spacing(table, 2, 32);
-
   for (i = 0; i < 16; ++i) {
     sprintf(btnname, "colorBtnAnsi[%d]", i);
     colorBtnAnsi = gtk_builder_get_object(ui_builder, btnname);
-    gtk_color_button_set_color(GTK_COLOR_BUTTON(colorBtnAnsi),
+    gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(colorBtnAnsi),
                                &world->ansicolors[i]);
     g_object_set_data(colorBtnAnsi, "colorptr", &world->ansicolors[i]);
   }
 
   /* Default colors category */
-  table = GTK_TABLE(gtk_builder_get_object(ui_builder, "table_default_colors"));
-  gtk_table_set_col_spacing(table, 2, 32);
-
   colorBtnDeffore = gtk_builder_get_object(ui_builder, "colorBtnDeffore");
-  gtk_color_button_set_color(GTK_COLOR_BUTTON(colorBtnDeffore),
+  gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(colorBtnDeffore),
                              world->deffore);
   g_object_set_data(colorBtnDeffore, "colorptr", world->deffore);
 
   colorBtnDefbold = gtk_builder_get_object(ui_builder, "colorBtnDefbold");
-  gtk_color_button_set_color(GTK_COLOR_BUTTON(colorBtnDefbold),
+  gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(colorBtnDefbold),
                              world->defbold);
   g_object_set_data(colorBtnDefbold, "colorptr", world->defbold);
 
   colorBtnDefback = gtk_builder_get_object(ui_builder, "colorBtnDefback");
-  gtk_color_button_set_color(GTK_COLOR_BUTTON(colorBtnDefback),
+  gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(colorBtnDefback),
                              world->defback);
   g_object_set_data(colorBtnDefback, "colorptr", world->defback);
 
   colorBtnDefboldback = gtk_builder_get_object(ui_builder,
                                                "colorBtnDefboldback");
-  gtk_color_button_set_color(GTK_COLOR_BUTTON(colorBtnDefboldback),
+  gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(colorBtnDefboldback),
                              world->defboldback);
   g_object_set_data(colorBtnDefboldback, "colorptr", world->defboldback);
 
@@ -101,10 +92,9 @@ color_changed_cb(GtkColorButton *btn, gpointer data)
 {
   World    *world = (World *) data;
   WorldGUI *gui   = world->gui;
-  GdkColor *color = (GdkColor *) g_object_get_data(G_OBJECT(btn),
-                                                   "colorptr");
+  GdkRGBA  *color = (GdkRGBA *) g_object_get_data(G_OBJECT(btn), "colorptr");
 
-  gtk_color_button_get_color(btn, color);
+  gtk_color_chooser_get_rgba(GTK_COLOR_CHOOSER(btn), color);
   if (gui) {
     ansitextview_update_color_tags(gui, gui->world);
   }
@@ -114,46 +104,46 @@ color_changed_cb(GtkColorButton *btn, gpointer data)
 void
 set_default_colors_cb(GtkButton *btn, gpointer data)
 {
-  World          *world = (World *) data;
-  WorldGUI       *gui   = world->gui;
-  GtkBuilder     *ui_builder;
-  GtkColorButton *colorBtnAnsi;
-  GtkColorButton *colorBtnDeffore;
-  GtkColorButton *colorBtnDefbold;
-  GtkColorButton *colorBtnDefback;
-  GtkColorButton *colorBtnDefboldback;
-  gchar           btnname[18]; /* 18 is enough to hold the button names */
-  int             i;
+  World           *world = (World *) data;
+  WorldGUI        *gui   = world->gui;
+  GtkBuilder      *ui_builder;
+  GtkColorChooser *colorBtnAnsi;
+  GtkColorChooser *colorBtnDeffore;
+  GtkColorChooser *colorBtnDefbold;
+  GtkColorChooser *colorBtnDefback;
+  GtkColorChooser *colorBtnDefboldback;
+  gchar            btnname[18]; /* 18 is enough to hold the button names */
+  int              i;
 
   ui_builder = world->ui_builder;
 
   colorBtnDeffore
-    = GTK_COLOR_BUTTON(gtk_builder_get_object(ui_builder, "colorBtnDeffore"));
+    = GTK_COLOR_CHOOSER(gtk_builder_get_object(ui_builder, "colorBtnDeffore"));
   colorBtnDefbold
-    = GTK_COLOR_BUTTON(gtk_builder_get_object(ui_builder, "colorBtnDefbold"));
+    = GTK_COLOR_CHOOSER(gtk_builder_get_object(ui_builder, "colorBtnDefbold"));
   colorBtnDefback
-    = GTK_COLOR_BUTTON(gtk_builder_get_object(ui_builder, "colorBtnDefback"));
+    = GTK_COLOR_CHOOSER(gtk_builder_get_object(ui_builder, "colorBtnDefback"));
   colorBtnDefboldback
-    = GTK_COLOR_BUTTON(gtk_builder_get_object(ui_builder, "colorBtnDefboldback"));
+    = GTK_COLOR_CHOOSER(gtk_builder_get_object(ui_builder, "colorBtnDefboldback"));
 
-  memcpy(world->deffore, &deffore, sizeof(GdkColor));
-  gtk_color_button_set_color(colorBtnDeffore, world->deffore);
+  memcpy(world->deffore, &deffore, sizeof(GdkRGBA));
+  gtk_color_chooser_set_rgba(colorBtnDeffore, world->deffore);
 
-  memcpy(world->defbold, &defbold, sizeof(GdkColor));
-  gtk_color_button_set_color(colorBtnDefbold, world->defbold);
+  memcpy(world->defbold, &defbold, sizeof(GdkRGBA));
+  gtk_color_chooser_set_rgba(colorBtnDefbold, world->defbold);
 
-  memcpy(world->defback, &defback, sizeof(GdkColor));
-  gtk_color_button_set_color(colorBtnDefback, world->defback);
+  memcpy(world->defback, &defback, sizeof(GdkRGBA));
+  gtk_color_chooser_set_rgba(colorBtnDefback, world->defback);
 
-  memcpy(world->defboldback, &defboldback, sizeof(GdkColor));
-  gtk_color_button_set_color(colorBtnDefboldback, world->defboldback);
+  memcpy(world->defboldback, &defboldback, sizeof(GdkRGBA));
+  gtk_color_chooser_set_rgba(colorBtnDefboldback, world->defboldback);
 
-  memcpy(world->ansicolors, &defansi, 16*sizeof(GdkColor));
+  memcpy(world->ansicolors, &defansi, 16*sizeof(GdkRGBA));
   for (i = 0; i < 16; ++i) {
     sprintf(btnname, "colorBtnAnsi[%d]", i);
-    colorBtnAnsi = GTK_COLOR_BUTTON(gtk_builder_get_object(ui_builder,
-                                                           btnname));
-    gtk_color_button_set_color(colorBtnAnsi, &world->ansicolors[i]);
+    colorBtnAnsi = GTK_COLOR_CHOOSER(gtk_builder_get_object(ui_builder,
+                                                            btnname));
+    gtk_color_chooser_set_rgba(colorBtnAnsi, &world->ansicolors[i]);
   }
 
   if (gui) {
