@@ -1,7 +1,5 @@
-/* $Id: we_generic_guied.c 1143 2011-02-13 16:13:58Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +22,7 @@
 #endif
 
 #include <string.h>
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
@@ -96,9 +94,9 @@ we_guied_export_cb(GtkButton *button, gpointer data)
   filechooser = gtk_file_chooser_dialog_new(_("Export"),
                                             GTK_WINDOW(world->dlgEditWorld),
                                             GTK_FILE_CHOOSER_ACTION_SAVE,
-                                            GTK_STOCK_CANCEL,
+                                            _("_Cancel"),
                                             GTK_RESPONSE_CANCEL,
-                                            GTK_STOCK_SAVE,
+                                            _("_Save"),
                                             GTK_RESPONSE_YES,
                                             NULL);
   gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(filechooser),
@@ -294,6 +292,11 @@ we_guied_do_export(World *world, const gchar *file, GtkBuilder *ui_builder)
                                  "OnCloseConnected",
                                  GTK_TREE_MODEL(world->hooks.OnCloseConnected_model_filter),
                                  ui_builder);
+  we_guied_export_selected_hooks(file_contents,
+                                 world->selected_OnServerData_hooks,
+                                 "OnServerData",
+                                 GTK_TREE_MODEL(world->hooks.OnServerData_model_filter),
+                                 ui_builder);
   g_string_append(file_contents, "  </hooksv2>\n");
 
   g_string_append(file_contents, "</kcworld-export>\n");
@@ -363,9 +366,9 @@ we_guied_import_cb(GtkButton *button, gpointer data)
   filechooser = gtk_file_chooser_dialog_new(_("Import"),
                                             GTK_WINDOW(world->dlgEditWorld),
                                             GTK_FILE_CHOOSER_ACTION_OPEN,
-                                            GTK_STOCK_CANCEL,
+                                            _("_Cancel"),
                                             GTK_RESPONSE_CANCEL,
-                                            GTK_STOCK_OPEN,
+                                            _("_Open"),
                                             GTK_RESPONSE_YES,
                                             NULL);
   if (gtk_dialog_run(GTK_DIALOG(filechooser)) == GTK_RESPONSE_YES) {
@@ -472,12 +475,20 @@ we_guied_do_import(World *world, const gchar *filename)
                       (UpdateFunc) we_hook_insert_hook,
                       world,
                       "OnCloseConnected");
+  we_guied_try_import(&world->hooks.OnServerData,
+                      newworld->hooks.OnServerData,
+                      &world->hooks.OnServerData_pos,
+                      (UpdateFunc) we_hook_insert_hook,
+                      world,
+                      "OnServerData");
 
   /* This can only be done after the importing because it uses a Perl
      interpreter. The ones that have already the precomputed RE's are
      not changed. */
-  trigger_precompute_res(world);
-  alias_precompute_res(world);
+  if (world->connected_at_least_once) {
+    trigger_precompute_res(world);
+    alias_precompute_res(world);
+  }
 
   /* Since we used the lists directly, we set them to NULL to avoid
      freeing them. */
@@ -493,6 +504,7 @@ we_guied_do_import(World *world, const gchar *filename)
   newworld->hooks.OnGetFocus       = NULL;
   newworld->hooks.OnLoseFocus      = NULL;
   newworld->hooks.OnCloseConnected = NULL;
+  newworld->hooks.OnServerData     = NULL;
   free_world(newworld);
 }
 
@@ -674,7 +686,7 @@ we_guied_text_func(GtkTreeViewColumn *column,
     break;
 
   case TIMER_INTERVAL:
-    g_snprintf(printed_value, 100, "%d", ((Timer *) object)->interval);
+    g_snprintf(printed_value, 100, "%.3f", ((Timer *) object)->interval);
     value = printed_value;
     break;
 
@@ -795,13 +807,13 @@ we_guied_bool_col_toggled_cb(GtkCellRendererToggle *renderer,
                                            "forworld");
       Trigger *trigger = (Trigger *) object;
       if (world->perl_interpreter) {
-        PERL_SET_CONTEXT(world->perl_interpreter);
+        GRAB_PERL(world);
         if (trigger->pattern_re) {
           SvREFCNT_dec(trigger->pattern_re);
         }
-        trigger->pattern_re = precompute_re(world->perl_interpreter,
-                                            trigger->pattern,
+        trigger->pattern_re = precompute_re(trigger->pattern,
                                             trigger->ignore_case);
+        RELEASE_PERL(world);
       }
     }
     if (column == ALIAS_ICASE) {
@@ -809,13 +821,13 @@ we_guied_bool_col_toggled_cb(GtkCellRendererToggle *renderer,
                                        "forworld");
       Alias *alias = (Alias *) object;
       if (world->perl_interpreter) {
-        PERL_SET_CONTEXT(world->perl_interpreter);
+        GRAB_PERL(world);
         if (alias->pattern_re) {
           SvREFCNT_dec(alias->pattern_re);
         }
-        alias->pattern_re = precompute_re(world->perl_interpreter,
-                                          alias->pattern,
+        alias->pattern_re = precompute_re(alias->pattern,
                                           alias->ignore_case);
+        RELEASE_PERL(world);
       }
     }
 
@@ -941,8 +953,6 @@ we_guied_confirm_delete_dialog_new(GtkWindow   *parent,
                                    const gchar *plural)
 {
   GtkWidget *msgdlg;
-  GtkWidget *dlgbutton;
-  GtkWidget *dlgbuttonimage;
 
   if (n == 1) {
     msgdlg = gtk_message_dialog_new(parent,
@@ -961,21 +971,10 @@ we_guied_confirm_delete_dialog_new(GtkWindow   *parent,
                                     plural);
   }
   gtk_window_set_title(GTK_WINDOW(msgdlg), _("Confirm deletion"));
-  dlgbutton = gtk_button_new_with_label(_("Do not delete"));
-  dlgbuttonimage = gtk_image_new_from_stock(GTK_STOCK_NO,
-                                            GTK_ICON_SIZE_MENU);
-  gtk_button_set_image(GTK_BUTTON(dlgbutton), dlgbuttonimage);
-  gtk_widget_show_all(dlgbutton);
-  gtk_dialog_add_action_widget(GTK_DIALOG(msgdlg),
-                               dlgbutton, GTK_RESPONSE_NO);
-
-  dlgbutton = gtk_button_new_with_label(_("Delete"));
-  dlgbuttonimage = gtk_image_new_from_stock(GTK_STOCK_DELETE,
-                                            GTK_ICON_SIZE_MENU);
-  gtk_button_set_image(GTK_BUTTON(dlgbutton), dlgbuttonimage);
-  gtk_widget_show_all(dlgbutton);
-  gtk_dialog_add_action_widget(GTK_DIALOG(msgdlg),
-                               dlgbutton, GTK_RESPONSE_YES);
+  gtk_dialog_add_buttons(GTK_DIALOG(msgdlg),
+                         _("Do not delete"), GTK_RESPONSE_NO,
+                         _("Delete"), GTK_RESPONSE_YES,
+                         NULL);
 
   return msgdlg;
 }
