@@ -1,7 +1,5 @@
-/* $Id: we_hooks.c 1153 2011-03-20 15:41:50Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +22,7 @@
 #endif
 
 #include <string.h>
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
@@ -38,8 +36,8 @@
 static void          create_we_hooks_models(World *world);
 static GtkTreeModel *create_we_hooks_model(GSList *hookptr);
 static void          we_hook_configure_view(GtkTreeView *view);
-static void          we_hook_combo_changed_cb(GtkComboBox *combo,
-                                              gpointer     data);
+static void          we_hook_combo_changed_cb(GtkComboBoxText *combo,
+                                              gpointer         data);
 static void          we_hook_save_currently_selected(GtkTreeModel     *model,
                                                      GtkTreeSelection *selection);
 static void          we_hook_add_cb(GtkButton *button, gpointer data);
@@ -75,10 +73,10 @@ get_we_hooks_tab(World *world)
   GObject          *btnExport;
   GtkTreeSelection *selection;
 
-  GtkWidget   *label;
-  GtkComboBox *cmbWhichHook;
-  GtkWidget   *hboxWhich;
-  GtkBox      *vbox_int;
+  GtkWidget       *label;
+  GtkComboBoxText *cmbWhichHook;
+  GtkWidget       *hboxWhich;
+  GtkBox          *vbox_int;
 
   /* Create the dialog */
   ui_builder = gtk_builder_new();
@@ -130,22 +128,23 @@ get_we_hooks_tab(World *world)
 
   /* The manually added combo for selecting which hook */
   label = gtk_label_new_with_mnemonic(_("_Event:"));
-  cmbWhichHook = GTK_COMBO_BOX(gtk_combo_box_new_text());
+  cmbWhichHook = GTK_COMBO_BOX_TEXT(gtk_combo_box_text_new());
   g_signal_connect(G_OBJECT(cmbWhichHook), "changed",
                    G_CALLBACK(we_hook_combo_changed_cb), viewObjects);
   gtk_label_set_mnemonic_widget(GTK_LABEL(label), GTK_WIDGET(cmbWhichHook));
-  hboxWhich = gtk_hbox_new(FALSE, 8);
+  hboxWhich = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 8);
   gtk_box_pack_start(GTK_BOX(hboxWhich), label, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(hboxWhich), GTK_WIDGET(cmbWhichHook),
                      FALSE, FALSE, 0);
 
-  gtk_combo_box_append_text(cmbWhichHook, "OnConnect");
-  gtk_combo_box_append_text(cmbWhichHook, "OnDisconnect");
-  gtk_combo_box_append_text(cmbWhichHook, "OnReceivedText");
-  gtk_combo_box_append_text(cmbWhichHook, "OnSentCommand");
-  gtk_combo_box_append_text(cmbWhichHook, "OnGetFocus");
-  gtk_combo_box_append_text(cmbWhichHook, "OnLoseFocus");
-  gtk_combo_box_append_text(cmbWhichHook, "OnCloseConnected");
+  gtk_combo_box_text_append_text(cmbWhichHook, "OnConnect");
+  gtk_combo_box_text_append_text(cmbWhichHook, "OnDisconnect");
+  gtk_combo_box_text_append_text(cmbWhichHook, "OnReceivedText");
+  gtk_combo_box_text_append_text(cmbWhichHook, "OnSentCommand");
+  gtk_combo_box_text_append_text(cmbWhichHook, "OnGetFocus");
+  gtk_combo_box_text_append_text(cmbWhichHook, "OnLoseFocus");
+  gtk_combo_box_text_append_text(cmbWhichHook, "OnCloseConnected");
+  gtk_combo_box_text_append_text(cmbWhichHook, "OnServerData");
 
   vbox_int
     = GTK_BOX(gtk_builder_get_object(ui_builder,
@@ -167,7 +166,7 @@ get_we_hooks_tab(World *world)
                    G_CALLBACK(we_guied_selection_changed_cb), NULL);
 
   /* Select first, will cause a model to be associated with the view */
-  gtk_combo_box_set_active(cmbWhichHook, 0);
+  gtk_combo_box_set_active(GTK_COMBO_BOX(cmbWhichHook), 0);
 
   gtk_builder_connect_signals(ui_builder, world);
   /* The callback for the Export buttons in the Hooks tab is a
@@ -270,6 +269,18 @@ create_we_hooks_models(World *world)
                                          we_guied_plugin_filter_func,
                                          world,
                                          NULL);
+
+  world->hooks.OnServerData_model = create_we_hooks_model(world->hooks.OnServerData);
+  g_object_set_data(G_OBJECT(world->hooks.OnServerData_model),
+                    "forevent", "OnServerData");
+  g_object_set_data(G_OBJECT(world->hooks.OnServerData_model),
+                    "selected_list", &world->selected_OnServerData_hooks);
+  world->hooks.OnServerData_model_filter = GTK_TREE_MODEL_FILTER(gtk_tree_model_filter_new(world->hooks.OnServerData_model, NULL));
+  g_object_unref(G_OBJECT(world->hooks.OnServerData_model));
+  gtk_tree_model_filter_set_visible_func(GTK_TREE_MODEL_FILTER(world->hooks.OnServerData_model_filter),
+                                         we_guied_plugin_filter_func,
+                                         world,
+                                         NULL);
 }
 
 
@@ -368,8 +379,8 @@ we_hook_configure_view(GtkTreeView *view)
 
 static
 void
-we_hook_combo_changed_cb(GtkComboBox *combo,
-                         gpointer     data)
+we_hook_combo_changed_cb(GtkComboBoxText *combo,
+                         gpointer         data)
 {
   GtkTreeView       *view;
   World             *world;
@@ -388,7 +399,7 @@ we_hook_combo_changed_cb(GtkComboBox *combo,
                                     gtk_tree_view_get_selection(view));
   }
 
-  newevent = gtk_combo_box_get_active_text(combo);
+  newevent = gtk_combo_box_text_get_active_text(combo);
   newmodel = get_hook_model(world, newevent);
   gtk_tree_view_set_model(view, newmodel);
   selected_list = g_object_get_data(G_OBJECT(newmodel), "selected_list");
@@ -700,7 +711,7 @@ we_hook_view_keypress_cb(GtkWidget   *widget,
                          GdkEventKey *evt,
                          gpointer     data)
 {
-  if (evt->keyval == GDK_Delete || evt->keyval == GDK_KP_Delete) {
+  if (evt->keyval == GDK_KEY_Delete || evt->keyval == GDK_KEY_KP_Delete) {
     we_hook_delete_cb(NULL, widget);
     return TRUE;
   }
