@@ -1,7 +1,5 @@
-/* $Id: we_logging.c 1143 2011-02-13 16:13:58Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +22,7 @@
 #endif
 
 #include <string.h>
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <gtk/gtk.h>
 #include <gmodule.h>
 
@@ -49,20 +47,10 @@ GObject *
 get_we_logging_tab(GtkBuilder *ui_builder, World *world)
 {
   GObject *panel;
-  GObject *btnStart;
 
   panel = gtk_builder_get_object(ui_builder, "we_panel_logging");
 
-  if (world != default_world) {
-    GtkWidget *image;
-
-    /* FIXME: Someday it should be possible to do this directly in the
-       XML file. */
-    btnStart = gtk_builder_get_object(ui_builder, "btnLogStart");
-    image = gtk_image_new_from_stock(GTK_STOCK_MEDIA_PLAY,
-                                     GTK_ICON_SIZE_BUTTON);
-    gtk_button_set_image(GTK_BUTTON(btnStart), image);
-  } else {
+  if (world == default_world) {
     /* These parts make no sense for the default world, so let's hide them */
     GObject *vbox_log_status;
     GObject *vbox_log_file;
@@ -223,7 +211,7 @@ we_log_start_cb(GtkWidget *widget, gpointer data)
                                                GTK_DIALOG_DESTROY_WITH_PARENT,
                                                GTK_MESSAGE_ERROR,
                                                GTK_BUTTONS_CLOSE,
-                                               errmsg);
+                                               "%s", errmsg);
     gtk_window_set_title(GTK_WINDOW(msgdlg), _("Logging not started"));
     gtk_dialog_run(GTK_DIALOG(msgdlg));
     gtk_widget_destroy(msgdlg);
@@ -257,9 +245,9 @@ select_logfile_cb(gpointer data, GtkWidget *widget)
   filedlg = gtk_file_chooser_dialog_new(_("Select log file"),
                                         dlgEditWorld,
                                         GTK_FILE_CHOOSER_ACTION_SAVE,
-                                        GTK_STOCK_CANCEL,
+                                        _("_Cancel"),
                                         GTK_RESPONSE_CANCEL,
-                                        GTK_STOCK_OK,
+                                        _("_OK"),
                                         GTK_RESPONSE_OK,
                                         NULL);
   gtk_dialog_set_default_response(GTK_DIALOG(filedlg), GTK_RESPONSE_OK);
