@@ -1,7 +1,5 @@
-/* $Id: we_misc.c 1143 2011-02-13 16:13:58Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +22,7 @@
 #endif
 
 #include <string.h>
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <gtk/gtk.h>
 #include <gmodule.h>
 
@@ -37,6 +35,9 @@
 /***********************
  * Function prototypes *
  ***********************/
+G_MODULE_EXPORT void scroll_output_changed_cb(GtkToggleButton *widget,
+                                              gpointer         data);
+
 
 
 GObject *
@@ -109,7 +110,6 @@ fill_we_misc_tab(World *world, GtkBuilder *ui_builder)
 void
 update_we_misc_parameters(World *world, GtkBuilder *ui_builder)
 {
-  GtkToggleButton *chkScrollOutput;
   GtkSpinButton   *txtBufferLines;
   GtkToggleButton *radNameDisplayWorld;
   GtkToggleButton *radNameDisplayWorldChar;
@@ -121,8 +121,6 @@ update_we_misc_parameters(World *world, GtkBuilder *ui_builder)
 
 
   /* Get widgets */
-  chkScrollOutput
-    = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder, "chkScrollOutput"));
   txtBufferLines
     = GTK_SPIN_BUTTON(gtk_builder_get_object(ui_builder, "txtBufferLines"));
   radNameDisplayWorld
@@ -136,8 +134,6 @@ update_we_misc_parameters(World *world, GtkBuilder *ui_builder)
   txtNameDisplayCustom
     = GTK_ENTRY(gtk_builder_get_object(ui_builder, "txtNameDisplayCustom"));
 
-  world->scrollOutput = gtk_toggle_button_get_active(chkScrollOutput);
-
   world->buffer_lines = gtk_spin_button_get_value_as_int(txtBufferLines);
 
   if (gtk_toggle_button_get_active(radNameDisplayWorld)) {
@@ -157,3 +153,13 @@ update_we_misc_parameters(World *world, GtkBuilder *ui_builder)
     world->name_display_pattern = g_strdup(newval);
   }
 }
+
+
+void scroll_output_changed_cb(GtkToggleButton *widget,
+                              gpointer         data)
+{
+  World *world = (World *) data;
+
+  world->scrollOutput = gtk_toggle_button_get_active(widget);
+}
+
