@@ -1,7 +1,5 @@
-/* $Id: we_plugins.c 1143 2011-02-13 16:13:58Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +22,7 @@
 #endif
 
 #include <string.h>
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
@@ -207,7 +205,7 @@ we_splugin_add_cb(gpointer data, GtkButton *button)
                                               GTK_DIALOG_MODAL,
                                               GTK_MESSAGE_ERROR,
                                               GTK_BUTTONS_OK,
-                                              error->message);
+                                              "%s", error->message);
       gtk_window_set_title(GTK_WINDOW(dlg), _("Error Loading Plugin"));
       gtk_dialog_run(GTK_DIALOG(dlg));
       gtk_widget_destroy(dlg);
@@ -272,7 +270,7 @@ we_splugin_view_keypress_cb(GtkWidget   *widget, /* The treeview */
                             GdkEventKey *evt,
                             gpointer     data)
 {
-  if (evt->keyval == GDK_Delete || evt->keyval == GDK_KP_Delete) {
+  if (evt->keyval == GDK_KEY_Delete || evt->keyval == GDK_KEY_KP_Delete) {
     we_splugin_remove_cb(widget, NULL);
     return TRUE;
   }
@@ -422,8 +420,9 @@ we_lplugin_ena_toggled_cb(GtkCellRendererToggle *renderer,
     } else {
       cmd = g_strdup_printf("$::world->enaplugin('%s');", plugin->name);
     }
-    PERL_SET_CONTEXT(world->perl_interpreter);
+    GRAB_PERL(world);
     eval_pv(cmd, TRUE);
+    RELEASE_PERL(world);
     g_free(cmd);
   }
 }
@@ -463,7 +462,7 @@ we_lplugin_load_cb(gpointer data, GtkButton *button)
                                               GTK_DIALOG_MODAL,
                                               GTK_MESSAGE_ERROR,
                                               GTK_BUTTONS_OK,
-                                              error->message);
+                                              "%s", error->message);
       gtk_window_set_title(GTK_WINDOW(dlg), _("Error Loading Plugin"));
       gtk_dialog_run(GTK_DIALOG(dlg));
       gtk_widget_destroy(dlg);
@@ -493,8 +492,9 @@ we_lplugin_help_cb(gpointer data, GtkButton *button)
 
     gtk_tree_model_get(model, &iter, WE_GUIED_POINTER, &plugin, -1);
     cmd = g_strdup_printf("%s::help();", plugin->name);
-    PERL_SET_CONTEXT(world->perl_interpreter);
+    GRAB_PERL(world);
     eval_pv(cmd, FALSE);
+    RELEASE_PERL(world);
     g_free(cmd);
 
     /* Show message telling where the help appears. */
@@ -581,9 +581,9 @@ we_plugin_get_load_filechooser(GtkWindow *parent)
   filechooser = gtk_file_chooser_dialog_new(_("Load plugin"),
                                            parent,
                                            GTK_FILE_CHOOSER_ACTION_OPEN,
-                                           GTK_STOCK_CANCEL,
+                                           _("_Cancel"),
                                            GTK_RESPONSE_CANCEL,
-                                           GTK_STOCK_OPEN,
+                                           _("_Open"),
                                            GTK_RESPONSE_YES,
                                            NULL);
   plugindir = get_kildclient_installed_file("plugins");
