@@ -1,7 +1,5 @@
-/* $Id: we_statusbar.c 1143 2011-02-13 16:13:58Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +21,7 @@
 #  include <kcconfig.h>
 #endif
 
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <gtk/gtk.h>
 
 #include "kildclient.h"
@@ -122,7 +120,7 @@ statusfont_set_cb(GtkFontButton *font_btn, gpointer data)
     GtkWidget *widget = GTK_WIDGET(world->gui->lblStatus);
 
     fontDesc = pango_font_description_from_string(world->statusfont);
-    gtk_widget_modify_font(widget, fontDesc);
+    gtk_widget_override_font(widget, fontDesc);
     pango_font_description_free(fontDesc);
   }
 }
