@@ -1,7 +1,5 @@
-/* $Id: we_timers.c 1143 2011-02-13 16:13:58Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +22,7 @@
 #endif
 
 #include <string.h>
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
@@ -449,7 +447,7 @@ edit_timer(World *world, Timer *timer, gboolean editing)
   const gchar     *newname;
   const gchar     *newaction;
   gboolean         newenabled;
-  gint             newinterval;
+  gdouble          newinterval;
 
   ui_builder = gtk_builder_new();
   if (!gtk_builder_add_objects_from_file(ui_builder,
@@ -508,7 +506,7 @@ edit_timer(World *world, Timer *timer, gboolean editing)
       /* Validate */
       newenabled = gtk_toggle_button_get_active(chkEnabled);
 
-      newinterval = gtk_spin_button_get_value_as_int(spnInterval);
+      newinterval = gtk_spin_button_get_value(spnInterval);
       if (newinterval <= 0) {
         GtkWidget *dlg = gtk_message_dialog_new(GTK_WINDOW(dlgEditTimer),
                                                 GTK_DIALOG_MODAL,
@@ -575,8 +573,8 @@ edit_timer(World *world, Timer *timer, gboolean editing)
         }
       }
 
-      timer->interval  = newinterval;
-      timer->enabled = newenabled;
+      timer->interval = newinterval;
+      timer->enabled  = newenabled;
 
       /* We've finished successfully */
       gtk_widget_destroy(dlgEditTimer);
@@ -596,7 +594,7 @@ we_timer_view_keypress_cb(GtkWidget   *widget,
                           GdkEventKey *evt,
                           gpointer     data)
 {
-  if (evt->keyval == GDK_Delete || evt->keyval == GDK_KP_Delete) {
+  if (evt->keyval == GDK_KEY_Delete || evt->keyval == GDK_KEY_KP_Delete) {
     we_timer_delete_cb(NULL, widget);
     return TRUE;
   }
