@@ -1,7 +1,5 @@
-/* $Id: we_triggers.c 1143 2011-02-13 16:13:58Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +22,7 @@
 #endif
 
 #include <string.h>
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
@@ -693,13 +691,13 @@ edit_trigger(World *world, Trigger *trigger)
       trigger->enabled       = gtk_toggle_button_get_active(chkEnabled);
 
       if (need_recompute && world->perl_interpreter) {
-        PERL_SET_CONTEXT(world->perl_interpreter);
+        GRAB_PERL(world);
         if (trigger->pattern_re) {
           SvREFCNT_dec(trigger->pattern_re);
         }
-        trigger->pattern_re = precompute_re(world->perl_interpreter,
-                                            trigger->pattern,
+        trigger->pattern_re = precompute_re(trigger->pattern,
                                             trigger->ignore_case);
+        RELEASE_PERL(world);
       }
 
       /* We've finished successfully */
@@ -821,7 +819,7 @@ we_trigger_view_keypress_cb(GtkWidget   *widget,
                             GdkEventKey *evt,
                             gpointer     data)
 {
-  if (evt->keyval == GDK_Delete || evt->keyval == GDK_KP_Delete) {
+  if (evt->keyval == GDK_KEY_Delete || evt->keyval == GDK_KEY_KP_Delete) {
     we_trigger_delete_cb(NULL, widget);
     return TRUE;
   }
