@@ -1,7 +1,5 @@
-/* $Id: worldeditor.c 1143 2011-02-13 16:13:58Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +24,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <sys/stat.h>
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <gtk/gtk.h>
 
 #include "kildclient.h"
@@ -458,7 +456,7 @@ new_descriptive_panel(const char *title, const char *message)
   GtkWidget *label;
   gchar     *temp;
 
-  vbox = gtk_vbox_new(FALSE, 6);
+  vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 6);
 
   temp = g_strdup_printf("<span weight=\"bold\" size=\"larger\">%s</span>",
                          title);
