@@ -1,7 +1,5 @@
-/* $Id: worldgui.c 1176 2011-05-28 14:43:45Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +24,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #ifdef __WIN32__
@@ -80,13 +78,11 @@ static gboolean  textview_button_press_cb(GtkWidget      *text_view,
                                           gpointer        data);
 static gboolean  act_if_link(WorldGUI       *gui,
                              GtkTextIter    *iter,
+                             GtkWidget      *widget,
                              GdkEventButton *event);
 static gboolean  textview_motion_notify_event(GtkWidget      *widget,
                                               GdkEventMotion *event,
                                               gpointer        data);
-static gboolean  textview_visibility_notify_event(GtkWidget          *widget,
-                                                  GdkEventVisibility *event,
-                                                  gpointer            data);
 static void      set_cursor_if_appropriate(GtkTextView *text_view,
                                            WorldGUI    *gui,
                                            gint         x,
@@ -117,7 +113,7 @@ static void      scrolled_win_value_changed_cb(GtkAdjustment *adjustment,
                                                gpointer       data);
 static gboolean  set_split_to_zero(gpointer data);
 /* XML UI Signals */
-G_MODULE_EXPORT void menu_popup_url_copy(GtkAction *action, gchar *url);
+G_MODULE_EXPORT void menu_popup_url_copy(GtkMenuItem *menu, gchar *url);
 
 
 
@@ -130,7 +126,6 @@ world_gui_new(gboolean is_for_mud)
      If FALSE, it is a KCWin gui, and it has less features.
   */
   WorldGUI      *gui;
-  GtkWidget     *iconClearWidget;
   GtkWidget     *frame;
   GtkAdjustment *verticalAdjust;
 
@@ -139,18 +134,19 @@ world_gui_new(gboolean is_for_mud)
 
   PangoFontDescription *fontDesc;
   GtkTextIter           end_iter;
-  GdkColor              black = { 0, 0, 0, 0 };
+  GdkRGBA               black = { 0,    0,    0,    1 };
+  GdkRGBA               blue  = { 0.58, 0.79, 0.93, 1 };
 
 
   /* First, create the cursors if this is the first run */
   if (!hand_cursor) {
-    hand_cursor = gdk_cursor_new (GDK_HAND2);
-    regular_cursor = gdk_cursor_new (GDK_XTERM);
+    hand_cursor = gdk_cursor_new(GDK_HAND2);
+    regular_cursor = gdk_cursor_new(GDK_XTERM);
   }
 
   gui = g_new0(WorldGUI, 1);
 
-  gui->vbox = gtk_vbox_new(FALSE, 0);
+  gui->vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
   g_object_set_data(G_OBJECT(gui->vbox), "gui", gui);
 
   gui->scrolled_win = GTK_SCROLLED_WINDOW(gtk_scrolled_window_new(NULL, NULL));
@@ -188,7 +184,7 @@ world_gui_new(gboolean is_for_mud)
                       GTK_WIDGET(gui->txtViewScroll));
 
     /* Create split pane */
-    gui->split_pane = GTK_PANED(gtk_vpaned_new());
+    gui->split_pane = GTK_PANED(gtk_paned_new(GTK_ORIENTATION_VERTICAL));
     gtk_paned_pack1(gui->split_pane, GTK_WIDGET(gui->scrwinScroll), FALSE, TRUE);
     gtk_paned_pack2(gui->split_pane, GTK_WIDGET(gui->scrolled_win), TRUE, TRUE);
     /* For some reason, we cannot set to 0 directly. */
@@ -207,19 +203,16 @@ world_gui_new(gboolean is_for_mud)
 
   if (is_for_mud) {
     /* Search box */
-    GtkWidget *iconCloseWidget;
     GtkWidget *btnClose;
     GtkWidget *label;
 
-    gui->search_box = gtk_hbox_new(FALSE, 6);
+    gui->search_box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6);
     gtk_container_set_border_width(GTK_CONTAINER(gui->search_box), 2);
     g_object_set(G_OBJECT(gui->search_box), "no-show-all", TRUE, NULL);
 
-    btnClose = gtk_button_new();
+    btnClose = gtk_button_new_from_icon_name("window-close",
+                                             GTK_ICON_SIZE_BUTTON);
     gtk_button_set_relief(GTK_BUTTON(btnClose), GTK_RELIEF_NONE);
-    iconCloseWidget = gtk_image_new_from_stock(GTK_STOCK_CLOSE,
-                                               GTK_ICON_SIZE_MENU);
-    gtk_container_add(GTK_CONTAINER(btnClose), iconCloseWidget);
     g_signal_connect(G_OBJECT(btnClose), "clicked",
                      G_CALLBACK(close_search_box_cb), gui);
     gtk_box_pack_start(GTK_BOX(gui->search_box), btnClose, FALSE, FALSE, 0);
@@ -239,9 +232,6 @@ world_gui_new(gboolean is_for_mud)
                        FALSE, FALSE, 0);
 
     gui->btnFindNext = gtk_button_new_with_label(_("Find Next"));
-    gtk_button_set_image(GTK_BUTTON(gui->btnFindNext),
-                         gtk_image_new_from_stock(GTK_STOCK_GO_DOWN,
-                                                  GTK_ICON_SIZE_MENU));
     gtk_button_set_relief(GTK_BUTTON(gui->btnFindNext), GTK_RELIEF_NONE);
     g_signal_connect(G_OBJECT(gui->btnFindNext), "clicked",
                      G_CALLBACK(find_next_cb), gui);
@@ -257,15 +247,13 @@ world_gui_new(gboolean is_for_mud)
   }
 
   /* Command area */
-  gui->commandArea = gtk_hbox_new(FALSE, 4);
+  gui->commandArea = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
   gtk_container_set_border_width(GTK_CONTAINER(gui->commandArea), 2);
 
-  gui->btnClear = gtk_button_new();
-  iconClearWidget = gtk_image_new_from_stock(GTK_STOCK_CLEAR,
-                                             GTK_ICON_SIZE_MENU);
+  gui->btnClear = gtk_button_new_from_icon_name("edit-clear",
+                                                GTK_ICON_SIZE_MENU);
   g_signal_connect(G_OBJECT(gui->btnClear), "clicked",
                    G_CALLBACK(clear_button_cb), gui);
-  gtk_container_add(GTK_CONTAINER(gui->btnClear), iconClearWidget);
   gtk_box_pack_start(GTK_BOX(gui->commandArea), gui->btnClear,
                      FALSE, FALSE, 0);
 
@@ -322,11 +310,11 @@ world_gui_new(gboolean is_for_mud)
   }
 
   /* Set initial colors and state */
-  gui->ta.state.fg_color = gui->ta.saved_state.fg_color
+  gui->ta_state.fg_color = gui->saved_ta_state.fg_color
     = ANSI_DEFAULT_COLOR_IDX;
-  gui->ta.state.bg_color = gui->ta.saved_state.bg_color
+  gui->ta_state.bg_color = gui->saved_ta_state.bg_color
     = ANSI_DEFAULT_COLOR_IDX;
-  gui->ta.state.underline = PANGO_UNDERLINE_NONE;
+  gui->ta_state.underline = PANGO_UNDERLINE_NONE;
 
   /* Create tag for text with attributes */
   gui->ta.underline_tag = gtk_text_buffer_create_tag(gui->txtBuffer,
@@ -366,29 +354,21 @@ world_gui_new(gboolean is_for_mud)
 
   if (is_for_mud) {   /* Special features */
     GtkWidget *btnClose;
-    GtkRcStyle *rcstyle;
-    GtkWidget *iconCloseWidget;
 
     /* Notebook tab */
-    gui->hboxTab = GTK_BOX(gtk_hbox_new(FALSE, 4));
+    gui->hboxTab = GTK_BOX(gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4));
     gui->lblNotebook = GTK_LABEL(gtk_label_new(_("No world")));
     gtk_box_pack_start(gui->hboxTab, GTK_WIDGET(gui->lblNotebook),
                        FALSE, FALSE, 0);
 
-    btnClose = gtk_button_new();
+    btnClose = gtk_button_new_from_icon_name("window-close",
+                                             GTK_ICON_SIZE_BUTTON);
     /* Make the button smaller. (Taken from gedit code) */
     gtk_button_set_focus_on_click(GTK_BUTTON(btnClose), FALSE);
     gtk_button_set_relief(GTK_BUTTON(btnClose), GTK_RELIEF_NONE);
-    rcstyle  = gtk_rc_style_new();
-    rcstyle->xthickness = rcstyle->ythickness = 0;
-    gtk_widget_modify_style(btnClose, rcstyle);
-    g_object_unref(rcstyle);
 
-    iconCloseWidget = gtk_image_new_from_stock(GTK_STOCK_CLOSE,
-                                               GTK_ICON_SIZE_MENU);
     g_signal_connect(G_OBJECT(btnClose), "clicked",
                      G_CALLBACK(tab_close_cb), gui);
-    gtk_container_add(GTK_CONTAINER(btnClose), iconCloseWidget);
     gtk_box_pack_start(GTK_BOX(gui->hboxTab), btnClose,
                        FALSE, FALSE, 0);
 
@@ -397,15 +377,21 @@ world_gui_new(gboolean is_for_mud)
        appears black while the world is being selected, and the font to a
        default font so that the size can be calculated correctly. */
     fontDesc = pango_font_description_from_string(DEFAULT_TERMINAL_FONT);
-    gtk_widget_modify_font(GTK_WIDGET(gui->txtView), fontDesc);
-    gtk_widget_modify_font(GTK_WIDGET(gui->txtViewScroll), fontDesc);
+    gtk_widget_override_font(GTK_WIDGET(gui->txtView), fontDesc);
+    gtk_widget_override_font(GTK_WIDGET(gui->txtViewScroll), fontDesc);
     pango_font_description_free(fontDesc);
-    gtk_widget_modify_base(GTK_WIDGET(gui->txtView),
-                           GTK_STATE_NORMAL,
-                           &black);
-    gtk_widget_modify_base(GTK_WIDGET(gui->txtViewScroll),
-                           GTK_STATE_NORMAL,
-                           &black);
+    gtk_widget_override_background_color(GTK_WIDGET(gui->txtView),
+                                         GTK_STATE_FLAG_NORMAL, &black);
+    gtk_widget_override_background_color(GTK_WIDGET(gui->txtView),
+                                         GTK_STATE_FLAG_SELECTED, &blue);
+    gtk_widget_override_color(GTK_WIDGET(gui->txtView),
+                              GTK_STATE_FLAG_SELECTED, &black);
+    gtk_widget_override_background_color(GTK_WIDGET(gui->txtViewScroll),
+                                         GTK_STATE_FLAG_NORMAL, &black);
+    gtk_widget_override_background_color(GTK_WIDGET(gui->txtViewScroll),
+                                         GTK_STATE_FLAG_SELECTED, &blue);
+    gtk_widget_override_color(GTK_WIDGET(gui->txtViewScroll),
+                              GTK_STATE_FLAG_SELECTED, &black);
 
     /* Detect resizes */
     g_signal_connect(G_OBJECT(gui->txtView), "size-allocate",
@@ -431,10 +417,6 @@ world_gui_new(gboolean is_for_mud)
                      G_CALLBACK(textview_motion_notify_event), gui);
     g_signal_connect(gui->txtViewScroll, "motion-notify-event",
                      G_CALLBACK(textview_motion_notify_event), gui);
-    g_signal_connect(gui->txtView, "visibility-notify-event",
-                     G_CALLBACK(textview_visibility_notify_event), gui);
-    g_signal_connect(gui->txtViewScroll, "visibility-notify-event",
-                     G_CALLBACK(textview_visibility_notify_event), gui);
 
     /* Tooltips */
     g_signal_connect(G_OBJECT(gui->txtView), "query-tooltip",
@@ -443,7 +425,7 @@ world_gui_new(gboolean is_for_mud)
                      G_CALLBACK(worldgui_query_tooltip_cb), gui);
 
     /* Status bar */
-    gui->statusbar_box = GTK_BOX(gtk_hbox_new(FALSE, 4));
+    gui->statusbar_box = GTK_BOX(gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4));
 
     gui->lblStatus = GTK_LABEL(gtk_label_new(_("KildClient ready")));
     gtk_misc_set_alignment(GTK_MISC(gui->lblStatus), 0, 0.5);
@@ -624,7 +606,7 @@ activate_cb(SimoComboBox *cmb, gpointer data)
     const gchar *str;
 
     str = simo_combo_box_get_text(cmb);
-    parse_commands(world, str, strlen(str));
+    parse_commands(world, str);
 
     /* Ignore empty commands and passwords */
     if (strcmp(str, "") != 0 && !world->noecho) {
@@ -671,24 +653,24 @@ keypress_cb(GtkWidget   *widget,
   }
 
   switch (evt->keyval) {
-  case GDK_Left:
-  case GDK_KP_Left:
+  case GDK_KEY_Left:
+  case GDK_KEY_KP_Left:
     if (evt->state & GDK_MOD1_MASK) { /* Alt+left */
       menu_perl_run_cb(NULL, "$world->prev");
       return TRUE;
     }
     break;
 
-  case GDK_Right:
-  case GDK_KP_Right:
+  case GDK_KEY_Right:
+  case GDK_KEY_KP_Right:
     if (evt->state & GDK_MOD1_MASK) { /* Alt+right */
       menu_perl_run_cb(NULL, "$world->next");
       return TRUE;
     }
     break;
 
-  case GDK_Tab:
-  case GDK_ISO_Left_Tab:
+  case GDK_KEY_Tab:
+  case GDK_KEY_ISO_Left_Tab:
     if (evt->state & GDK_CONTROL_MASK) {
       if (evt->state & GDK_SHIFT_MASK) { /* Ctrl+Shift+Tab */
         menu_perl_run_cb(NULL, "$world->prev");
@@ -711,13 +693,13 @@ keypress_cb(GtkWidget   *widget,
     }
     break;
 
-  case GDK_Up:
-  case GDK_KP_Up:
+  case GDK_KEY_Up:
+  case GDK_KEY_KP_Up:
     direction = 0;
     /* Fall through */
 
-  case GDK_Down:
-  case GDK_KP_Down:
+  case GDK_KEY_Down:
+  case GDK_KEY_KP_Down:
     /* Do nothing if option to ignore these keys is active. */
     if (world->ignore_up_down_keys) {
       return FALSE;
@@ -730,13 +712,13 @@ keypress_cb(GtkWidget   *widget,
     prev_or_next_command(world, direction);
     return TRUE;
 
-  case GDK_Page_Up:
-  case GDK_KP_Page_Up:
+  case GDK_KEY_Page_Up:
+  case GDK_KEY_KP_Page_Up:
     direction = -1;
     /* Fall through */
 
-  case GDK_Page_Down:
-  case GDK_KP_Page_Down:
+  case GDK_KEY_Page_Down:
+  case GDK_KEY_KP_Page_Down:
     /* If the scrollback window is visible, move that */
     if (gtk_paned_get_position(gui->split_pane) > 0) {
       adjustment = gtk_scrolled_window_get_vadjustment(gui->scrwinScroll);
@@ -753,8 +735,8 @@ keypress_cb(GtkWidget   *widget,
     gtk_adjustment_changed(adjustment);
     return TRUE;
 
-  case GDK_End:
-  case GDK_KP_End:
+  case GDK_KEY_End:
+  case GDK_KEY_KP_End:
     if (evt->state & GDK_CONTROL_MASK) {
       adjustment = gtk_scrolled_window_get_vadjustment(gui->scrolled_win);
       gtk_adjustment_set_value(adjustment,
@@ -899,8 +881,6 @@ tab_close_cb(GtkWidget *widget, gpointer data)
 
   if (world->connected) {
     GtkWidget *msgdlg;
-    GtkWidget *dlgbutton;
-    GtkWidget *dlgbuttonimage;
     gint       response;
 
     msgdlg = gtk_message_dialog_new(GTK_WINDOW(wndMain),
@@ -909,23 +889,10 @@ tab_close_cb(GtkWidget *widget, gpointer data)
                                     GTK_BUTTONS_NONE,
                                     _("Are you sure you want to close this world?"));
     gtk_window_set_title(GTK_WINDOW(msgdlg), _("Really close?"));
-    /* Keep Open is RESPONSE_NO because of NO close. But we use
-       STOCK_YES to pass a positive image. */
-    dlgbutton = gtk_button_new_with_mnemonic(_("Keep _open"));
-    dlgbuttonimage = gtk_image_new_from_stock(GTK_STOCK_YES,
-                                              GTK_ICON_SIZE_BUTTON);
-    gtk_button_set_image(GTK_BUTTON(dlgbutton), dlgbuttonimage);
-    gtk_dialog_add_action_widget(GTK_DIALOG(msgdlg),
-                                 dlgbutton, GTK_RESPONSE_NO);
-
-    /*    dlgbutton = gtk_button_new_with_label(_("Close"));
-    dlgbuttonimage = gtk_image_new_from_stock(GTK_STOCK_CLOSE,
-                                              GTK_ICON_SIZE_BUTTON);
-    gtk_button_set_image(GTK_BUTTON(dlgbutton), dlgbuttonimage);
-    gtk_widget_show_all(dlgbutton);*/
-    dlgbutton = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
-    gtk_dialog_add_action_widget(GTK_DIALOG(msgdlg),
-                                 dlgbutton, GTK_RESPONSE_YES);
+    gtk_dialog_add_buttons(GTK_DIALOG(msgdlg),
+                           _("Keep _open"), GTK_RESPONSE_NO,
+                           _("_Close"), GTK_RESPONSE_YES,
+                           NULL);
 
     gtk_widget_show_all(msgdlg);
     response = gtk_dialog_run(GTK_DIALOG(msgdlg));
@@ -972,19 +939,19 @@ configure_gui(WorldGUI *gui, World *world)
   GtkCellRenderer      *renderer;
 
   fontDesc = pango_font_description_from_string(world->terminalfont);
-  gtk_widget_modify_font(GTK_WIDGET(gui->txtView), fontDesc);
-  gtk_widget_modify_font(GTK_WIDGET(gui->txtViewScroll), fontDesc);
+  gtk_widget_override_font(GTK_WIDGET(gui->txtView), fontDesc);
+  gtk_widget_override_font(GTK_WIDGET(gui->txtViewScroll), fontDesc);
   pango_font_description_free(fontDesc);
   fontDesc = pango_font_description_from_string(world->entryfont);
   if (gui->world) {
     simo_combo_box_set_entry_font(gui->cmbEntry, fontDesc);
   } else {
-    gtk_widget_modify_font(GTK_WIDGET(gui->txtEntry), fontDesc);
+    gtk_widget_override_font(GTK_WIDGET(gui->txtEntry), fontDesc);
   }
   pango_font_description_free(fontDesc);
   if (gui->lblStatus) {
     fontDesc = pango_font_description_from_string(world->statusfont);
-    gtk_widget_modify_font(GTK_WIDGET(gui->lblStatus), fontDesc);
+    gtk_widget_override_font(GTK_WIDGET(gui->lblStatus), fontDesc);
     pango_font_description_free(fontDesc);
   }
 
@@ -1048,9 +1015,9 @@ configure_gui(WorldGUI *gui, World *world)
 void
 worldgui_configure_spell(WorldGUI *gui)
 {
+#ifdef HAVE_GTKSPELL
   GError *error = NULL;
 
-#ifdef HAVE_GTKSPELL
   simo_combo_box_set_spell(gui->cmbEntry,
                            gui->world->spell, gui->world->spell_language,
                            &error);
@@ -1076,7 +1043,8 @@ textview_button_press_cb(GtkWidget      *text_view,
 
   gui = (WorldGUI *) data;
 
-  if (event->button != 1 && event->button != 3) {
+  if (event->button != GDK_BUTTON_PRIMARY
+      && event->button != GDK_BUTTON_SECONDARY) {
     return FALSE;
   }
 
@@ -1086,13 +1054,14 @@ textview_button_press_cb(GtkWidget      *text_view,
 
   gtk_text_view_get_iter_at_location(GTK_TEXT_VIEW(text_view), &iter, x, y);
 
-  return act_if_link(gui, &iter, event);
+  return act_if_link(gui, &iter, text_view, event);
 }
 
 
 static
 gboolean
-act_if_link(WorldGUI *gui, GtkTextIter *iter, GdkEventButton *event)
+act_if_link(WorldGUI *gui, GtkTextIter *iter,
+            GtkWidget *widget, GdkEventButton *event)
 {
   GtkTextIter  start;
   GtkTextIter  end;
@@ -1113,7 +1082,7 @@ act_if_link(WorldGUI *gui, GtkTextIter *iter, GdkEventButton *event)
   }
   matched_url = gtk_text_buffer_get_text(gui->txtBuffer, &start, &end, FALSE);
 
-  if (event->button == 1) {
+  if (event->button == GDK_BUTTON_PRIMARY) {
     menu_url_open(NULL, matched_url);
   } else {
     GtkBuilder *ui_builder;
@@ -1130,8 +1099,10 @@ act_if_link(WorldGUI *gui, GtkTextIter *iter, GdkEventButton *event)
     }
 
     mnuPopupURL = GTK_MENU(gtk_builder_get_object(ui_builder, "mnuPopupURL"));
+    gtk_widget_show_all(GTK_WIDGET(mnuPopupURL));
     gtk_builder_connect_signals(ui_builder, matched_url);
 
+    gtk_menu_attach_to_widget(mnuPopupURL, widget, NULL);
     gtk_menu_popup(mnuPopupURL, NULL, NULL, NULL, NULL,
                    event->button, event->time);
     g_object_unref(ui_builder);
@@ -1143,7 +1114,7 @@ act_if_link(WorldGUI *gui, GtkTextIter *iter, GdkEventButton *event)
 
 #ifndef __WIN32__
 void
-menu_url_open(GtkAction *action, char *url)
+menu_url_open(GtkMenuItem *menu, char *url)
 {
   char *to_run;
 
@@ -1156,7 +1127,7 @@ menu_url_open(GtkAction *action, char *url)
 }
 #else /* defined __WIN32__ */
 void
-menu_url_open(GtkAction *action, char *url)
+menu_url_open(GtkMenuItem *menu, char *url)
 {
   ShellExecute(NULL, "open", url, NULL, NULL, SW_SHOWNORMAL);
 
@@ -1166,14 +1137,12 @@ menu_url_open(GtkAction *action, char *url)
 
 
 void
-menu_popup_url_copy(GtkAction *action, char *url)
+menu_popup_url_copy(GtkMenuItem *menu, char *url)
 {
-  GtkWidget    *widget;
   GdkDisplay   *display;
   GtkClipboard *clipboard;
 
-  widget = GTK_WIDGET(gtk_action_get_proxies(action)->data);
-  display = gtk_widget_get_display(GTK_WIDGET(widget));
+  display = gtk_widget_get_display(GTK_WIDGET(menu));
 
   clipboard = gtk_clipboard_get_for_display(display, GDK_SELECTION_PRIMARY);
   gtk_clipboard_set_text(clipboard, url, -1);
@@ -1204,37 +1173,6 @@ textview_motion_notify_event(GtkWidget      *widget,
 
   set_cursor_if_appropriate(text_view, gui, x, y);
 
-  /* I don't know why this call is here, but I copied this from the
-     gtk-demo, and it is necessary. */
-  gdk_window_get_pointer(gtk_widget_get_window(widget), NULL, NULL, NULL);
-
-  return FALSE;
-}
-
-
-/* Also update the cursor image if the window becomes visible
- * (e.g. when a window covering it got iconified).
- */
-static
-gboolean
-textview_visibility_notify_event(GtkWidget          *widget,
-                                 GdkEventVisibility *event,
-                                 gpointer            data)
-{
-  /* Also update the cursor image if the window becomes visible
-   * (e.g. when a window covering it got iconified). */
-  WorldGUI    *gui       = (WorldGUI *) data;
-  GtkTextView *text_view = GTK_TEXT_VIEW(widget);
-  gint      wx, wy, bx, by;
-
-  gdk_window_get_pointer(gtk_widget_get_window(widget), &wx, &wy, NULL);
-
-  gtk_text_view_window_to_buffer_coords(text_view,
-                                        GTK_TEXT_WINDOW_WIDGET,
-                                        wx, wy, &bx, &by);
-
-  set_cursor_if_appropriate(text_view, gui, bx, by);
-
   return FALSE;
 }
 
@@ -1308,17 +1246,17 @@ worldgui_query_tooltip_cb(GtkWidget  *widget,
                                         GTK_TEXT_WINDOW_WIDGET,
                                         0, tooltip_line_top_y,
                                         &rect.x, &rect.y);
-#if GTK_MINOR_VERSION < 18
-  alloc = widget->allocation;
-#else
   gtk_widget_get_allocation(widget, &alloc);
-#endif
   rect.width = alloc.width;
 
   gtk_tooltip_set_tip_area(tooltip, &rect);
 
   /* Get the tooltip text */
   text = worldgui_get_tooltip_text(gui, text_view, x, y);
+  if (text == NULL) {
+    return FALSE;
+  }
+
   gtk_tooltip_set_text(tooltip, text);
   g_free(text);
 
@@ -1347,6 +1285,10 @@ worldgui_get_tooltip_text(WorldGUI    *gui,
 #endif
     struct tm *time_tmp;
 
+    if (k_circular_queue_size(gui->line_times) == 0) {
+      return NULL;
+    }
+
     tip = g_malloc(MAX_BUFFER * sizeof(gchar));
 
     tooltip_line_number = gtk_text_iter_get_line(&iter);
@@ -1523,7 +1465,7 @@ search_box_keypress_cb(GtkWidget   *widget,
                        GdkEventKey *evt,
                        gpointer     data)
 {
-  if (evt->keyval == GDK_Escape) {
+  if (evt->keyval == GDK_KEY_Escape) {
     close_search_box_cb(NULL, data);
     return TRUE;
   }
