@@ -1,7 +1,5 @@
-/* $Id: worlds.c 1170 2011-05-21 21:23:27Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +24,7 @@
 #include <string.h>
 #include <errno.h>
 #include <stdarg.h>
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <gtk/gtk.h>
 
 #include "kildclient.h"
@@ -127,7 +125,7 @@ struct WorldParser_s
   };
   union {
     gchar      **currText;
-    GdkColor    *currColor;
+    GdkRGBA     *currColor;
     GtkTreeIter  curr_logon_iter;
   };
 };
@@ -160,37 +158,36 @@ static void xml_error_unknown_attribute(GError              **error,
                                         const gchar          *attribute_name,
                                         GMarkupParseContext  *context);
 static void copy_text(char **dest, const char *orig, gsize len);
-static void read_color(GdkColor *color,
-                       const char *text, gsize text_len,
+static void read_color(GdkRGBA *color, const char *text, gsize text_len,
                        GError **error);
-static void write_color(GString *str, char *name, int idx, GdkColor *color);
+static void write_color(GString *str, char *name, int idx, GdkRGBA *color);
 
 
 /********************
  * Global variables *
  ********************/
 World    *default_world = NULL;
-GdkColor deffore     = { 0, 0xbebe, 0xbebe, 0xbebe };
-GdkColor defbold     = { 0, 0xffff, 0xffff, 0xffff };
-GdkColor defback     = { 0, 0,      0,      0      };
-GdkColor defboldback = { 0, 0x5f5f, 0x5f5f, 0x5f5f };
-GdkColor defansi[] = {
-  { 0, 0,      0,      0      },        /* Black */
-  { 0, 0x8b8b, 0,      0      },        /* Red */
-  { 0, 0,      0x7373, 0      },        /* Green */
-  { 0, 0x8b8b, 0x6969, 0x1414 },        /* Yellow */
-  { 0, 0,      0,      0x8b8b },        /* Blue */
-  { 0, 0x8b8b, 0,      0x8b8b },        /* Magenta */
-  { 0, 0,      0x8b8b, 0x8b8b },        /* Cyan */
-  { 0, 0xbebe, 0xbebe, 0xbebe },        /* White */
-  { 0, 0x5f5f, 0x5f5f, 0x5f5f },        /* Bold Black */
-  { 0, 0xffff, 0,      0      },        /* Bold Red */
-  { 0, 0,      0xffff, 0      },        /* Bold Green */
-  { 0, 0xffff, 0xffff, 0      },        /* Bold Yellow */
-  { 0, 0,      0,      0xffff },        /* Bold Blue */
-  { 0, 0xffff, 0,      0xffff },        /* Bold Magenta */
-  { 0, 0,      0xffff, 0xffff },        /* Bold Cyan */
-  { 0, 0xffff, 0xffff, 0xffff },        /* Bold White */
+GdkRGBA deffore     = { 0.74509804, 0.74509804, 0.74509804, 1 };
+GdkRGBA defbold     = { 1,          1,          1,          1 };
+GdkRGBA defback     = { 0,          0,          0         , 1 };
+GdkRGBA defboldback = { 0.37254902, 0.37254902, 0.37254902, 1 };
+GdkRGBA defansi[] = {
+  { 0,          0,          0,          1 },        /* Black */
+  { 0.54509804, 0,          0,          1 },        /* Red */
+  { 0,          0.45098039, 0,          1 },        /* Green */
+  { 0.54509804, 0.41176471, 0.07843137, 1 },        /* Yellow */
+  { 0,          0,          0.54509804, 1 },        /* Blue */
+  { 0.54509804, 0,          0.54509804, 1 },        /* Magenta */
+  { 0,          0.54509804, 0.54509804, 1 },        /* Cyan */
+  { 0.74509804, 0.74509804, 0.74509804, 1 },        /* White */
+  { 0.37254902, 0.37254902, 0.37254902, 1 },        /* Bold Black */
+  { 1,          0,          0,          1 },        /* Bold Red */
+  { 0,          1,          0,          1 },        /* Bold Green */
+  { 1,          1,          0,          1 },        /* Bold Yellow */
+  { 0,          0,          1,          1 },        /* Bold Blue */
+  { 1,          0,          1,          1 },        /* Bold Magenta */
+  { 0,          1,          1,          1 },        /* Bold Cyan */
+  { 1,          1,          1,          1 },        /* Bold White */
 };
 
 
@@ -227,6 +224,8 @@ create_new_world(gboolean load_defaults)
 
   world->proxy.type = PROXY_USE_GLOBAL;
 
+  g_rec_mutex_init(&world->perl_mutex);
+
   if (!load_defaults) {
     return world;
   }
@@ -253,11 +252,11 @@ create_new_world(gboolean load_defaults)
   world->wrap_column     = default_world->wrap_column;
   world->use_tooltips    = default_world->use_tooltips;
 
-  memcpy(world->deffore,     default_world->deffore,     sizeof(GdkColor));
-  memcpy(world->defbold,     default_world->defbold,     sizeof(GdkColor));
-  memcpy(world->defback,     default_world->defback,     sizeof(GdkColor));
-  memcpy(world->defboldback, default_world->defboldback, sizeof(GdkColor));
-  memcpy(world->ansicolors,  default_world->ansicolors,  16*sizeof(GdkColor));
+  memcpy(world->deffore,     default_world->deffore,     sizeof(GdkRGBA));
+  memcpy(world->defbold,     default_world->defbold,     sizeof(GdkRGBA));
+  memcpy(world->defback,     default_world->defback,     sizeof(GdkRGBA));
+  memcpy(world->defboldback, default_world->defboldback, sizeof(GdkRGBA));
+  memcpy(world->ansicolors,  default_world->ansicolors,  16*sizeof(GdkRGBA));
   world->name_display_style   = default_world->name_display_style;
   world->name_display_pattern = g_strdup(default_world->name_display_pattern);
 
@@ -335,11 +334,11 @@ create_default_world(void)
   default_world->use_wrap_column = FALSE;
   default_world->use_tooltips    = TRUE;
 
-  memcpy(default_world->deffore,     &deffore,     sizeof(GdkColor));
-  memcpy(default_world->defbold,     &defbold,     sizeof(GdkColor));
-  memcpy(default_world->defback,     &defback,     sizeof(GdkColor));
-  memcpy(default_world->defboldback, &defboldback, sizeof(GdkColor));
-  memcpy(default_world->ansicolors,  &defansi,     16*sizeof(GdkColor));
+  memcpy(default_world->deffore,     &deffore,     sizeof(GdkRGBA));
+  memcpy(default_world->defbold,     &defbold,     sizeof(GdkRGBA));
+  memcpy(default_world->defback,     &defback,     sizeof(GdkRGBA));
+  memcpy(default_world->defboldback, &defboldback, sizeof(GdkRGBA));
+  memcpy(default_world->ansicolors,  &defansi,     16*sizeof(GdkRGBA));
   default_world->name_display_style   = NAME_DISPLAY_WORLD;
   default_world->name_display_pattern = g_strdup("%Kw [%Kc]");
 
@@ -388,11 +387,35 @@ static
 void
 world_allocate_space_for_colors(World *world)
 {
-  world->deffore     = g_new0(GdkColor, 1);
-  world->defbold     = g_new0(GdkColor, 1);
-  world->defback     = g_new0(GdkColor, 1);
-  world->defboldback = g_new0(GdkColor, 1);
-  world->ansicolors  = g_new0(GdkColor, 16);
+  world->deffore     = g_new0(GdkRGBA, 1);
+  world->defbold     = g_new0(GdkRGBA, 1);
+  world->defback     = g_new0(GdkRGBA, 1);
+  world->defboldback = g_new0(GdkRGBA, 1);
+  world->ansicolors  = g_new0(GdkRGBA, 16);
+}
+
+
+void
+init_threads(World *world)
+{
+  g_mutex_init(&world->write_log_line_mutex);
+
+  g_mutex_init(&world->wait_command_mutex);
+  g_cond_init(&world->wait_command_cond);
+
+  world->process_line_operations
+    = g_async_queue_new_full(free_process_line_operation);
+  world->process_line_thread = g_thread_new("process_line",
+                                             process_line_thread, world);
+
+  world->ansiparse_operations
+    = g_async_queue_new_full(free_ansiparse_operation);
+  world->ansiparse_thread = g_thread_new("ansiparse", ansiparse_thread, world);
+
+  world->mainthread_operations
+    = g_async_queue_new_full(free_mainthread_operation);
+  world->mainthread_operations_idle_id
+    = g_timeout_add(100, mainthread_operations_idle_cb, world);
 }
 
 
@@ -404,6 +427,47 @@ free_world(World *world)
     adjust_window_title();
   }
 
+  if (world->ansiparse_thread) {
+    AnsiParseOperation *ansi_op;
+    ProcessLineOperation *process_line_op;
+
+    g_source_remove(world->mainthread_operations_idle_id);
+    g_async_queue_unref(world->mainthread_operations);
+
+    g_async_queue_unref(world->ansiparse_operations);
+    g_async_queue_unref(world->process_line_operations);
+
+    world->to_be_destructed = TRUE;
+
+    /* We add an operation so that the threads wakes up (if the queue
+       is empty, it might never notice that it should be
+       destroyed). */
+    ansi_op = g_new(AnsiParseOperation, 1);
+    ansi_op->action = DUMMY_ANSIPARSE;
+    g_async_queue_push(world->ansiparse_operations, ansi_op);
+    /* Wait for the thread to exit. */
+    g_thread_join(world->ansiparse_thread);
+
+    /* The same, for process_line_thread */
+    process_line_op = g_new(ProcessLineOperation, 1);
+    process_line_op->action = DUMMY_PROCESS_LINE;
+    g_async_queue_push(world->process_line_operations, process_line_op);
+
+    /* Notify a possibly waiting match trigger thread */
+    g_mutex_lock(&world->wait_command_mutex);
+    world->command_executed = TRUE;
+    g_cond_signal(&world->wait_command_cond);
+    g_mutex_unlock(&world->wait_command_mutex);
+
+    /* Wait for the thread to exit */
+    g_thread_join(world->process_line_thread);
+
+    g_mutex_clear(&world->write_log_line_mutex);
+
+    g_cond_clear(&world->wait_command_cond);
+    g_mutex_clear(&world->wait_command_mutex);
+  }
+
   g_free(world->name);
   g_free(world->display_name);
   g_free(world->new_text_name);
@@ -414,6 +478,7 @@ free_world(World *world)
   g_free(world->host_ip);
 
   g_free(world->pbuffer);
+  g_free(world->server_data_buffer);
 
   if (world->character_used) {
     gtk_tree_row_reference_free(world->character_used);
@@ -437,7 +502,7 @@ free_world(World *world)
   g_free(world->scriptfile);
 
   if (world->perl_interpreter) {
-    PERL_SET_CONTEXT(world->perl_interpreter);
+    GRAB_PERL(world);
   }
 
   g_slist_foreach(world->plugins,
@@ -473,11 +538,17 @@ free_world(World *world)
   g_slist_free(world->hooks.OnLoseFocus);
   g_slist_foreach(world->hooks.OnCloseConnected, (GFunc) free_hook, NULL);
   g_slist_free(world->hooks.OnCloseConnected);
+  g_slist_foreach(world->hooks.OnServerData, (GFunc) free_hook, NULL);
+  g_slist_free(world->hooks.OnServerData);
 
   g_slist_foreach(world->permanent_variables,
                   (GFunc) free_permanent_variable, NULL);
   g_slist_free(world->permanent_variables);
 
+  if (world->perl_interpreter) {
+    RELEASE_PERL(world);
+  }
+
   g_free(world->flood_prevention_command);
   g_free(world->last_command);
   if (world->cmd_list) {
@@ -503,6 +574,7 @@ free_world(World *world)
     g_object_unref(world->hooks.OnGetFocus_model_filter);
     g_object_unref(world->hooks.OnLoseFocus_model_filter);
     g_object_unref(world->hooks.OnCloseConnected_model_filter);
+    g_object_unref(world->hooks.OnServerData_model_filter);
 
     g_slist_foreach(world->selected_OnConnect_hooks,
                     (GFunc) gtk_tree_row_reference_free, NULL);
@@ -525,6 +597,9 @@ free_world(World *world)
     g_slist_foreach(world->selected_OnCloseConnected_hooks,
                     (GFunc) gtk_tree_row_reference_free, NULL);
     g_slist_free(world->selected_OnCloseConnected_hooks);
+    g_slist_foreach(world->selected_OnServerData_hooks,
+                    (GFunc) gtk_tree_row_reference_free, NULL);
+    g_slist_free(world->selected_OnServerData_hooks);
   }
 
   if (world->dlgMLSend) {
@@ -556,6 +631,7 @@ free_world(World *world)
     g_timer_destroy(world->last_data_written);
   }
 
+  g_rec_mutex_clear(&world->perl_mutex);
 
   g_free(world);
 }
@@ -676,6 +752,16 @@ remove_plugin_objects(World *world, Plugin *plugin)
       delete_hook(world, "OnCloseConnected", 0, iter);
     }
   }
+
+  for (iter = world->hooks.OnServerData; iter; iter = next) {
+    /* Must do this here, since the list item may be deleted. */
+    next = iter->next;
+
+    Hook *hook = (Hook *) iter->data;
+    if (hook->owner_plugin == plugin) {
+      delete_hook(world, "OnServerData", 0, iter);
+    }
+  }
 }
 
 
@@ -1392,7 +1478,7 @@ xml_start_element(GMarkupParseContext *context,
         if (strcmp(*attribute_names, "name") == 0) {
           parser->currTimer->name = g_strdup(*attribute_values);
         } else if (strcmp(*attribute_names, "interval") == 0) {
-          parser->currTimer->interval = atoi(*attribute_values);
+          parser->currTimer->interval = strtod(*attribute_values, NULL);
         } else if (strcmp(*attribute_names, "count") == 0) {
           parser->currTimer->count = atoi(*attribute_values);
         } else if (strcmp(*attribute_names, "enabled") == 0) {
@@ -2394,24 +2480,24 @@ kc_g_string_append_escaped(GString *gstr, const char *template, ...)
 
 static
 void
-read_color(GdkColor *color, const char *text, gsize text_len, GError **error)
+read_color(GdkRGBA *color, const char *text, gsize text_len, GError **error)
 {
   gchar    *colordesc;
   int       idx;
-  GdkColor  temp;
+  guint16   red, green, blue;
 
   copy_text(&colordesc, text, text_len);
 
-  if (sscanf(colordesc, "%d %hu %hu %hu", &idx,
-             &temp.red, &temp.green, &temp.blue) != 4) {
+  if (sscanf(colordesc, "%d %hu %hu %hu", &idx, &red, &green, &blue) != 4) {
     g_set_error(error,
                 G_MARKUP_ERROR, G_MARKUP_ERROR_PARSE,
                 "Invalid color specification in config file.");
   }
 
-  color[idx].red   = temp.red;
-  color[idx].green = temp.green;
-  color[idx].blue  = temp.blue;
+  color[idx].alpha = 1.0;
+  color[idx].red   = red   / 65535.0;
+  color[idx].green = green / 65535.0;
+  color[idx].blue  = blue  / 65535.0;
 
   g_free(colordesc);
 }
@@ -2419,10 +2505,12 @@ read_color(GdkColor *color, const char *text, gsize text_len, GError **error)
 
 static
 void
-write_color(GString *str, char *name, int idx, GdkColor *color)
+write_color(GString *str, char *name, int idx, GdkRGBA *color)
 {
   g_string_append_printf(str, "    <%s>%d %d %d %d</%s>\n", name, idx,
-                         color[idx].red, color[idx].green, color[idx].blue,
+                         (int) (color[idx].red   * 65535),
+                         (int) (color[idx].green * 65535),
+                         (int) (color[idx].blue  * 65535),
                          name);
 }
 
