@@ -1,7 +1,5 @@
-/* $Id: worldselector.c 1143 2011-02-13 16:13:58Z ekalin $ */
-
 /*
- * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2014 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,7 +28,7 @@
 #else
 #  include <windows.h>
 #endif
-#include <libintl.h>
+#include "libintl-wrapper.h"
 #include <gtk/gtk.h>
 
 #include "kildclient.h"
@@ -517,8 +515,9 @@ get_world_from_cmdline(const gchar *cmdline)
         G_REGEX_NO_AUTO_CAPTURE,
         0,
         &error);
-      if (error)
-        fprintf(stderr, error->message);
+      if (error) {
+        fputs(error->message, stderr);
+      }
     }
 
     if (g_regex_match(parameters_regex,
@@ -807,20 +806,20 @@ disconnected_msgbox(gchar *msg, gboolean can_offline)
                                | GTK_DIALOG_DESTROY_WITH_PARENT,
                                GTK_MESSAGE_QUESTION,
                                GTK_BUTTONS_NONE,
-                               msg);
+                               "%s", msg);
   gtk_window_set_title(GTK_WINDOW(dlg), _("Disconnected"));
   if (can_offline) {
     gtk_dialog_add_buttons(GTK_DIALOG(dlg),
                            _("Reconnect"), RESP_RECONNECT,
                            _("Connect to another world"), RESP_OTHER,
                            _("Offline"), RESP_OFFLINE,
-                           GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+                           _("_Close"),GTK_RESPONSE_CLOSE,
                            NULL);
   } else {
     gtk_dialog_add_buttons(GTK_DIALOG(dlg),
                            _("Reconnect"), RESP_RECONNECT,
                            _("Connect to another world"), RESP_OTHER,
-                           GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+                           _("_Close"), GTK_RESPONSE_CLOSE,
                            NULL);
   }
 
